@@ -21,7 +21,8 @@ class GameMap;
 class AIClasses {
 
 public:
-	AIClasses() { unitIDs.resize(MAX_UNITS); }
+	AIClasses();
+	bool isMaster();
 
 	IAICallback    *cb;
 	IAICheats      *cbc;
@@ -38,7 +39,15 @@ public:
 	CDefenseMatrix *defensematrix;
 	CLogger        *logger;
 	int            team;
-	std::vector<int> unitIDs; // temporary container for GetEnemyUnits(), GetFriendlyUnits() etc. results
+		// team ID (will be AI ID in future?)
+	int            allyTeam;
+		// ally team ID
+	int            allyAITeam;
+		// ally AI team ID (internal)
+	difficultyLevel difficulty;
+						
+	std::vector<int> unitIDs;
+		// temporary container for GetEnemyUnits(), GetFriendlyUnits() etc. results
 };
 
 #endif
