@@ -11,10 +11,35 @@
 class AIClasses;
 class UnitType;
 
+struct Wish {
+	unsigned int goalCats;
+	buildPriority p;
+	UnitType *ut;
+
+	Wish() {
+		goalCats = 0;
+		p = NORMAL;
+		ut = NULL;
+	}
+	Wish(UnitType *ut, buildPriority p, unsigned int gcats) {
+		this->ut = ut;
+		this->p  = p;
+		this->goalCats = gcats;
+	}
+
+	bool operator< (const Wish &w) const {
+		return p > w.p;
+	}
+
+	bool operator== (const Wish &w) const {
+		return p == w.p;
+	}
+};
+
 class CWishList {
 	public:
 		CWishList(AIClasses *ai);
-		~CWishList() {}
+		~CWishList();
 
 		/* Insert a unit in the wishlist, sorted by priority p */
 		void push(unsigned int categories, buildPriority p);
@@ -26,32 +51,16 @@ class CWishList {
 		bool empty(int factory);
 
 		/* View the top unit from the wishlist */
-		UnitType* top(int factory);
+		Wish top(int factory);
 
 	private:
-		struct Wish {
-			buildPriority p;
-			UnitType *ut;
+		AIClasses *ai;
 
-			Wish(UnitType *ut, buildPriority p) {
-				this->ut = ut;
-				this->p  = p;
-			}
-
-			bool operator< (const Wish &w) const {
-				return p > w.p;
-			}
-
-			bool operator== (const Wish &w) const {
-				return p == w.p;
-			}
-		};
+		int maxWishlistSize;
 
 		void unique(std::vector<Wish> &vector);
 
-		std::map<int, std::vector<Wish> > wishlist;  /* <factory type, wishes> */
-
-		AIClasses *ai;
+		std::map<int, std::vector<Wish> > wishlist; /* <factory type, wishes> */
 };
 
 #endif
