@@ -1,19 +1,24 @@
 #ifndef UTIL_H
 #define UTIL_H
 
+#include <algorithm>
 #include <list>
 #include <set>
 #include <string>
+#include <vector>
+#include <map>
 
 class IAICallback;
 
 namespace util {
 	std::string GetAbsFileName(IAICallback*, const std::string&, bool readonly = false);
+	void SanitizeFileNameInPlace(std::string&);
 
 	void StringToLowerInPlace(std::string&);
 	std::string StringToLower(std::string);
 	std::string StringStripSpaces(const std::string&);
 	int RemoveWhiteSpaceInPlace(std::string&);
+	std::string IntToString(int i, const std::string& format = "%i");
 
 	float WeightedAverage(std::list<float>&, std::list<float>&);
 
@@ -30,8 +35,9 @@ namespace util {
 	 */
 	bool IsBinarySubset(unsigned A, unsigned B);
 
-	template<typename T> std::set<T> IntersectSets(const std::set<T>& s1, const std::set<T>& s2) {
-		typename std::set<T> r;
+	template<class T>
+	std::set<T> IntersectSets(const std::set<T>& s1, const std::set<T>& s2) {
+		std::set<T> r;
 		typename std::set<T>::const_iterator sit;
 
 		for (sit = s1.begin(); sit != s1.end(); sit++) {
@@ -43,16 +49,24 @@ namespace util {
 		return r;
 	}
 
-	template<typename T> std::set<T> UnionSets(const std::set<T>& s1, const std::set<T>& s2) {
-		typename std::set<T> r;
-		typename std::set<T>::const_iterator sit;
-
-		for (sit = s1.begin(); sit != s1.end(); sit++) { r.insert(*sit); }
-		for (sit = s2.begin(); sit != s2.end(); sit++) { r.insert(*sit); }
+	template<class T> 
+	std::set<T> UnionSets(const std::set<T>& s1, const std::set<T>& s2) {
+		std::set<T> r;
+		
+		r.insert(s1.begin(), s1.end());
+		r.insert(s2.begin(), s2.end());
 
 		return r;
 	}
 
+	template <class Key, class T>
+	void GetShuffledKeys(std::vector<Key> &out, std::map<Key, T> &in) {
+		typename std::map<Key, T>::iterator it;
+		for (it = in.begin(); it != in.end(); it++) {
+			out.push_back(it->first);
+		}
+		std::random_shuffle(out.begin(), out.end());
+	}
 }
 
 #endif
