@@ -43,13 +43,13 @@ void CBuildUp::Update(int frame) {
 		// multiplier <m> must not be a constant
 		// (more or less assumes starting storage
 		// capacity of 1000)
-		float m = 900.0f / (ai->cb->GetMetalStorage());
-		bool b1 = (ai->cb->GetMetal()) > (ai->cb->GetMetalStorage() * m);
-		bool b2 = (ai->cb->GetEnergyIncome()) > (ai->cb->GetEnergyUsage() * 1.3f);
-		bool b3 = (ai->cb->GetMetalIncome()) > (ai->cb->GetMetalUsage() * 1.3f);
+		const float m = 900.0f / (ai->cb->GetMetalStorage());
+		const bool b1 = (ai->cb->GetMetal()) > (ai->cb->GetMetalStorage() * m);
+		const bool b2 = (ai->cb->GetEnergyIncome()) > (ai->cb->GetEnergyUsage() * 1.3f);
+		const bool b3 = (ai->cb->GetMetalIncome()) > (ai->cb->GetMetalUsage() * 1.3f);
 
 		if ((b1 && b2 && b3) && builderTimer > 0 && !(rand() % 3) && frame > 3600) {
-			// decrease builderTime iif we have more metal
+			// decrease builderTime if we have more metal
 			// than 90% of our metal storage capacity and
 			// we are generating more than 130% the amount
 			// of M and E used (meaning we have excess M
@@ -235,7 +235,7 @@ void CBuildUp::Buildup(int frame) {
 					// FIXME: not happening often enough during res. stalls
 					//
 					const bool reclaimFeature =
-						((frame & 1) && ai->MyUnits[econState.builderID]->ReclaimBestFeature(true, 4096));
+						((frame & 1) && ai->GetUnit(econState.builderID)->ReclaimBestFeature(true, 4096));
 
 					if (!reclaimFeature) {
 						const bool haveNewMex = BuildUpgradeExtractor(econState.builderID);
@@ -289,8 +289,8 @@ void CBuildUp::Buildup(int frame) {
 								bool r = false;
 
 								if (building) {
-									float3 buildPos = ai->dm->GetDefensePos(building, ai->MyUnits[econState.builderID]->pos());
-									r = ai->MyUnits[econState.builderID]->Build_ClosestSite(building, buildPos, 2);
+									float3 buildPos = ai->dm->GetDefensePos(building, ai->GetUnit(econState.builderID)->pos());
+									r = ai->GetUnit(econState.builderID)->Build_ClosestSite(building, buildPos, 2);
 								} else {
 									FallbackBuild(econState.builderID, CAT_DEFENCE);
 								}
@@ -344,7 +344,7 @@ void CBuildUp::FactoryCycle(int frame) {
 		// pick the i-th idle factory we have
 		UnitCategory producedCat = CAT_LAST;
 		const int factoryUnitID  = ai->uh->GetIU(CAT_FACTORY);
-		const CUNIT* u           = ai->MyUnits[factoryUnitID];
+		const CUNIT* u           = ai->GetUnit(factoryUnitID);
 		const bool isHub         = u->isHub();
 		const UnitDef* factDef   = u->def();
 
@@ -434,7 +434,7 @@ void CBuildUp::NukeSiloCycle(void) {
 
 		// always keep at least 5 nukes in queue for a rainy day
 		if (silo->numNukesQueued < 5)
-			ai->MyUnits[silo->id]->NukeSiloBuild();
+			ai->GetUnit(silo->id)->NukeSiloBuild();
 	}
 }
 
@@ -466,22 +466,22 @@ void CBuildUp::FallbackBuild(int builderID, int failedCat) {
 		const UnitDef* udef4 = ai->ut->GetUnitByScore(builderID, CAT_FACTORY);
 
 		if (udef2 && failedCat != CAT_ENERGY) {
-			ai->MyUnits[builderID]->Build_ClosestSite(udef2, builderPos);
+			ai->GetUnit(builderID)->Build_ClosestSite(udef2, builderPos);
 			return;
 		}
 		if (udef3 && failedCat != CAT_DEFENCE) {
 			float3 pos = ai->dm->GetDefensePos(udef3, builderPos);
-			ai->MyUnits[builderID]->Build_ClosestSite(udef3, pos);
+			ai->GetUnit(builderID)->Build_ClosestSite(udef3, pos);
 			return;
 		}
 		if (udef4 && failedCat != CAT_FACTORY) {
-			ai->MyUnits[builderID]->Build_ClosestSite(udef4, builderPos);
+			ai->GetUnit(builderID)->Build_ClosestSite(udef4, builderPos);
 			return;
 		}
 		if (udef1 && failedCat != CAT_MEX) {
 			float3 pos = ai->mm->GetNearestMetalSpot(builderID, udef1);
 			if (pos != ERRORVECTOR)
-				ai->MyUnits[builderID]->Build(pos, udef1, -1);
+				ai->GetUnit(builderID)->Build(pos, udef1, -1);
 			return;
 		}
 	}
@@ -489,7 +489,7 @@ void CBuildUp::FallbackBuild(int builderID, int failedCat) {
 
 	// unable to assist and unable to build, just patrol
 	if (!b1 && !b2 && !b3 && !b4) {
-		ai->MyUnits[builderID]->Patrol(builderPos);
+		ai->GetUnit(builderID)->Patrol(builderPos);
 	}
 }
 
@@ -539,7 +539,7 @@ bool CBuildUp::BuildNow(int builderID, UnitCategory cat, const UnitDef* udef) {
 		const bool b1 = ((ai->uh->AllUnitsByType[udef->id]).size() < 1);
 
 		if (!b0 || b1) {
-			r = ai->MyUnits[builderID]->Build_ClosestSite(udef, ai->cb->GetUnitPos(builderID));
+			r = ai->GetUnit(builderID)->Build_ClosestSite(udef, ai->cb->GetUnitPos(builderID));
 		}
 	} else {
 		FallbackBuild(builderID, cat);
@@ -554,14 +554,15 @@ bool CBuildUp::BuildUpgradeExtractor(int builderID) {
 	const UnitDef* newMexDef = ai->ut->GetUnitByScore(builderID, CAT_MEX);
 
 	if (newMexDef != NULL) {
-		const float3 builderPos = ai->MyUnits[builderID]->pos();
-		const float3 newMexPos  = ai->mm->GetNearestMetalSpot(builderID, newMexDef);
+		const CUNIT*  builder    = ai->GetUnit(builderID);
+		const float3& builderPos = builder->pos();
+		const float3  newMexPos  = ai->mm->GetNearestMetalSpot(builderID, newMexDef);
 		// const float  oldMexDist = newMexPos.distance2D(builderPos);
 
 		if (newMexPos != ERRORVECTOR) {
 			if (!ai->uh->BuildTaskAddBuilder(builderID, CAT_MEX)) {
 				// build a new metal extractor
-				return (ai->MyUnits[builderID]->Build(newMexPos, newMexDef, -1));
+				return (builder->Build(newMexPos, newMexDef, -1));
 			}
 		} else {
 			// upgrade an existing extractor (NOTE: GetNearestMetalSpot()
@@ -606,7 +607,7 @@ bool CBuildUp::BuildUpgradeReactor(int builderID) {
 			float        closestDstSq = MY_FLT_MAX;
 
 			int             oldReactorID  = -1;
-			float3          oldReactorPos = ZEROVECTOR;
+			float3          oldReactorPos = ZeroVector;
 			const  UnitDef* oldReactorDef = NULL;
 
 			std::list<int> lst = ai->uh->AllUnitsByCat[CAT_ENERGY];
