@@ -60,7 +60,7 @@ void CDGunController::Init(int unitID) {
 
 	// set the commander to hold fire (we need this since
 	// FAW and RF interfere with dgun and reclaim orders)
-	ai->MyUnits[commanderID]->SetFireState(0);
+	ai->GetUnit(commanderID)->SetFireState(0);
 
 	std::vector<UnitDef::UnitDefWeapon>::const_iterator i = commanderUD->weapons.begin();
 
@@ -119,51 +119,45 @@ void CDGunController::TrackAttackTarget(unsigned int currentFrame) {
 		const float  maxRange     = ai->cb->GetUnitMaxRange(commanderID);
 
 		bool haveClearShot = true;
-		int orderType = -1;
+		int numSegments = 0;
 
-		AIHCTraceRay rayData = {
-			commanderPos,
-			targetDif / targetDist, // direction
-			maxRange,
-			commanderID,
-			-1,
-			0
-		};
 
-		ai->cb->HandleCommand(AIHCTraceRayId, &rayData);
+		AIHCTraceRay rayData;
+		float  rayLen = maxRange * 1.1f;
+		float3 rayPos = commanderPos;
+		float3 rayDir = targetDif / targetDist;
+			rayDir.y = 0.0f;
 
-		if (rayData.hitUID != -1) {
-			// note: still fails when allied structure is between us and enemy
-			// can also fail if enemy is in front of allied structure and both
-			// are within the d-gun's range
-			haveClearShot = (ai->cb->GetUnitAllyTeam(rayData.hitUID) != ai->cb->GetMyAllyTeam());
+		while (haveClearShot && (rayLen >= 1.0f)) {
+			rayData.rayPos = rayPos;
+			rayData.rayDir = targetDif / targetDist;
+			rayData.rayLen = rayLen;
+			rayData.srcUID = commanderID;
+			rayData.hitUID = -1;
+			rayData.flags  = 0;
 
-			// TODO: get DGun weapon properties & check if it can pass through
-			// a unit, if yes then allow executing the code below
-			if(haveClearShot) {
-				// check if there is a unit next to hit unit on DGun path...
-				const float3 enemyPos = ai->cb->GetUnitPos(rayData.hitUID);
-				const float segmentLeft = maxRange - commanderPos.distance(enemyPos);
+			// modifies hitUID and rayLen
+			ai->ccb->HandleCommand(AIHCTraceRayId, &rayData);
 
-				if(segmentLeft > 0.0) {
-					AIHCTraceRay rayData2 = {
-						enemyPos,
-						targetDif / targetDist,
-						segmentLeft,
-						rayData.hitUID,
-						-1,
-						0
-					};
-
-					ai->cb->HandleCommand(AIHCTraceRayId, &rayData2);
-
-					if(rayData2.hitUID != -1) {
-						haveClearShot = (ai->cb->GetUnitAllyTeam(rayData2.hitUID) != ai->cb->GetMyAllyTeam());
-					}
+			// note: even if hitUnitID is -1, we can still end up destroying
+			// our own units because dgun projectiles keep hugging the ground
+			// rather than being absorbed by it on impact (which is assumed by
+			// the trace)
+			if (rayData.hitUID != -1) {
+				if (ai->cb->GetUnitAllyTeam(rayData.hitUID) == ai->cb->GetMyAllyTeam()) {
+					// hitUID is an allied unit
+					haveClearShot = false;
+				} else {
+					// hitUID is an enemy unit
+					rayPos = ai->ccb->GetUnitPos(rayData.hitUID);
 				}
 			}
+
+			// make sure we always terminate
+			rayLen -= rayData.rayLen;
 		}
 
+
 		// multiply by 0.9 to ensure commander does not have to walk
 		if ((commanderPos - dgunPos).Length() < maxRange * 0.9f) {
 			bool canDGun =
@@ -171,24 +165,21 @@ void CDGunController::TrackAttackTarget(unsigned int currentFrame) {
 				&& haveClearShot
 				&& (udef != NULL && !udef->weapons.empty());
 
-			if(canDGun) {
-				IssueOrder(dgunPos, orderType = CMD_DGUN, 0);
+			if (canDGun) {
+				IssueOrder(dgunPos, CMD_DGUN, 0);
 			} else {
 				bool bDanger = ai->tm->ThreatAtThisPoint(commanderPos/*curTargetPos*/) > ai->tm->GetAverageThreat();
-				if(bDanger) {
+
+				if (bDanger) {
 					state.Reset(currentFrame, true);
 				} else {
 					if (ai->cb->GetUnitHealth(state.targetID) < ai->cb->GetUnitMaxHealth(state.targetID) * 0.5f) {
-						IssueOrder(state.targetID, orderType = CMD_RECLAIM, 0);
+						IssueOrder(state.targetID, CMD_RECLAIM, 0);
 					} else {
-						IssueOrder(state.targetID, orderType = CMD_CAPTURE, 0);
+						IssueOrder(state.targetID, CMD_CAPTURE, 0);
 					}
 				}
 			}
-
-			if (orderType == CMD_DGUN   ) { state.dgunOrderFrame    = ai->cb->GetCurrentFrame(); }
-			if (orderType == CMD_RECLAIM) { state.reclaimOrderFrame = ai->cb->GetCurrentFrame(); }
-			if (orderType == CMD_CAPTURE) { state.captureOrderFrame = ai->cb->GetCurrentFrame(); }
 		} else {
 			state.Reset(currentFrame, true);
 		}
@@ -200,14 +191,14 @@ void CDGunController::TrackAttackTarget(unsigned int currentFrame) {
 void CDGunController::SelectTarget(unsigned int currentFrame) {
 	float3 commanderPos = ai->cb->GetUnitPos(commanderID);
 
-	// if our commander is dead then position will be (0, 0, 0)
+	// if our commander is dead, its position will be (0, 0, 0)
 	if (commanderPos.x <= 0.0f && commanderPos.z <= 0.0f) {
 		return;
 	}
 
 	// get all units within immediate (non-walking) dgun range
-	float maxRange = ai->cb->GetUnitMaxRange(commanderID);
-	int numUnits = ai->cb->GetEnemyUnits(&ai->unitIDs[0], commanderPos, maxRange * 0.9f);
+	const float maxRange = ai->cb->GetUnitMaxRange(commanderID);
+	const int numUnits = ai->cb->GetEnemyUnits(&ai->unitIDs[0], commanderPos, maxRange * 0.9f);
 
 	for (int i = 0; i < numUnits; i++) {
 		// if enemy unit with valid ID found in array
@@ -248,24 +239,32 @@ void CDGunController::Stop(void) const {
 	Command c; c.id = CMD_STOP; ai->ct->GiveOrder(commanderID, &c);
 }
 
-void CDGunController::IssueOrder(const float3& pos, int orderType, int keyMod) const {
+void CDGunController::IssueOrder(const float3& pos, int orderType, int keyMod) {
 	Command c;
-	c.id = orderType;
-	c.options |= keyMod;
-	c.params.push_back(pos.x);
-	c.params.push_back(pos.y);
-	c.params.push_back(pos.z);
+		c.id = orderType;
+		c.options |= keyMod;
+		c.params.push_back(pos.x);
+		c.params.push_back(pos.y);
+		c.params.push_back(pos.z);
 
 	ai->ct->GiveOrder(commanderID, &c);
+
+	if (orderType == CMD_DGUN   ) { state.dgunOrderFrame    = ai->cb->GetCurrentFrame(); }
+	if (orderType == CMD_RECLAIM) { state.reclaimOrderFrame = ai->cb->GetCurrentFrame(); }
+	if (orderType == CMD_CAPTURE) { state.captureOrderFrame = ai->cb->GetCurrentFrame(); }
 }
 
-void CDGunController::IssueOrder(int target, int orderType, int keyMod) const {
+void CDGunController::IssueOrder(int target, int orderType, int keyMod) {
 	Command c;
-	c.id = orderType;
-	c.options |= keyMod;
-	c.params.push_back(target);
+		c.id = orderType;
+		c.options |= keyMod;
+		c.params.push_back(target);
 
 	ai->ct->GiveOrder(commanderID, &c);
+
+	if (orderType == CMD_DGUN   ) { state.dgunOrderFrame    = ai->cb->GetCurrentFrame(); }
+	if (orderType == CMD_RECLAIM) { state.reclaimOrderFrame = ai->cb->GetCurrentFrame(); }
+	if (orderType == CMD_CAPTURE) { state.captureOrderFrame = ai->cb->GetCurrentFrame(); }
 }
 
 bool CDGunController::IsBusy(void) const {
