@@ -5,6 +5,7 @@
 #include "Defines.h"
 #include "Util.h"
 #include "IncExternAI.h"
+#include "System/Util.h"
 
 std::string CLogger::GetLogName() const {
 	if (name.size() > 0) {
@@ -17,9 +18,11 @@ std::string CLogger::GetLogName() const {
 
 	std::stringstream ss;
 		ss << std::string(LOGFOLDER);
-		ss << icb->GetMapName();
+		ss << AIUtil::MakeFileSystemCompatible(icb->GetMapName());
+		ss << "-" << IntToString(icb->GetMapHash(), "%x");
 		ss << "_";
-		ss << icb->GetModName();
+		ss << AIUtil::MakeFileSystemCompatible(icb->GetModHumanName());
+		ss << "-" << IntToString(icb->GetModHash(), "%x");
 		ss << "_";
 		ss << now2->tm_mon + 1;
 		ss << "-";
