@@ -57,121 +57,140 @@ void CUnitTable::PostLoad() {
 void CUnitTable::ReadModConfig() {
 	L(ai, "[CUnitTable::ReadModConfig()]");
 
-	std::stringstream msg;
 	std::string cfgFileName = GetModCfgName();
+	std::fstream cfgFile;
+	std::stringstream msg;
 
-	FILE* f = fopen(cfgFileName.c_str(), "r");
+	int cfgVersion = 0;
 
-	if (f != NULL) {
-		msg << "\tparsing existing mod configuration file ";
-		msg << cfgFileName;
-		L(ai, msg.str());
+	if (ai->cb->GetFileSize(cfgFileName.c_str()) != -1) {
+		if (!ai->luaParser->Execute(cfgFileName, "config")) {
+			msg << "\tparse-error in existing mod configuration file \"";
+			msg << cfgFileName << "\": " << ai->luaParser->GetError();
 
-		// read the mod's .cfg file
-		char str[1024] = {0};
+			L(ai, msg.str());
+			return;
+		} else {
+			msg << "\tparsed existing mod configuration file \"";
+			msg << cfgFileName << "\"";
 
-		char         unitName[512] = {0};
-		float        unitCostMult  =  1.0f;
-		int          unitTechLvl   = -1;
-		UnitCategory defUnitCat    = CAT_LAST;
-		UnitCategory cfgUnitCat    = CAT_LAST;
-
-		while (fgets(str, 1024, f) != NULL) {
-			if (str[0] == '/' && str[1] == '/') {
-				continue;
-			}
-
-			const int i = sscanf(str, "%s %f %d %d", unitName, &unitCostMult, &unitTechLvl, (int*) &cfgUnitCat);
-			const UnitDef* udef = ai->cb->GetUnitDef(unitName);
-
-			if ((i == 4) && udef != NULL) {
-				UnitType* utype = &unitTypes[udef->id];
-
-				utype->costMultiplier = unitCostMult;
-				utype->techLevel      = unitTechLvl;
-
-				defUnitCat = utype->category;
-
-				msg.str("");
-				msg << "\t\tudef->id: " << udef->id << ", udef->name: " << udef->name;
-				msg << ", default cat.: " << defUnitCat << ", .cfg cat.: " << cfgUnitCat;
-				L(ai, msg.str());
-
-				// TODO: look for any possible side-effects that might arise
-				// from overriding categories like this, then enable overrides
-				// other than builder --> attacker?
-				//
-				// FIXME: SEGV when unarmed CAT_BUILDER units masquerading as
-				// CAT_G_ATTACK'ers want to or are attacked (NULL weapondefs)
-				//
-				if (cfgUnitCat >= 0 && cfgUnitCat < CAT_LAST) {
-					if (cfgUnitCat == CAT_G_ATTACK && defUnitCat == CAT_BUILDER) {
-						msg.str("");
-						msg << "\t\t\t.cfg unit category (CAT_G_ATTACK) overrides utype->category (CAT_BUILDER)";
-						L(ai, msg.str());
-
-						std::vector<int>::iterator vit;
-
-						std::vector<int>& oldDefs = categoryData.GetDefsForUnitCat(defUnitCat);
-						std::vector<int>& newDefs = categoryData.GetDefsForUnitCat(cfgUnitCat);
-
-						for (vit = oldDefs.begin(); vit != oldDefs.end(); vit++) {
-							const int udefID = *vit;
-
-							if (udefID == udef->id) {
-								oldDefs.erase(vit);
-								newDefs.push_back(udef->id);
-								vit--;
-							}
-						}
-
-						utype->category = cfgUnitCat;
-					}
-				}
-			}
-		}
-
-		msg.str("");
-		msg << "read mod configuration file ";
-		msg << cfgFileName;
-		L(ai, msg.str());
-	} else {
-		msg.str("");
-		msg << "\tcreating new mod configuration file ";
-		msg << cfgFileName;
-		L(ai, msg.str());
-
-		// write a new .cfg file with default values
-		f = fopen(cfgFileName.c_str(), "w");
-		fprintf(f, "// unitName costMultiplier techLevel category\n");
-
-		for (int i = 1; i <= numDefs; i++) {
-			UnitType* utype = &unitTypes[i];
-			// assign and write default values for costMultiplier
-			// and techLevel, category is already set in ::Init()
-			utype->costMultiplier =  1.0f;
-			utype->techLevel      = -1;
-
-			fprintf(
-				f,
-				"%s %.2f %d %d\n",
-				utype->def->name.c_str(), utype->costMultiplier,
-				utype->techLevel, utype->category
-			);
-
-			msg.str("");
-			msg << "\t\tname: " << (utype->def->name);
-			msg << ", .cfg category: " << (utype->category);
 			L(ai, msg.str());
 		}
 
-		msg.str("");
-		msg << "wrote mod configuration file ";
-		msg << cfgFileName;
-		L(ai, msg.str());
-	}
+		const LuaTable* rootTbl = ai->luaParser->GetRootTbl();
+		const LuaTable* unitTbl = NULL;
+		const UnitDef*  unitDef = NULL;
 
-	fclose(f);
+		if (rootTbl->GetIntVal("version", cfgVersion) > CFGVERSION) {
+			msg.str("");
+			msg << "\tconfig-file version (" << cfgVersion << ") is newer than current version (" << CFGVERSION << ")";
+			return;
+		}
+
+		UnitType*    unitType   = NULL;
+		UnitCategory defUnitCat = CAT_LAST;
+		UnitCategory cfgUnitCat = CAT_LAST;
+
+		std::list<std::string> keys;
+		rootTbl->GetStrTblKeys(&keys);
+
+		for (std::list<std::string>::const_iterator it = keys.begin(); it != keys.end(); ++it) {
+			unitDef = ai->cb->GetUnitDef((*it).c_str());
+
+			if (unitDef == NULL) {
+				msg.str("");
+				msg << "\t\t.cfg entry \"" << (*it) << "\" does not refer to a valid unit-type";
+
+				L(ai, msg.str());
+				continue;
+			}
+
+			unitTbl = rootTbl->GetTblVal(*it);
+			unitType = &unitTypes[unitDef->id];
+
+			unitType->costMultiplier = unitTbl->GetIntVal("costMult", 100) / 100.0f;
+			unitType->techLevel      = unitTbl->GetIntVal("techLevel", -1);
+
+			defUnitCat = unitType->category;
+			cfgUnitCat = UnitCategory(unitTbl->GetIntVal("category", CAT_LAST));
+
+			{
+				msg.str("");
+				msg << "\t\tunitDef->id: " << unitDef->id << ", unitDef->name: " << unitDef->name;
+				msg << ", default cat.: " << defUnitCat << ", .cfg cat.: " << cfgUnitCat;
+
+				L(ai, msg.str());
+			}
+
+			/*
+			 * TODO: look for any possible "side-effects" that might arise
+			 * from overriding categories like this, then enable overrides
+			 * other than builder --> attacker (ie. SEGV when an *unarmed*
+			 * CAT_BUILDER unit masquerading as a CAT_G_ATTACK'er wants to
+			 * or is attacked, due to NULL weapondefs)
+			 */
+			if (defUnitCat != cfgUnitCat) {
+				if (cfgUnitCat < 0 || cfgUnitCat >= CAT_LAST) {
+					// invalid unit-category number
+					continue;
+				}
+
+				if (cfgUnitCat == CAT_G_ATTACK && defUnitCat == CAT_BUILDER) {
+					{
+						msg.str("");
+						msg << "\t\t\t.cfg unit category (CAT_G_ATTACK) overrides unitType->category (CAT_BUILDER)";
+						L(ai, msg.str());
+					}
+
+					std::vector<int>::iterator vit;
+					std::vector<int>& oldDefs = categoryData.GetDefsForUnitCat(defUnitCat);
+					std::vector<int>& newDefs = categoryData.GetDefsForUnitCat(cfgUnitCat);
+
+					for (vit = oldDefs.begin(); vit != oldDefs.end(); vit++) {
+						const int unitDefID = *vit;
+
+						if (unitDefID == unitDef->id) {
+							oldDefs.erase(vit);
+							newDefs.push_back(unitDefID);
+							vit--;
+						}
+					}
+
+					unitType->category = cfgUnitCat;
+				}
+			}
+		}
+	} else {
+		{
+			msg.str("");
+			msg << "\twriting new mod configuration file \"";
+			msg << cfgFileName << "\"";
+
+			L(ai, msg.str());
+		}
+
+		cfgFile.open(cfgFileName.c_str(), std::ios::out);
+		cfgFile << "config = {\n";
+		cfgFile << "\tversion = " << CFGVERSION << ",\n\n";
+
+		for (int i = 1; i <= numDefs; i++) {
+			UnitType* unitType = &unitTypes[i];
+
+			// assign and write default values for costMultiplier
+			// and techLevel, category is already set in ::Init()
+			unitType->costMultiplier =  1.0f;
+			unitType->techLevel      = -1;
+
+			cfgFile << "\t" << unitType->def->name << " = {\n";
+			cfgFile << "\t\tcostMult = " << unitType->costMultiplier << " * 100" << ",\n";
+			cfgFile << "\t\ttechLevel = " << unitType->techLevel << ",\n";
+			cfgFile << "\t\tcategory = " << unitType->category << ",\n";
+			cfgFile << "\t},\n";
+		}
+
+		cfgFile << "}\n";
+		cfgFile.close();
+	}
 }
 
 
@@ -383,7 +402,8 @@ float CUnitTable::GetDPSvsUnit(const UnitDef* unitDef, const UnitDef* victim) {
 
 
 float CUnitTable::GetCurrentDamageScore(const UnitDef* unit) {
-	int numEnemies = ai->ccb->GetEnemyUnits(&ai->unitIDs[0]);
+	const int numEnemies = ai->ccb->GetEnemyUnits(&ai->unitIDs[0]);
+
 	std::vector<int> enemiesOfType(ai->cb->GetNumUnitDefs() + 1, 0);
 
 	float score = 0.01f;
@@ -764,7 +784,6 @@ void CUnitTable::Init() {
 
 	ai->cb->GetUnitDefList(&unitDefs[0]);
 
-	// add units to UnitTable
 	for (int i = 1; i <= numDefs; i++) {
 		unitTypes[i].def      = unitDefs[i - 1];
 		unitTypes[i].category = CAT_LAST;
@@ -826,19 +845,19 @@ void CUnitTable::Init() {
 
 		else if (!uType->def->canfly) {
 			if (true /* uType->def->minWaterDepth <= 0 */) {
-				if (uType->def->buildOptions.size() >= 1 && uType->def->builder) {
-					uType->isHub = false;
+				const UnitDef* uDef = uType->def;
 
-					// a hub is any non-mobile builder
-					// that can build other non-mobile
-					// units
-					const UnitDef* uDef = uType->def;
+				if (uDef->buildOptions.size() >= 1 && uDef->builder) {
+					uType->isHub = false;
 
 					#define IS_MOBILE(uDef)                                           \
 						((uDef->speed > 0.0f) &&                                      \
 						((uDef->canmove && uDef->movedata != NULL) || uDef->canfly))
 
 					if (!IS_MOBILE(uDef)) {
+						// a hub is any non-mobile builder
+						// that can build other non-mobile
+						// units
 						typedef std::map<int, std::string>::const_iterator BuildOptIt;
 
 						for (BuildOptIt boIt = uDef->buildOptions.begin(); boIt != uDef->buildOptions.end(); boIt++) {
@@ -851,13 +870,14 @@ void CUnitTable::Init() {
 						}
 					}
 
+					#undef IS_MOBILE
+
 					categoryData.groundFactories.push_back(i);
 					uType->category = CAT_FACTORY;
-				}
-				else {
-					const WeaponDef* weapon = (uType->def->weapons.empty())? 0: uType->def->weapons.begin()->def;
+				} else {
+					const WeaponDef* weapon = (uDef->weapons.empty())? 0: uDef->weapons.begin()->def;
 
-					if (weapon && !weapon->stockpile && uType->def->extractsMetal == 0.0f) {
+					if (weapon && !weapon->stockpile && uDef->extractsMetal == 0.0f) {
 						// we don't want armed extractors to be seen as general-purpose defense
 						if (!weapon->waterweapon) {
 							// filter out depth-charge launchers etc
@@ -866,42 +886,42 @@ void CUnitTable::Init() {
 						}
 					}
 
-					if (uType->def->stockpileWeaponDef) {
-						if (uType->def->stockpileWeaponDef->targetable) {
+					if (uDef->stockpileWeaponDef) {
+						if (uDef->stockpileWeaponDef->targetable) {
 							// nuke
 							categoryData.nukeSilos.push_back(i);
 							uType->category = CAT_NUKE;
 						}
-						if (uType->def->stockpileWeaponDef->interceptor) {
+						if (uDef->stockpileWeaponDef->interceptor) {
 							// anti-nuke, not implemented yet
 						}
 					}
 
-					if (uType->def->shieldWeaponDef && uType->def->shieldWeaponDef->isShield) {
+					if (uDef->shieldWeaponDef && uDef->shieldWeaponDef->isShield) {
 						// shield, not implemented yet
 						// uType->category = CAT_SHIELD;
 					}
 
-					if (uType->def->makesMetal) {
+					if (uDef->makesMetal) {
 						categoryData.metalMakers.push_back(i);
 						uType->category = CAT_MMAKER;
 					}
-					if (uType->def->extractsMetal > 0.0f) {
+					if (uDef->extractsMetal > 0.0f) {
 						categoryData.metalExtractors.push_back(i);
 						uType->category = CAT_MEX;
 					}
-					if (((uType->def->energyMake - uType->def->energyUpkeep) / UnitCost) > 0.002 || uType->def->tidalGenerator || uType->def->windGenerator) {
-						if (/* uType->def->minWaterDepth <= 0 && */ !uType->def->needGeo) {
+					if (((uDef->energyMake - uDef->energyUpkeep) / UnitCost) > 0.002 || uDef->tidalGenerator || uDef->windGenerator) {
+						if (/* uDef->minWaterDepth <= 0 && */ !uDef->needGeo) {
 							// filter tidals and geothermals
 							categoryData.groundEnergy.push_back(i);
 							uType->category = CAT_ENERGY;
 						}
 					}
-					if (uType->def->energyStorage / UnitCost > 0.2) {
+					if (uDef->energyStorage / UnitCost > 0.2) {
 						categoryData.energyStorages.push_back(i);
 						uType->category = CAT_ESTOR;
 					}
-					if (uType->def->metalStorage / UnitCost > 0.1) {
+					if (uDef->metalStorage / UnitCost > 0.1) {
 						categoryData.metalStorages.push_back(i);
 						uType->category = CAT_MSTOR;
 					}
@@ -1023,10 +1043,14 @@ std::string CUnitTable::GetDbgLogName() const {
 }
 
 std::string CUnitTable::GetModCfgName() const {
+	// name is used for human readability,
+	// while hash is used for uniqueness
+	// (in case the modder forgets changing the name inbetween versions)
 	std::string relFile =
 		std::string(CFGFOLDER) +
-		(ai->cb->GetModName()) +
-		".cfg";
+		AIUtil::MakeFileSystemCompatible(ai->cb->GetModHumanName()) +
+		"-" + IntToString(ai->cb->GetModHash(), "%x") +
+		".lua";
 	std::string absFile = AIUtil::GetAbsFileName(ai->cb, relFile);
 
 	return absFile;
