@@ -2,6 +2,7 @@
 #include "RAI.h"
 #include <set>
 #include "Sim/Features/FeatureDef.h"
+#include "System/Util.h"
 //#include <time.h>
 #include <stdio.h>
 
@@ -100,11 +101,12 @@ GlobalResourceMap::GlobalResourceMap(IAICallback* _cb, cLogFile* l, GlobalTerrai
 				fList[i] = fList[--fSize];
 	}
 
-	relResourceFileName = "cache/" + string(cb->GetModName());
-	relResourceFileName.resize(relResourceFileName.size()-4);
-	relResourceFileName += "-" + string(cb->GetMapName());
-	relResourceFileName.resize(relResourceFileName.size()-3);
-	relResourceFileName += "res";
+	relResourceFileName = "cache/" + cRAI::MakeFileSystemCompatible(cb->GetModHumanName());
+	relResourceFileName += "-" + IntToString(cb->GetModHash(), "%x");
+	relResourceFileName += "-" + cRAI::MakeFileSystemCompatible(cb->GetMapName());
+	relResourceFileName.resize(relResourceFileName.size() - 4); // cut off extension
+	relResourceFileName += "-" + IntToString(cb->GetMapHash(), "%x");
+	relResourceFileName += ".res";
 
 	string resourceFileName_r;
 	FILE* resourceFile_r = NULL;
