@@ -1,16 +1,15 @@
 #include "RAI.h"
-#include "ExternalAI/IGlobalAICallback.h"
+#include "LegacyCpp/IGlobalAICallback.h"
 //#include "ExternalAI/IAICheats.h"
-#include "Sim/Units/UnitDef.h"
+#include "LegacyCpp/UnitDef.h"
+#include "Sim/Units/CommandAI/CommandQueue.h"
 #include "Sim/MoveTypes/MoveInfo.h"
 #include "CUtils/Util.h"
-//#include <vector>
-//#include <iostream>
+#include "System/Util.h"
 #include <stdio.h>
 //#include <direct.h>	// mkdir function (windows)
 //#include <sys/stat.h>	// mkdir function (linux)
 #include <time.h>		// time(NULL)
-//#include "KrogsMetalClass-v0.4/MetalMap.h"
 
 static GlobalResourceMap* GRMap=0;
 static GlobalTerrainMap* GTMap=0;
@@ -132,9 +131,7 @@ void cRAI::InitAI(IGlobalAICallback* callback, int team)
 	cb = callback->GetAICallback();
 	if( GRMap == 0 )
 		ClearLogFiles();
-	char c[3];
-	SNPRINTF(c, 3, "%i", cb->GetMyTeam());
-	l = new cLogFile(cb, "RAI"+string(c)+"_LastGame.log", false);
+	l = new cLogFile(cb, GetLogFileSubPath(cb->GetMyTeam()), false);
 
 /*	string test = (char*)cb->GetMyTeam(); // Crashes Spring?  Spring-Version(v0.76b1)
 	*l<<"cb->GetMyTeam()="<<test<<"\n";
@@ -143,8 +140,8 @@ void cRAI::InitAI(IGlobalAICallback* callback, int team)
 	{
 		ClearLogFiles();
 		*l<<"Loading Global RAI...";
-		*l<<"\n Mod = "<<cb->GetModName();
-		*l<<"\n Map = "<<cb->GetMapName();
+		*l<<"\n Mod = " << cb->GetModHumanName() << "(" << IntToString(cb->GetModHash(), "%x") << ")";
+		*l<<"\n Map = " << cb->GetMapName()      << "(" << IntToString(cb->GetMapHash(), "%x") << ")";
 		int seed = time(NULL);
 		srand(seed);
 		RAIs=0;
@@ -628,42 +625,49 @@ int cRAI::HandleEvent(int msg,const void* data)
 	switch (msg)
 	{
 	case AI_EVENT_UNITGIVEN:
-		{
-			const IGlobalAI::ChangeTeamEvent* cte = (const IGlobalAI::ChangeTeamEvent*) data;
-			if( cte->newteam != cb->GetMyTeam() )
-			{
-				cb->SendTextMsg("cRAI::HandleEvent(AI_EVENT_UNITGIVEN): This AI is out of date, check for a more recent one.",0);
-				*l<<"\nERROR: cRAI::HandleEvent(AI_EVENT_UNITGIVEN): This AI is out of date, check for a more recent one.\n";
-			}
-
-			if( Enemies.find(cte->unit) != Enemies.end() )
-				EnemyDestroyed(cte->unit,-1);
-
-			if( cb->GetUnitHealth(cte->unit) <= 0 ) // ! Work Around:  Spring-Version(v0.74b1-0.75b2)
-			{
-				*l<<"\nERROR: HandleEvent(AI_EVENT_UNITGIVEN): given unit is dead or does not exist";
-				return 0;
-			}
-
-			UnitCreated(cte->unit, -1);
-			Units.find(cte->unit)->second.AIDisabled=false;
-			if( !cb->UnitBeingBuilt(cte->unit) )
-			{
-				UnitFinished(cte->unit);
-				UnitIdle(cte->unit);
-			}
-		}
-		break;
 	case AI_EVENT_UNITCAPTURED:
 		{
 			const IGlobalAI::ChangeTeamEvent* cte = (const IGlobalAI::ChangeTeamEvent*) data;
-			if( cte->oldteam != cb->GetMyTeam() )
+
+			const int myAllyTeamId = cb->GetMyAllyTeam();
+			const bool oldEnemy = !cb->IsAllied(myAllyTeamId, cb->GetTeamAllyTeam(cte->oldteam));
+			const bool newEnemy = !cb->IsAllied(myAllyTeamId, cb->GetTeamAllyTeam(cte->newteam));
+
+			if ( oldEnemy && !newEnemy ) {
 			{
-				cb->SendTextMsg("cRAI::HandleEvent(AI_EVENT_UNITCAPTURED): This AI is out of date, check for a more recent one.",0);
-				*l<<"\nERROR: cRAI::HandleEvent(AI_EVENT_UNITCAPTURED): This AI is out of date, check for a more recent one.\n";
+				if( Enemies.find(cte->unit) != Enemies.end() )
+					EnemyDestroyed(cte->unit,-1);
+				}
+			}
+			else if( !oldEnemy && newEnemy )
+			{
+				// unit changed from an ally to an enemy team
+				// we lost a friend! :(
+				EnemyCreated(cte->unit);
+				if (!cb->UnitBeingBuilt(cte->unit)) {
+					EnemyFinished(cte->unit);
+				}
 			}
 
-			UnitDestroyed(cte->unit,-1);
+			if( cte->oldteam == cb->GetMyTeam() )
+			{
+				UnitDestroyed(cte->unit,-1);
+			}
+			else if( cte->newteam == cb->GetMyTeam() )
+			{
+				if( cb->GetUnitHealth(cte->unit) <= 0 ) // ! Work Around:  Spring-Version(v0.74b1-0.75b2)
+				{
+					*l<<"\nERROR: HandleEvent(AI_EVENT_(UNITGIVEN|UNITCAPTURED)): given unit is dead or does not exist";
+					return 0;
+				}
+				UnitCreated(cte->unit, -1);
+				Units.find(cte->unit)->second.AIDisabled=false;
+				if( !cb->UnitBeingBuilt(cte->unit) )
+				{
+					UnitFinished(cte->unit);
+					UnitIdle(cte->unit);
+				}
+			}
 		}
 		break;
 	case AI_EVENT_PLAYER_COMMAND:
@@ -1106,6 +1110,33 @@ bool cRAI::LocateFile(IAICallback* cb, const string& relFileName, string& absFil
 	return located;
 }
 
+static bool IsFSGoodChar(const char c) {
+
+	if ((c >= '0') && (c <= '9')) {
+		return true;
+	} else if ((c >= 'a') && (c <= 'z')) {
+		return true;
+	} else if ((c >= 'A') && (c <= 'Z')) {
+		return true;
+	} else if ((c == '.') || (c == '_') || (c == '-')) {
+		return true;
+	}
+
+	return false;
+}
+std::string cRAI::MakeFileSystemCompatible(const std::string& str) {
+
+	std::string cleaned = str;
+
+	for (std::string::size_type i=0; i < cleaned.size(); i++) {
+		if (!IsFSGoodChar(cleaned[i])) {
+			cleaned[i] = '_';
+		}
+	}
+
+	return cleaned;
+}
+
 void cRAI::RemoveLogFile(string relFileName) const {
 
 	string absFileName;
@@ -1114,22 +1145,26 @@ void cRAI::RemoveLogFile(string relFileName) const {
 	}
 }
 
+std::string cRAI::GetLogFileSubPath(int teamId) const {
+
+	static const size_t logFileSubPath_sizeMax = 64;
+	char logFileSubPath[logFileSubPath_sizeMax];
+	SNPRINTF(logFileSubPath, logFileSubPath_sizeMax, "log/RAI%i_LastGame.log", teamId);
+	return std::string(logFileSubPath);
+}
+
 void cRAI::ClearLogFiles()
 {
-	string logDir = "";
-
-	for( int i=0; i<32; i++ )
+	for( int t=0; t < 255; t++ )
 	{
-		char c[3];
-		SNPRINTF(c, 3, "%i", i);
-		RemoveLogFile(logDir+"RAI"+string(c)+"_LastGame.log");
+		RemoveLogFile(GetLogFileSubPath(t));
 	}
 
-	RemoveLogFile(logDir+"RAIGlobal_LastGame.log");
-	RemoveLogFile(logDir+"TerrainMapDebug.log");
-//	RemoveLogFile(logDir+"PathfinderDebug.log");
-//	RemoveLogFile(logDir+"PathFinderAPNDebug.log");
-//	RemoveLogFile(logDir+"PathFinderNPNDebug.log");
-//	RemoveLogFile(logDir+"Prerequisite.log");
-//	RemoveLogFile(logDir+"Debug.log");
+	RemoveLogFile("log/RAIGlobal_LastGame.log");
+	RemoveLogFile("log/TerrainMapDebug.log");
+//	RemoveLogFile("log/PathfinderDebug.log");
+//	RemoveLogFile("log/PathFinderAPNDebug.log");
+//	RemoveLogFile("log/PathFinderNPNDebug.log");
+//	RemoveLogFile("log/Prerequisite.log");
+//	RemoveLogFile("log/Debug.log");
 }
