@@ -16,24 +16,19 @@
 */
 
 #include "AIAICallback.h"
-
+#include "UnitDef.h"
 
 #include "ExternalAI/Interface/SSkirmishAICallback.h"
 #include "ExternalAI/Interface/AISCommands.h"
 
 #include "creg/creg_cond.h"
-#include "Sim/Units/UnitDef.h"
 #include "Sim/Units/CommandAI/CommandQueue.h"
 #ifdef USING_CREG
 creg::Class* CCommandQueue::GetClass() { return NULL; }
 #endif
 #include "Sim/MoveTypes/MoveInfo.h"
-UnitDef::~UnitDef() {
-	delete movedata;
-}
 CIcon::CIcon() {}
 CIcon::~CIcon() {}
-UnitDef::UnitDefWeapon::UnitDefWeapon() {}
 #include "Sim/Features/FeatureDef.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
 WeaponDef::~WeaponDef() {}
@@ -152,6 +147,14 @@ const char* CAIAICallback::GetTeamSide(int team) {
 	return sAICallback->Clb_Game_getTeamSide(teamId, team);
 }
 
+int CAIAICallback::GetTeamAllyTeam(int team) {
+	return sAICallback->Clb_Game_getTeamAllyTeam(teamId, team);
+}
+
+bool CAIAICallback::IsAllied(int firstAllyTeamId, int secondAllyTeamId) {
+	return sAICallback->Clb_Game_isAllied(teamId, firstAllyTeamId, secondAllyTeamId);
+}
+
 int CAIAICallback::GetUnitGroup(int unitId) {
 	return sAICallback->Clb_Unit_getGroup(teamId, unitId);
 }
@@ -304,10 +307,18 @@ const UnitDef* CAIAICallback::GetUnitDef(int unitId) {
 	return this->GetUnitDefById(unitDefId);
 }
 
+
+
 float3 CAIAICallback::GetUnitPos(int unitId) {
 	return float3(sAICallback->Clb_Unit_getPos(teamId, unitId));
 }
 
+float3 CAIAICallback::GetUnitVel(int unitId) {
+	return float3(sAICallback->Clb_Unit_getVel(teamId, unitId));
+}
+
+
+
 int CAIAICallback::GetBuildingFacing(int unitId) {
 	return sAICallback->Clb_Unit_getBuildingFacing(teamId, unitId);
 }
@@ -351,15 +362,15 @@ const UnitDef* CAIAICallback::GetUnitDefById(int unitDefId) {
 		return NULL;
 	}
 
-	bool doRecreate = unitDefFrames[unitDefId] < 0;
+	const bool doRecreate = (unitDefFrames[unitDefId] < 0);
 
 	if (doRecreate) {
 //		int currentFrame = this->GetCurrentFrame();
 		int currentFrame = 1;
 
 		UnitDef* unitDef = new UnitDef();
-		unitDef->valid = sAICallback->Clb_UnitDef_isValid(teamId, unitDefId);
 		unitDef->name = sAICallback->Clb_UnitDef_getName(teamId, unitDefId);
+
 		unitDef->humanName = sAICallback->Clb_UnitDef_getHumanName(teamId, unitDefId);
 		unitDef->filename = sAICallback->Clb_UnitDef_getFileName(teamId, unitDefId);
 		//unitDef->id = sAICallback->Clb_UnitDef_getId(teamId, unitDefId);
@@ -431,17 +442,12 @@ const UnitDef* CAIAICallback::GetUnitDefById(int unitDefId) {
 		unitDef->flankingBonusMax = sAICallback->Clb_UnitDef_FlankingBonus_getMax(teamId, unitDefId);
 		unitDef->flankingBonusMin = sAICallback->Clb_UnitDef_FlankingBonus_getMin(teamId, unitDefId);
 		unitDef->flankingBonusMobilityAdd = sAICallback->Clb_UnitDef_FlankingBonus_getMobilityAdd(teamId, unitDefId);
-		unitDef->collisionVolumeTypeStr = sAICallback->Clb_UnitDef_CollisionVolume_getType(teamId, unitDefId);
-		unitDef->collisionVolumeScales = float3(sAICallback->Clb_UnitDef_CollisionVolume_getScales(teamId, unitDefId));
-		unitDef->collisionVolumeOffsets = float3(sAICallback->Clb_UnitDef_CollisionVolume_getOffsets(teamId, unitDefId));
-		unitDef->collisionVolumeTest = sAICallback->Clb_UnitDef_CollisionVolume_getTest(teamId, unitDefId);
 		unitDef->maxWeaponRange = sAICallback->Clb_UnitDef_getMaxWeaponRange(teamId, unitDefId);
 		unitDef->type = sAICallback->Clb_UnitDef_getType(teamId, unitDefId);
 		unitDef->tooltip = sAICallback->Clb_UnitDef_getTooltip(teamId, unitDefId);
 		unitDef->wreckName = sAICallback->Clb_UnitDef_getWreckName(teamId, unitDefId);
 		unitDef->deathExplosion = sAICallback->Clb_UnitDef_getDeathExplosion(teamId, unitDefId);
 		unitDef->selfDExplosion = sAICallback->Clb_UnitDef_getSelfDExplosion(teamId, unitDefId);
-		unitDef->TEDClassString = sAICallback->Clb_UnitDef_getTedClassString(teamId, unitDefId);
 		unitDef->categoryString = sAICallback->Clb_UnitDef_getCategoryString(teamId, unitDefId);
 		unitDef->canSelfD = sAICallback->Clb_UnitDef_isAbleToSelfD(teamId, unitDefId);
 		unitDef->selfDCountdown = sAICallback->Clb_UnitDef_getSelfDCountdown(teamId, unitDefId);
@@ -546,7 +552,6 @@ const UnitDef* CAIAICallback::GetUnitDefById(int unitDefId) {
 		unitDef->flareSalvoDelay = sAICallback->Clb_UnitDef_getFlareSalvoDelay(teamId, unitDefId);
 		//unitDef->smoothAnim = sAICallback->Clb_UnitDef_isSmoothAnim(teamId, unitDefId);
 		unitDef->smoothAnim = false;
-		unitDef->isMetalMaker = sAICallback->Clb_UnitDef_0REF1Resource2resourceId0isResourceMaker(teamId, unitDefId, m);
 		unitDef->canLoopbackAttack = sAICallback->Clb_UnitDef_isAbleToLoopbackAttack(teamId, unitDefId);
 		unitDef->levelGround = sAICallback->Clb_UnitDef_isLevelGround(teamId, unitDefId);
 		unitDef->useBuildingGroundDecal = sAICallback->Clb_UnitDef_isUseBuildingGroundDecal(teamId, unitDefId);
@@ -564,13 +569,16 @@ const UnitDef* CAIAICallback::GetUnitDefById(int unitDefId) {
 		unitDef->stockpileWeaponDef = this->GetWeaponDefById(sAICallback->Clb_UnitDef_0SINGLE1FETCH2WeaponDef0getStockpileDef(teamId, unitDefId));
 
 		{
-			int numBo = sAICallback->Clb_UnitDef_0ARRAY1SIZE1UnitDef0getBuildOptions(teamId, unitDefId);
-			int* bo = new int[numBo];
-			numBo = sAICallback->Clb_UnitDef_0ARRAY1VALS1UnitDef0getBuildOptions(teamId, unitDefId, bo, numBo);
-			for (int b=0; b < numBo; b++) {
-				unitDef->buildOptions[b] = sAICallback->Clb_UnitDef_getName(teamId, bo[b]);
+			int numBuildOpts = sAICallback->Clb_UnitDef_0ARRAY1SIZE1UnitDef0getBuildOptions(teamId, unitDefId);
+			int* buildOpts = new int[numBuildOpts];
+
+			numBuildOpts = sAICallback->Clb_UnitDef_0ARRAY1VALS1UnitDef0getBuildOptions(teamId, unitDefId, buildOpts, numBuildOpts);
+
+			for (int b = 0; b < numBuildOpts; b++) {
+				unitDef->buildOptions[b] = sAICallback->Clb_UnitDef_getName(teamId, buildOpts[b]);
 			}
-			delete [] bo;
+
+			delete[] buildOpts;
 		}
 		{
 			int size = sAICallback->Clb_UnitDef_0MAP1SIZE0getCustomParams(teamId, unitDefId);
@@ -678,7 +686,7 @@ const float* CAIAICallback::GetHeightMap() {
 
 	if (heightMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getHeightMap(teamId);
-		heightMap = new float[size]; // NOTE: memory leack, but will be used till end of the game anyway
+		heightMap = new float[size]; // NOTE: memory leak, but will be used till end of the game anyway
 		sAICallback->Clb_Map_0ARRAY1VALS0getHeightMap(teamId, heightMap, size);
 	}
 
@@ -691,7 +699,7 @@ const float* CAIAICallback::GetCornersHeightMap() {
 
 	if (cornersHeightMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getCornersHeightMap(teamId);
-		cornersHeightMap = new float[size]; // NOTE: memory leack, but will be used till end of the game anyway
+		cornersHeightMap = new float[size]; // NOTE: memory leak, but will be used till end of the game anyway
 		sAICallback->Clb_Map_0ARRAY1VALS0getCornersHeightMap(teamId, cornersHeightMap, size);
 	}
 
@@ -712,7 +720,7 @@ const float* CAIAICallback::GetSlopeMap() {
 
 	if (slopeMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getSlopeMap(teamId);
-		slopeMap = new float[size]; // NOTE: memory leack, but will be used till end of the game anyway
+		slopeMap = new float[size]; // NOTE: memory leak, but will be used till end of the game anyway
 		sAICallback->Clb_Map_0ARRAY1VALS0getSlopeMap(teamId, slopeMap, size);
 	}
 
@@ -725,7 +733,7 @@ const unsigned short* CAIAICallback::GetLosMap() {
 
 	if (losMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getLosMap(teamId);
-		losMap = new unsigned short[size]; // NOTE: memory leack, but will be used till end of the game anyway
+		losMap = new unsigned short[size]; // NOTE: memory leak, but will be used till end of the game anyway
 		sAICallback->Clb_Map_0ARRAY1VALS0getLosMap(teamId, losMap, size);
 	}
 
@@ -746,7 +754,7 @@ const unsigned short* CAIAICallback::GetRadarMap() {
 
 	if (radarMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getRadarMap(teamId);
-		radarMap = new unsigned short[size]; // NOTE: memory leack, but will be used till end of the game anyway
+		radarMap = new unsigned short[size]; // NOTE: memory leak, but will be used till end of the game anyway
 		sAICallback->Clb_Map_0ARRAY1VALS0getRadarMap(teamId, radarMap, size);
 	}
 
@@ -759,7 +767,7 @@ const unsigned short* CAIAICallback::GetJammerMap() {
 
 	if (jammerMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getJammerMap(teamId);
-		jammerMap = new unsigned short[size]; // NOTE: memory leack, but will be used till end of the game anyway
+		jammerMap = new unsigned short[size]; // NOTE: memory leak, but will be used till end of the game anyway
 		sAICallback->Clb_Map_0ARRAY1VALS0getJammerMap(teamId, jammerMap, size);
 	}
 
@@ -773,21 +781,45 @@ const unsigned char* CAIAICallback::GetMetalMap() {
 
 	if (metalMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0REF1Resource2resourceId0getResourceMapRaw(teamId, m);
-		metalMap = new unsigned char[size]; // NOTE: memory leack, but will be used till end of the game anyway
+		metalMap = new unsigned char[size]; // NOTE: memory leak, but will be used till end of the game anyway
 		sAICallback->Clb_Map_0ARRAY1VALS0REF1Resource2resourceId0getResourceMapRaw(teamId, m, metalMap, size);
 	}
 
 	return metalMap;
 }
 
+int CAIAICallback::GetMapHash() {
+	return sAICallback->Clb_Map_getHash(teamId);
+}
+
 const char* CAIAICallback::GetMapName() {
 	return sAICallback->Clb_Map_getName(teamId);
 }
 
+const char* CAIAICallback::GetMapHumanName() {
+	return sAICallback->Clb_Map_getHumanName(teamId);
+}
+
+int CAIAICallback::GetModHash() {
+	return sAICallback->Clb_Mod_getHash(teamId);
+}
+
 const char* CAIAICallback::GetModName() {
 	return sAICallback->Clb_Mod_getFileName(teamId);
 }
 
+const char* CAIAICallback::GetModHumanName() {
+	return sAICallback->Clb_Mod_getHumanName(teamId);
+}
+
+const char* CAIAICallback::GetModShortName() {
+	return sAICallback->Clb_Mod_getShortName(teamId);
+}
+
+const char* CAIAICallback::GetModVersion() {
+	return sAICallback->Clb_Mod_getVersion(teamId);
+}
+
 float CAIAICallback::GetElevation(float x, float z) {
 	return sAICallback->Clb_Map_getElevationAt(teamId, x, z);
 }
@@ -943,10 +975,11 @@ int CAIAICallback::GetMapPoints(PointMarker* pm, int pm_sizeMax, bool includeAll
 	for (int p=0; p < numPoints; ++p) {
 		pm[p].pos = float3(sAICallback->Clb_Map_Point_getPosition(teamId, p));
 		SAIFloat3 tmpColor = sAICallback->Clb_Map_Point_getColor(teamId, p);
-		pm[p].color = (unsigned char*) calloc(3, sizeof(unsigned char));
-		pm[p].color[0] = (unsigned char) tmpColor.x;
-		pm[p].color[1] = (unsigned char) tmpColor.y;
-		pm[p].color[2] = (unsigned char) tmpColor.z;
+		unsigned char* pColor = (unsigned char*) calloc(3, sizeof(unsigned char));
+		pColor[0] = (unsigned char) tmpColor.x;
+		pColor[1] = (unsigned char) tmpColor.y;
+		pColor[2] = (unsigned char) tmpColor.z;
+		pm[p].color = pColor;
 		pm[p].label = sAICallback->Clb_Map_Point_getLabel(teamId, p);
 	}
 
@@ -960,10 +993,11 @@ int CAIAICallback::GetMapLines(LineMarker* lm, int lm_sizeMax, bool includeAllie
 		lm[l].pos = float3(sAICallback->Clb_Map_Line_getFirstPosition(teamId, l));
 		lm[l].pos2 = float3(sAICallback->Clb_Map_Line_getSecondPosition(teamId, l));
 		SAIFloat3 tmpColor = sAICallback->Clb_Map_Line_getColor(teamId, l);
-		lm[l].color = (unsigned char*) calloc(3, sizeof(unsigned char));
-		lm[l].color[0] = (unsigned char) tmpColor.x;
-		lm[l].color[1] = (unsigned char) tmpColor.y;
-		lm[l].color[2] = (unsigned char) tmpColor.z;
+		unsigned char* lColor = (unsigned char*) calloc(3, sizeof(unsigned char));
+		lColor[0] = (unsigned char) tmpColor.x;
+		lColor[1] = (unsigned char) tmpColor.y;
+		lColor[2] = (unsigned char) tmpColor.z;
+		lm[l].color = lColor;
 	}
 
 	return numLines;
@@ -1045,35 +1079,31 @@ const FeatureDef* CAIAICallback::GetFeatureDefById(int featureDefId) {
 //		int currentFrame = this->GetCurrentFrame();
 		int currentFrame = 1;
 	FeatureDef* featureDef = new FeatureDef();
-featureDef->myName = sAICallback->Clb_FeatureDef_getName(teamId, featureDefId);
-featureDef->description = sAICallback->Clb_FeatureDef_getDescription(teamId, featureDefId);
-featureDef->filename = sAICallback->Clb_FeatureDef_getFileName(teamId, featureDefId);
-//featureDef->id = sAICallback->Clb_FeatureDef_getId(teamId, featureDefId);
-featureDef->id = featureDefId;
-featureDef->metal = sAICallback->Clb_FeatureDef_0REF1Resource2resourceId0getContainedResource(teamId, featureDefId, m);
-featureDef->energy = sAICallback->Clb_FeatureDef_0REF1Resource2resourceId0getContainedResource(teamId, featureDefId, e);
-featureDef->maxHealth = sAICallback->Clb_FeatureDef_getMaxHealth(teamId, featureDefId);
-featureDef->reclaimTime = sAICallback->Clb_FeatureDef_getReclaimTime(teamId, featureDefId);
-featureDef->mass = sAICallback->Clb_FeatureDef_getMass(teamId, featureDefId);
-featureDef->collisionVolumeTypeStr = sAICallback->Clb_FeatureDef_CollisionVolume_getType(teamId, featureDefId);
-featureDef->collisionVolumeScales = float3(sAICallback->Clb_FeatureDef_CollisionVolume_getScales(teamId, featureDefId));
-featureDef->collisionVolumeOffsets = float3(sAICallback->Clb_FeatureDef_CollisionVolume_getOffsets(teamId, featureDefId));
-featureDef->collisionVolumeTest = sAICallback->Clb_FeatureDef_CollisionVolume_getTest(teamId, featureDefId);
-featureDef->upright = sAICallback->Clb_FeatureDef_isUpright(teamId, featureDefId);
-featureDef->drawType = sAICallback->Clb_FeatureDef_getDrawType(teamId, featureDefId);
-featureDef->modelname = sAICallback->Clb_FeatureDef_getModelName(teamId, featureDefId);
-featureDef->resurrectable = sAICallback->Clb_FeatureDef_getResurrectable(teamId, featureDefId);
-featureDef->smokeTime = sAICallback->Clb_FeatureDef_getSmokeTime(teamId, featureDefId);
-featureDef->destructable = sAICallback->Clb_FeatureDef_isDestructable(teamId, featureDefId);
-featureDef->reclaimable = sAICallback->Clb_FeatureDef_isReclaimable(teamId, featureDefId);
-featureDef->blocking = sAICallback->Clb_FeatureDef_isBlocking(teamId, featureDefId);
-featureDef->burnable = sAICallback->Clb_FeatureDef_isBurnable(teamId, featureDefId);
-featureDef->floating = sAICallback->Clb_FeatureDef_isFloating(teamId, featureDefId);
-featureDef->noSelect = sAICallback->Clb_FeatureDef_isNoSelect(teamId, featureDefId);
-featureDef->geoThermal = sAICallback->Clb_FeatureDef_isGeoThermal(teamId, featureDefId);
-featureDef->deathFeature = sAICallback->Clb_FeatureDef_getDeathFeature(teamId, featureDefId);
-featureDef->xsize = sAICallback->Clb_FeatureDef_getXSize(teamId, featureDefId);
-featureDef->zsize = sAICallback->Clb_FeatureDef_getZSize(teamId, featureDefId);
+	featureDef->myName = sAICallback->Clb_FeatureDef_getName(teamId, featureDefId);
+	featureDef->description = sAICallback->Clb_FeatureDef_getDescription(teamId, featureDefId);
+	featureDef->filename = sAICallback->Clb_FeatureDef_getFileName(teamId, featureDefId);
+	//featureDef->id = sAICallback->Clb_FeatureDef_getId(teamId, featureDefId);
+	featureDef->id = featureDefId;
+	featureDef->metal = sAICallback->Clb_FeatureDef_0REF1Resource2resourceId0getContainedResource(teamId, featureDefId, m);
+	featureDef->energy = sAICallback->Clb_FeatureDef_0REF1Resource2resourceId0getContainedResource(teamId, featureDefId, e);
+	featureDef->maxHealth = sAICallback->Clb_FeatureDef_getMaxHealth(teamId, featureDefId);
+	featureDef->reclaimTime = sAICallback->Clb_FeatureDef_getReclaimTime(teamId, featureDefId);
+	featureDef->mass = sAICallback->Clb_FeatureDef_getMass(teamId, featureDefId);
+	featureDef->upright = sAICallback->Clb_FeatureDef_isUpright(teamId, featureDefId);
+	featureDef->drawType = sAICallback->Clb_FeatureDef_getDrawType(teamId, featureDefId);
+	featureDef->modelname = sAICallback->Clb_FeatureDef_getModelName(teamId, featureDefId);
+	featureDef->resurrectable = sAICallback->Clb_FeatureDef_getResurrectable(teamId, featureDefId);
+	featureDef->smokeTime = sAICallback->Clb_FeatureDef_getSmokeTime(teamId, featureDefId);
+	featureDef->destructable = sAICallback->Clb_FeatureDef_isDestructable(teamId, featureDefId);
+	featureDef->reclaimable = sAICallback->Clb_FeatureDef_isReclaimable(teamId, featureDefId);
+	featureDef->blocking = sAICallback->Clb_FeatureDef_isBlocking(teamId, featureDefId);
+	featureDef->burnable = sAICallback->Clb_FeatureDef_isBurnable(teamId, featureDefId);
+	featureDef->floating = sAICallback->Clb_FeatureDef_isFloating(teamId, featureDefId);
+	featureDef->noSelect = sAICallback->Clb_FeatureDef_isNoSelect(teamId, featureDefId);
+	featureDef->geoThermal = sAICallback->Clb_FeatureDef_isGeoThermal(teamId, featureDefId);
+	featureDef->deathFeature = sAICallback->Clb_FeatureDef_getDeathFeature(teamId, featureDefId);
+	featureDef->xsize = sAICallback->Clb_FeatureDef_getXSize(teamId, featureDefId);
+	featureDef->zsize = sAICallback->Clb_FeatureDef_getZSize(teamId, featureDefId);
 {
 	int size = sAICallback->Clb_FeatureDef_0MAP1SIZE0getCustomParams(teamId, featureDefId);
 	featureDef->customParams = std::map<std::string,std::string>();
@@ -1115,11 +1145,13 @@ int CAIAICallback::GetNumUnitDefs() {
 }
 
 void CAIAICallback::GetUnitDefList(const UnitDef** list) {
-
 	int size = sAICallback->Clb_0MULTI1SIZE0UnitDef(teamId);
 	int* unitDefIds = new int[size];
+
+	// get actual number of IDs
 	size = sAICallback->Clb_0MULTI1VALS0UnitDef(teamId, unitDefIds, size);
-	for (int i=0; i < size; ++i) {
+
+	for (int i = 0; i < size; ++i) {
 		list[i] = this->GetUnitDefById(unitDefIds[i]);
 	}
 }
@@ -1431,16 +1463,16 @@ int CAIAICallback::Internal_GiveOrder(int unitId, int groupId, Command* c) {
 	return ret;
 }
 
-int CAIAICallback::InitPath(float3 start, float3 end, int pathType) {
-		SInitPathCommand cmd = {start.toSAIFloat3(), end.toSAIFloat3(), pathType}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_PATH_INIT, &cmd); return cmd.ret_pathId;
+int CAIAICallback::InitPath(float3 start, float3 end, int pathType, float goalRadius) {
+		SInitPathCommand cmd = {start.toSAIFloat3(), end.toSAIFloat3(), pathType, goalRadius}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_PATH_INIT, &cmd); return cmd.ret_pathId;
 }
 
 float3 CAIAICallback::GetNextWaypoint(int pathId) {
 		SGetNextWaypointPathCommand cmd = {pathId}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_PATH_GET_NEXT_WAYPOINT, &cmd); return float3(cmd.ret_nextWaypoint);
 }
 
-float CAIAICallback::GetPathLength(float3 start, float3 end, int pathType) {
-		SGetApproximateLengthPathCommand cmd = {start.toSAIFloat3(), end.toSAIFloat3(), pathType}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_PATH_GET_APPROXIMATE_LENGTH, &cmd); return cmd.ret_approximatePathLength;
+float CAIAICallback::GetPathLength(float3 start, float3 end, int pathType, float goalRadius) {
+		SGetApproximateLengthPathCommand cmd = {start.toSAIFloat3(), end.toSAIFloat3(), pathType, goalRadius}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_PATH_GET_APPROXIMATE_LENGTH, &cmd); return cmd.ret_approximatePathLength;
 }
 
 void CAIAICallback::FreePath(int pathId) {
@@ -1488,26 +1520,119 @@ void CAIAICallback::LineDrawerRestartSameColor() {
 }
 
 int CAIAICallback::CreateSplineFigure(float3 pos1, float3 pos2, float3 pos3, float3 pos4, float width, int arrow, int lifeTime, int figureGroupId) {
-		SCreateSplineFigureDrawerCommand cmd = {pos1.toSAIFloat3(), pos2.toSAIFloat3(), pos3.toSAIFloat3(), pos4.toSAIFloat3(), width, arrow, lifeTime, figureGroupId}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_CREATE_SPLINE, &cmd); return cmd.ret_newFigureGroupId;
+	SCreateSplineFigureDrawerCommand cmd = {
+		pos1.toSAIFloat3(),
+		pos2.toSAIFloat3(),
+		pos3.toSAIFloat3(),
+		pos4.toSAIFloat3(),
+		width,
+		arrow,
+		lifeTime,
+		figureGroupId
+	};
+
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_CREATE_SPLINE, &cmd);
+	return cmd.ret_newFigureGroupId;
 }
 
 int CAIAICallback::CreateLineFigure(float3 pos1, float3 pos2, float width, int arrow, int lifeTime, int figureGroupId) {
-		SCreateLineFigureDrawerCommand cmd = {pos1.toSAIFloat3(), pos2.toSAIFloat3(), width, arrow, lifeTime, figureGroupId}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_CREATE_LINE, &cmd); return cmd.ret_newFigureGroupId;
+	SCreateLineFigureDrawerCommand cmd = {
+		pos1.toSAIFloat3(),
+		pos2.toSAIFloat3(),
+		width,
+		arrow,
+		lifeTime,
+		figureGroupId
+	};
+
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_CREATE_LINE, &cmd);
+	return cmd.ret_newFigureGroupId;
 }
 
 void CAIAICallback::SetFigureColor(int figureGroupId, float red, float green, float blue, float alpha) {
 	SAIFloat3 col3 = {red, green, blue};
-	SSetColorFigureDrawerCommand cmd = {figureGroupId, col3, alpha}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_SET_COLOR, &cmd);
+	SSetColorFigureDrawerCommand cmd = {figureGroupId, col3, alpha};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_SET_COLOR, &cmd);
 }
 
 void CAIAICallback::DeleteFigureGroup(int figureGroupId) {
-	SDeleteFigureDrawerCommand cmd = {figureGroupId}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_DELETE, &cmd);
+	SDeleteFigureDrawerCommand cmd = {figureGroupId};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_FIGURE_DELETE, &cmd);
 }
 
 void CAIAICallback::DrawUnit(const char* name, float3 pos, float rotation, int lifeTime, int unitTeamId, bool transparent, bool drawBorder, int facing) {
-	SDrawUnitDrawerCommand cmd = {sAICallback->Clb_0MULTI1FETCH3UnitDefByName0UnitDef(teamId, name), pos.toSAIFloat3(), rotation, lifeTime, unitTeamId, transparent, drawBorder, facing}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_DRAW_UNIT, &cmd);
+	SDrawUnitDrawerCommand cmd = {
+		sAICallback->Clb_0MULTI1FETCH3UnitDefByName0UnitDef(teamId, name),
+		pos.toSAIFloat3(),
+		rotation,
+		lifeTime,
+		unitTeamId,
+		transparent,
+		drawBorder,
+		facing
+	};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DRAWER_DRAW_UNIT, &cmd);
 }
 
+
+
+bool CAIAICallback::IsDebugDrawerEnabled() const {
+	return sAICallback->Clb_Debug_Drawer_isEnabled(teamId);
+}
+
+void CAIAICallback::DebugDrawerAddGraphPoint(int lineId, float x, float y) {
+	SDebugDrawerAddGraphPointCommand cmd = {x, y, lineId};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_ADD_GRAPH_POINT, &cmd);
+}
+void CAIAICallback::DebugDrawerDelGraphPoints(int lineId, int numPoints) {
+	SDebugDrawerDeleteGraphPointsCommand cmd = {lineId, numPoints};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_DELETE_GRAPH_POINTS, &cmd);
+}
+void CAIAICallback::DebugDrawerSetGraphPos(float x, float y) {
+	SDebugDrawerSetGraphPositionCommand cmd = {x, y};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_SET_GRAPH_POS, &cmd);
+}
+void CAIAICallback::DebugDrawerSetGraphSize(float w, float h) {
+	SDebugDrawerSetGraphSizeCommand cmd = {w, h};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_SET_GRAPH_SIZE, &cmd);
+}
+void CAIAICallback::DebugDrawerSetGraphLineColor(int lineId, const float3& color) {
+	SDebugDrawerSetGraphLineColorCommand cmd = {lineId, color.toSAIFloat3()};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_SET_GRAPH_LINE_COLOR, &cmd);
+}
+void CAIAICallback::DebugDrawerSetGraphLineLabel(int lineId, const char* label) {
+	SDebugDrawerSetGraphLineLabelCommand cmd = {lineId, label};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_SET_GRAPH_LINE_LABEL, &cmd);
+}
+
+int CAIAICallback::DebugDrawerAddOverlayTexture(const float* texData, int w, int h) {
+	SDebugDrawerAddOverlayTextureCommand cmd = {0, texData, w, h};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_ADD_OVERLAY_TEXTURE, &cmd);
+	return cmd.texHandle;
+}
+void CAIAICallback::DebugDrawerUpdateOverlayTexture(int texHandle, const float* texData, int x, int y, int w, int h) {
+	SDebugDrawerUpdateOverlayTextureCommand cmd = {texHandle, texData, x, y, w, h};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_UPDATE_OVERLAY_TEXTURE, &cmd);
+}
+void CAIAICallback::DebugDrawerDelOverlayTexture(int texHandle) {
+	SDebugDrawerDelOverlayTextureCommand cmd = {texHandle};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_DEL_OVERLAY_TEXTURE, &cmd);
+}
+void CAIAICallback::DebugDrawerSetOverlayTexturePos(int texHandle, float x, float y) {
+	SDebugDrawerSetOverlayTexturePosCommand cmd = {texHandle, x, y};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_SET_OVERLAY_TEXTURE_POS, &cmd);
+}
+void CAIAICallback::DebugDrawerSetOverlayTextureSize(int texHandle, float w, float h) {
+	SDebugDrawerSetOverlayTextureSizeCommand cmd = {texHandle, w, h};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_SET_OVERLAY_TEXTURE_SIZE, &cmd);
+}
+void CAIAICallback::DebugDrawerSetOverlayTextureLabel(int texHandle, const char* texLabel) {
+	SDebugDrawerSetOverlayTextureLabelCommand cmd = {texHandle, texLabel};
+	sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_DEBUG_DRAWER_SET_OVERLAY_TEXTURE_LABEL, &cmd);
+}
+
+
+
 int CAIAICallback::HandleCommand(int commandId, void* data) {
 	int ret = -99;
 
@@ -1597,4 +1722,34 @@ const char* CAIAICallback::CallLuaRules(const char* data, int inSize, int* outSi
 		SCallLuaRulesCommand cmd = {data, inSize, outSize}; sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_CALL_LUA_RULES, &cmd); return cmd.ret_outData;
 }
 
+std::map<std::string, std::string> CAIAICallback::GetMyInfo()
+{
+	std::map<std::string, std::string> info;
 
+	const int info_size = sAICallback->Clb_SkirmishAI_Info_getSize(teamId);
+	for (int ii = 0; ii < info_size; ++ii) {
+		const char* key   = sAICallback->Clb_SkirmishAI_Info_getKey(teamId, ii);
+		const char* value = sAICallback->Clb_SkirmishAI_Info_getValue(teamId, ii);
+		if ((key != NULL) && (value != NULL)) {
+			info[key] = value;
+		}
+	}
+
+	return info;
+}
+
+std::map<std::string, std::string> CAIAICallback::GetMyOptionValues()
+{
+	std::map<std::string, std::string> optionVals;
+
+	const int optionVals_size = sAICallback->Clb_SkirmishAI_OptionValues_getSize(teamId);
+	for (int ovi = 0; ovi < optionVals_size; ++ovi) {
+		const char* key   = sAICallback->Clb_SkirmishAI_OptionValues_getKey(teamId, ovi);
+		const char* value = sAICallback->Clb_SkirmishAI_OptionValues_getValue(teamId, ovi);
+		if ((key != NULL) && (value != NULL)) {
+			optionVals[key] = value;
+		}
+	}
+
+	return optionVals;
+}
