@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
@@ -11,6 +13,7 @@
 COverviewController::COverviewController()
 {
 	enabled = false;
+	minimizeMinimap = false;
 }
 
 void COverviewController::KeyMove(float3 move)
@@ -34,7 +37,7 @@ float3 COverviewController::GetPos()
 	// map not created when constructor run
 	pos.x = gs->mapx * 4.0f;
 	pos.z = gs->mapy * 4.0f;
-	const float height = std::max(pos.x / gu->aspectRatio, pos.z);
+	const float height = std::max(pos.x / globalRendering->aspectRatio, pos.z);
 	pos.y = ground->GetHeight(pos.x, pos.z) + (2.5f * height);
 	return pos;
 }
@@ -54,7 +57,7 @@ float3 COverviewController::SwitchFrom() const
 	float length=ground->LineGroundCol(pos,pos+dir*50000);
 	float3 rpos=pos+dir*length;
 
-	if (!gu->dualScreenMode) {
+	if (!globalRendering->dualScreenMode) {
 		minimap->SetMinimized(minimizeMinimap);
 	}
 
@@ -67,7 +70,7 @@ void COverviewController::SwitchTo(bool showText)
 		logOutput.Print("Switching to Overview style camera");
 	}
 
-	if (!gu->dualScreenMode) {
+	if (!globalRendering->dualScreenMode) {
 		minimizeMinimap = minimap->GetMinimized();
 		minimap->SetMinimized(true);
 	}
