@@ -1,9 +1,10 @@
-#include "StdAfx.h"
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
+#include "StdAfx.h"
+#include "mmgr.h"
 #include <SDL_keysym.h>
 #include <SDL_timer.h>
-
-#include "mmgr.h"
+#include <boost/cstdint.hpp>
 
 #include "SmoothController.h"
 
@@ -14,8 +15,8 @@
 #include "LogOutput.h"
 #include "Map/Ground.h"
 #include "GlobalUnsynced.h"
-#include <boost/cstdint.hpp>
-
+#include "Rendering/GlobalRendering.h"
+#include "myMath.h"
 
 extern boost::uint8_t *keys;
 
@@ -28,6 +29,7 @@ SmoothController::SmoothController()
 	maxHeight(10000),
 	speedFactor(1)
 {
+	middleClickScrollSpeed = configHandler->Get("MiddleClickScrollSpeed", 0.01f);
 	scrollSpeed = configHandler->Get("SmoothScrollSpeed",10)*0.1f;
 	tiltSpeed = configHandler->Get("SmoothTiltSpeed",1.0f);
 	enabled = !!configHandler->Get("SmoothEnabled",1);
@@ -47,8 +49,8 @@ void SmoothController::KeyMove(float3 move)
 		move.x = -move.x;
 		move.y = -move.y;
 	}
-	move*=sqrt(move.z)*200;
-	float pixelsize= camera->GetTanHalfFov()*2/gu->viewSizeY*height*2;
+	move *= sqrt(move.z)*200;
+	float pixelsize = camera->GetTanHalfFov() * 2/globalRendering->viewSizeY * height * 2;
 	const float3 thisMove(move.x*pixelsize*2*scrollSpeed, 0, -move.y*pixelsize*2*scrollSpeed);
 	static unsigned lastKeyMove = SDL_GetTicks();
 	const unsigned timeDiff = SDL_GetTicks() - lastKeyMove;
@@ -73,7 +75,8 @@ void SmoothController::MouseMove(float3 move)
 		move.x = -move.x;
 		move.y = -move.y;
 	}
-	float pixelsize=100*mouseScale* camera->GetTanHalfFov() *2/gu->viewSizeY*height*2;
+	move *= 100 * middleClickScrollSpeed;
+	float pixelsize = camera->GetTanHalfFov() * 2/globalRendering->viewSizeY * height * 2;
 	float3 thisMove(move.x*pixelsize*(1+keys[SDLK_LSHIFT]*3)*scrollSpeed, 0, move.y*pixelsize*(1+keys[SDLK_LSHIFT]*3)*scrollSpeed);
 	// do little smoothing here (and because its little it won't hurt if it depends on framerate)
 	static float3 lastMove(0, 0, 0);
@@ -89,7 +92,7 @@ void SmoothController::ScreenEdgeMove(float3 move)
 		move.y = -move.y;
 	}
 	move*=sqrt(move.z)*200;
-	float pixelsize= camera->GetTanHalfFov()*2/gu->viewSizeY*height*2;
+	float pixelsize= camera->GetTanHalfFov()*2/globalRendering->viewSizeY*height*2;
 	const float3 thisMove(move.x*pixelsize*2*scrollSpeed, 0, -move.y*pixelsize*2*scrollSpeed);
 	static unsigned lastScreenMove = SDL_GetTicks();
 	const unsigned timeDiff = SDL_GetTicks() - lastScreenMove;
@@ -112,15 +115,14 @@ void SmoothController::MouseWheelMove(float move)
 {
 	// tilt the camera if LCTRL is pressed
 	if (keys[SDLK_LCTRL]) {
-		zscale *= (1.0f + (move * tiltSpeed * mouseScale * (keys[SDLK_LSHIFT] ? 3.0f : 1.0f)));
-		if (zscale < 0.05f) zscale = 0.05f;
-		if (zscale > 10) zscale = 10;
+		zscale *= (1.0f + (0.01f * move * tiltSpeed * (keys[SDLK_LSHIFT] ? 3.0f : 1.0f)));
+		zscale = Clamp(zscale, 0.05f, 10.0f);
 	} else { // holding down LALT uses 'instant-zoom' from here to the end of the function
 		// ZOOM IN to mouse cursor instead of mid screen
 		if (move < 0) {
-			float3 cpos=pos-dir*height;
-			float dif=-height * move * mouseScale*0.7f * (keys[SDLK_LSHIFT] ? 3:1);
-			if ((height - dif) <60.0f) {
+			float3 cpos = pos - dir * height;
+			float dif = -height * move * 0.007f * (keys[SDLK_LSHIFT] ? 3:1);
+			if ((height - dif) < 60.0f) {
 				dif = height - 60.0f;
 			}
 			if (keys[SDLK_LALT]) { // instant-zoom: zoom in to standard view
@@ -129,7 +131,7 @@ void SmoothController::MouseWheelMove(float move)
 			float3 wantedPos = cpos + mouse->dir * dif;
 			float newHeight = ground->LineGroundCol(wantedPos, wantedPos + dir * 15000);
 			if (newHeight < 0) {
-				newHeight = height* (1.0f + move * mouseScale * 0.7f * (keys[SDLK_LSHIFT] ? 3:1));
+				newHeight = height * (1.0f + move * 0.007f * (keys[SDLK_LSHIFT] ? 3:1));
 			}
 			if ((wantedPos.y + (dir.y * newHeight)) < 0) {
 				newHeight = -wantedPos.y / dir.y;
@@ -141,15 +143,15 @@ void SmoothController::MouseWheelMove(float move)
 		// ZOOM OUT from mid screen
 		} else {
 			if (keys[SDLK_LALT]) { // instant-zoom: zoom out to the max
-				if(height<maxHeight*0.5f && changeAltHeight){
-					oldAltHeight=height;
-					changeAltHeight=false;
+				if(height < maxHeight*0.5f && changeAltHeight){
+					oldAltHeight = height;
+					changeAltHeight = false;
 				}
-				height=maxHeight;
-				pos.x=gs->mapx*4;
-				pos.z=gs->mapy*4.8f; // somewhat longer toward bottom
+				height = maxHeight;
+				pos.x  = gs->mapx * 4;
+				pos.z  = gs->mapy * 4.8f; // somewhat longer toward bottom
 			} else {
-				height*=1+move * mouseScale*0.7f * (keys[SDLK_LSHIFT] ? 3:1);
+				height *= 1 + move * 0.007f * (keys[SDLK_LSHIFT] ? 3:1);
 			}
 		}
 		// instant-zoom: turn on the smooth transition and reset the camera tilt
@@ -168,22 +170,11 @@ void SmoothController::Update()
 
 float3 SmoothController::GetPos()
 {
-	maxHeight = 9.5f * std::max(gs->mapx,gs->mapy);		//map not created when constructor run
+	maxHeight = 9.5f * std::max(gs->mapx, gs->mapy);	//map not created when constructor run
 
-	if (pos.x < 0.01f) { pos.x = 0.01f; }
-	if (pos.z < 0.01f) { pos.z = 0.01f; }
-	if (pos.x > ((gs->mapx * SQUARE_SIZE) - 0.01f)) {
-		pos.x = ((gs->mapx * SQUARE_SIZE) - 0.01f);
-	}
-	if (pos.z > ((gs->mapy * SQUARE_SIZE) - 0.01f)) {
-		pos.z = ((gs->mapy * SQUARE_SIZE) - 0.01f);
-	}
-	if (height < 60.0f) {
-		height = 60.0f;
-	}
-	if (height > maxHeight) {
-		height = maxHeight;
-	}
+	pos.x = Clamp(pos.x, 0.01f, gs->mapx * SQUARE_SIZE - 0.01f);
+	pos.z = Clamp(pos.z, 0.01f, gs->mapy * SQUARE_SIZE - 0.01f);
+	height = Clamp(height, 60.0f, maxHeight);
 
 	pos.y = ground->GetHeight(pos.x,pos.z);
 	dir = float3(0.0f, -1.0f, flipped ? zscale : -zscale).ANormalize();
