@@ -1,8 +1,7 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef PLAYER_H
 #define PLAYER_H
-// Player.h: interface for the CPlayer class.
-//
-//////////////////////////////////////////////////////////////////////
 
 #include <string>
 #include <set>
@@ -35,6 +34,8 @@ struct DirectControlClientState {
 		oldHeading = 0;
 		oldState   = 255;
 		oldDCpos   = ZeroVector;
+
+		playerControlledUnit = NULL;
 	}
 
 	void SendStateUpdate(bool*);
@@ -65,7 +66,7 @@ public:
 	void StartSpectating();
 	void GameFrame(int frameNum);
 
-	void operator=(const PlayerBase& base) { PlayerBase::operator=(base); };
+	CPlayer& operator=(const PlayerBase& base) { PlayerBase::operator=(base); return *this; };
 
 	bool active;
 
