@@ -1,6 +1,4 @@
-// SelectedUnits.cpp: implementation of the CSelectedUnits class.
-//
-//////////////////////////////////////////////////////////////////////
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "StdAfx.h"
 #include <map>
@@ -22,7 +20,6 @@
 #include "UI/GuiHandler.h"
 #include "UI/TooltipConsole.h"
 #include "LogOutput.h"
-#include "Rendering/UnitModels/3DOParser.h"
 #include "Rendering/GL/VertexArray.h"
 #include "SelectedUnitsAI.h"
 #include "Sim/Features/Feature.h"
@@ -37,7 +34,7 @@
 #include "ConfigHandler.h"
 #include "PlayerHandler.h"
 #include "Camera.h"
-#include "Sound/AudioChannel.h"
+#include "Sound/IEffectChannel.h"
 #include "Util.h"
 
 extern boost::uint8_t *keys;
@@ -634,6 +631,7 @@ void CSelectedUnits::DrawCommands()
 
 	glLineWidth(cmdColors.QueuedLineWidth());
 
+	GML_RECMUTEX_LOCK(unit); // DrawCommands
 	GML_RECMUTEX_LOCK(grpsel); // DrawCommands
 	GML_STDMUTEX_LOCK(cai); // DrawCommands
 
@@ -699,6 +697,10 @@ std::string CSelectedUnits::GetTooltip(void)
 	float exp = 0.0f, cost = 0.0f, range = 0.0f;
 	float metalMake = 0.0f, metalUse = 0.0f, energyMake = 0.0f, energyUse = 0.0f;
 
+#define NO_TEAM -32
+#define MULTI_TEAM -64
+	int ctrlTeam = NO_TEAM;
+
 	CUnitSet::iterator ui;
 	for (ui = selectedUnits.begin(); ui != selectedUnits.end(); ++ui) {
 		const CUnit* unit = *ui;
@@ -716,6 +718,10 @@ std::string CSelectedUnits::GetTooltip(void)
 		if (unit->unitDef->maxFuel > 0) {
 			numFuel++;
 		}
+		if(ctrlTeam == NO_TEAM)
+			ctrlTeam = unit->team;
+		else if(ctrlTeam != unit->team)
+			ctrlTeam = MULTI_TEAM;
 	}
 	if ((numFuel > 0) && (maxFuel > 0.0f)) {
 		curFuel = curFuel / numFuel;
@@ -730,13 +736,20 @@ std::string CSelectedUnits::GetTooltip(void)
 	       metalMake,  metalUse,
 	       energyMake, energyUse);
 
-  if (gs->cheatEnabled && (selectedUnits.size() == 1)) {
+  if (gs->cheatEnabled && (num == 1)) {
   	CUnit* unit = *selectedUnits.begin();
     SNPRINTF(tmp, sizeof(tmp), "\xff\xc0\xc0\xff  [TechLevel %i]",
              unit->unitDef->techLevel);
     s += tmp;
 	}
 
+	std::string ctrlName = "";
+	if(ctrlTeam == MULTI_TEAM)
+		ctrlName = "(Multiple teams)";
+	else if(ctrlTeam != NO_TEAM)
+		ctrlName = teamHandler->Team(ctrlTeam)->GetControllerName();
+	s += "\n\xff\xff\xff\xff" + ctrlName;
+
 	return s;
 }
 
