@@ -1,7 +1,12 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
+#ifdef _MSC_VER
 #include "StdAfx.h"
-// LuaUI.cpp: implementation of the CLuaUI class.
-//
-//////////////////////////////////////////////////////////////////////
+#endif
+#include "Lua/LuaLobby.h" // ugh, streflop namespace corruption...
+#ifndef _MSC_VER
+#include "StdAfx.h"
+#endif
 
 #include <stdio.h>
 #include <set>
@@ -9,17 +14,14 @@
 #include <SDL_keysym.h>
 #include <SDL_mouse.h>
 #include <SDL_timer.h>
-#include "Lua/LuaUnsyncedCtrl.h"
 
 #include "mmgr.h"
 
 #include "LuaUI.h"
 
-
-using namespace std;
-
 #include "LuaInclude.h"
 
+#include "Lua/LuaUnsyncedCtrl.h"
 #include "Lua/LuaCallInCheck.h"
 #include "Lua/LuaUtils.h"
 #include "Lua/LuaConstGL.h"
@@ -36,6 +38,7 @@ using namespace std;
 #include "Lua/LuaOpenGL.h"
 #include "Lua/LuaVFS.h"
 #include "Lua/LuaIO.h"
+#include "Lua/LuaZip.h"
 #include "Game/Camera.h"
 #include "Game/Camera/CameraController.h"
 #include "Game/Game.h"
@@ -54,7 +57,6 @@ using namespace std;
 #include "Map/ReadMap.h"
 #include "Rendering/IconHandler.h"
 #include "Rendering/InMapDraw.h"
-#include "Rendering/FontTexture.h"
 #include "EventHandler.h"
 #include "LogOutput.h"
 #include "FileSystem/FileHandler.h"
@@ -63,6 +65,7 @@ using namespace std;
 #include "FileSystem/FileSystem.h"
 #include "Util.h"
 
+using namespace std;
 
 #if (LUA_VERSION_NUM < 500)
 #  define LUA_OPEN_LIB(L, lib) lib(L)
@@ -163,7 +166,7 @@ CLuaUI::CLuaUI()
 		//lua_pushliteral(L, "rename");    lua_pushnil(L); lua_rawset(L, -3);
 		lua_pushliteral(L, "tmpname");   lua_pushnil(L); lua_rawset(L, -3);
 		lua_pushliteral(L, "getenv");    lua_pushnil(L); lua_rawset(L, -3);
-		lua_pushliteral(L, "setlocale"); lua_pushnil(L); lua_rawset(L, -3);
+		//lua_pushliteral(L, "setlocale"); lua_pushnil(L); lua_rawset(L, -3);
 	}
 	lua_pop(L, 1); // os
 
@@ -179,11 +182,14 @@ CLuaUI::CLuaUI()
 	// load the spring libraries
 	if (!LoadCFunctions(L)                                                 ||
 	    !AddEntriesToTable(L, "VFS",         LuaVFS::PushUnsynced)         ||
+	    !AddEntriesToTable(L, "VFS",       LuaZipFileReader::PushUnsynced) ||
+	    !AddEntriesToTable(L, "VFS",       LuaZipFileWriter::PushUnsynced) ||
 	    !AddEntriesToTable(L, "UnitDefs",    LuaUnitDefs::PushEntries)     ||
 	    !AddEntriesToTable(L, "WeaponDefs",  LuaWeaponDefs::PushEntries)   ||
 	    !AddEntriesToTable(L, "FeatureDefs", LuaFeatureDefs::PushEntries)  ||
 	    !AddEntriesToTable(L, "Script",      LuaUnsyncedCall::PushEntries) ||
 	    !AddEntriesToTable(L, "Script",      LuaScream::PushEntries)       ||
+	    !AddEntriesToTable(L, "Script",      LuaLobby::PushEntries)        ||
 	    !AddEntriesToTable(L, "Spring",      LuaSyncedRead::PushEntries)   ||
 	    !AddEntriesToTable(L, "Spring",      LuaUnsyncedCtrl::PushEntries) ||
 	    !AddEntriesToTable(L, "Spring",      LuaUnsyncedRead::PushEntries) ||
@@ -197,7 +203,6 @@ CLuaUI::CLuaUI()
 	}
 
 	lua_settop(L, 0);
-
 	if (!LoadCode(code, "luaui.lua")) {
 		KillLua();
 		return;
