@@ -1,7 +1,6 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
-// LuaRules.cpp: implementation of the CLuaRules class.
-//
-//////////////////////////////////////////////////////////////////////
 
 #include "Game/SelectedUnits.h"
 
@@ -40,10 +39,11 @@ using namespace std;
 #include "Map/BaseGroundDrawer.h"
 #include "Map/Ground.h"
 #include "Map/ReadMap.h"
+#include "Rendering/GlobalRendering.h"
 #include "Rendering/IconHandler.h"
 #include "Rendering/ShadowHandler.h"
+#include "Rendering/UnitDrawer.h"
 #include "Rendering/Env/BaseWater.h"
-#include "Rendering/UnitModels/UnitDrawer.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Features/FeatureDef.h"
 #include "Sim/Features/FeatureHandler.h"
@@ -58,7 +58,7 @@ using namespace std;
 #include "FileSystem/FileHandler.h"
 #include "FileSystem/VFSHandler.h"
 #include "FileSystem/FileSystem.h"
-#include "Sound/Music.h"
+#include "Sound/IMusicChannel.h"
 
 
 extern boost::uint8_t *keys;
@@ -111,6 +111,7 @@ bool LuaUnsyncedRead::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(GetUnitNoDraw);
 	REGISTER_LUA_CFUNC(GetUnitNoMinimap);
 	REGISTER_LUA_CFUNC(GetUnitNoSelect);
+	REGISTER_LUA_CFUNC(GetFeatureLuaDraw);
 
 	REGISTER_LUA_CFUNC(GetUnitTransformMatrix);
 	REGISTER_LUA_CFUNC(GetUnitViewPosition);
@@ -240,6 +241,24 @@ static inline CUnit* ParseUnit(lua_State* L, const char* caller, int index)
 	return unit;
 }
 
+static inline CFeature* ParseFeature(lua_State* L, const char* caller, int index)
+{
+	if (!lua_isnumber(L, index)) {
+		luaL_error(L, "%s(): Bad featureID", caller);
+		return NULL;
+	}
+	const int featureID = lua_toint(L, index);
+	CFeature* feature = featureHandler->GetFeature(featureID);
+
+	if (fullRead) { return feature; }
+	if (readAllyTeam < 0) { return NULL; }
+	if (feature == NULL) { return NULL; }
+	if (feature->IsInLosForAllyTeam(readAllyTeam)) { return feature; }
+
+	return NULL;
+}
+
+
 
 static inline void CheckNoArgs(lua_State* L, const char* funcName)
 {
@@ -281,10 +300,10 @@ int LuaUnsyncedRead::GetModUICtrl(lua_State* L)
 int LuaUnsyncedRead::GetViewGeometry(lua_State* L)
 {
 	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->viewSizeX);
-	lua_pushnumber(L, gu->viewSizeY);
-	lua_pushnumber(L, gu->viewPosX);
-	lua_pushnumber(L, gu->viewPosY);
+	lua_pushnumber(L, globalRendering->viewSizeX);
+	lua_pushnumber(L, globalRendering->viewSizeY);
+	lua_pushnumber(L, globalRendering->viewPosX);
+	lua_pushnumber(L, globalRendering->viewPosY);
 	return 4;
 }
 
@@ -292,10 +311,10 @@ int LuaUnsyncedRead::GetViewGeometry(lua_State* L)
 int LuaUnsyncedRead::GetWindowGeometry(lua_State* L)
 {
 	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->winSizeX);
-	lua_pushnumber(L, gu->winSizeY);
-	lua_pushnumber(L, gu->winPosX);
-	lua_pushnumber(L, gu->winPosY);
+	lua_pushnumber(L, globalRendering->winSizeX);
+	lua_pushnumber(L, globalRendering->winSizeY);
+	lua_pushnumber(L, globalRendering->winPosX);
+	lua_pushnumber(L, globalRendering->winPosY);
 	return 4;
 }
 
@@ -303,8 +322,8 @@ int LuaUnsyncedRead::GetWindowGeometry(lua_State* L)
 int LuaUnsyncedRead::GetScreenGeometry(lua_State* L)
 {
 	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->screenSizeX);
-	lua_pushnumber(L, gu->screenSizeY);
+	lua_pushnumber(L, globalRendering->screenSizeX);
+	lua_pushnumber(L, globalRendering->screenSizeY);
 	lua_pushnumber(L, 0.0f);
 	lua_pushnumber(L, 0.0f);
 	return 4;
@@ -332,10 +351,10 @@ int LuaUnsyncedRead::GetMiniMapDualScreen(lua_State* L)
 	if (minimap == NULL) {
 		return 0;
 	}
-	if (!gu->dualScreenMode) {
+	if (!globalRendering->dualScreenMode) {
 		lua_pushboolean(L, false);
 	} else {
-		if (gu->dualScreenMiniMapOnLeft) {
+		if (globalRendering->dualScreenMiniMapOnLeft) {
 			lua_pushliteral(L, "left");
 		} else {
 			lua_pushliteral(L, "right");
@@ -355,7 +374,7 @@ int LuaUnsyncedRead::IsAboveMiniMap(lua_State* L)
 		return false;
 	}
 
-	const int x = luaL_checkint(L, 1) + gu->viewPosX;
+	const int x = luaL_checkint(L, 1) + globalRendering->viewPosX;
 	const int y = luaL_checkint(L, 2);
 
 	const int x0 = minimap->GetPosX();
@@ -375,8 +394,8 @@ int LuaUnsyncedRead::IsAboveMiniMap(lua_State* L)
 int LuaUnsyncedRead::GetDrawFrame(lua_State* L)
 {
 	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->drawFrame & 0xFFFF);
-	lua_pushnumber(L, (gu->drawFrame >> 16) & 0xFFFF);
+	lua_pushnumber(L, globalRendering->drawFrame & 0xFFFF);
+	lua_pushnumber(L, (globalRendering->drawFrame >> 16) & 0xFFFF);
 	return 2;
 }
 
@@ -384,7 +403,7 @@ int LuaUnsyncedRead::GetDrawFrame(lua_State* L)
 int LuaUnsyncedRead::GetFrameTimeOffset(lua_State* L)
 {
 	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, gu->timeOffset);
+	lua_pushnumber(L, globalRendering->timeOffset);
 	return 1;
 }
 
@@ -530,7 +549,6 @@ int LuaUnsyncedRead::GetUnitLuaDraw(lua_State* L)
 	return 1;
 }
 
-
 int LuaUnsyncedRead::GetUnitNoDraw(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
@@ -541,7 +559,6 @@ int LuaUnsyncedRead::GetUnitNoDraw(lua_State* L)
 	return 1;
 }
 
-
 int LuaUnsyncedRead::GetUnitNoMinimap(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
@@ -552,7 +569,6 @@ int LuaUnsyncedRead::GetUnitNoMinimap(lua_State* L)
 	return 1;
 }
 
-
 int LuaUnsyncedRead::GetUnitNoSelect(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
@@ -564,6 +580,18 @@ int LuaUnsyncedRead::GetUnitNoSelect(lua_State* L)
 }
 
 
+int LuaUnsyncedRead::GetFeatureLuaDraw(lua_State* L)
+{
+	CFeature* feature = ParseFeature(L, __FUNCTION__, 1);
+	if (feature == NULL) {
+		return 0;
+	}
+	lua_pushboolean(L, feature->luaDraw);
+	return 1;
+}
+
+
+
 int LuaUnsyncedRead::GetUnitTransformMatrix(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
@@ -598,9 +626,9 @@ int LuaUnsyncedRead::GetUnitViewPosition(lua_State* L)
 	float3 pos = midPos ? (float3)unit->midPos : (float3)unit->pos;
 	CTransportUnit *trans=unit->GetTransporter();
 	if (trans == NULL) {
-		pos += (unit->speed * gu->timeOffset);
+		pos += (unit->speed * globalRendering->timeOffset);
 	} else {
-		pos += (trans->speed * gu->timeOffset);
+		pos += (trans->speed * globalRendering->timeOffset);
 	}
 
 	lua_pushnumber(L, pos.x);
@@ -1094,6 +1122,7 @@ int LuaUnsyncedRead::GetMapDrawMode(lua_State* L)
 		case CBaseGroundDrawer::drawMetal:  { HSTR_PUSH(L, "metal");  break; }
 		case CBaseGroundDrawer::drawPath:   { HSTR_PUSH(L, "path");   break; }
 		case CBaseGroundDrawer::drawLos:    { HSTR_PUSH(L, "los");    break; }
+		case CBaseGroundDrawer::drawHeat:   { HSTR_PUSH(L, "heat");   break; }
 	}
 	return 1;
 }
@@ -1216,11 +1245,11 @@ int LuaUnsyncedRead::TraceScreenRay(lua_State* L)
 	const bool onlyCoords = (lua_isboolean(L, 3) && lua_toboolean(L, 3));
 	const bool useMiniMap = (lua_isboolean(L, 4) && lua_toboolean(L, 4));
 
-	const int wx = mx + gu->viewPosX;
-	const int wy = gu->viewSizeY - 1 - my - gu->viewPosY;
+	const int wx = mx + globalRendering->viewPosX;
+	const int wy = globalRendering->viewSizeY - 1 - my - globalRendering->viewPosY;
 
 	if (useMiniMap && (minimap != NULL) && !minimap->GetMinimized()) {
-		const int px = minimap->GetPosX() - gu->viewPosX; // for left dualscreen
+		const int px = minimap->GetPosX() - globalRendering->viewPosX; // for left dualscreen
 		const int py = minimap->GetPosY();
 		const int sx = minimap->GetSizeX();
 		const int sy = minimap->GetSizeY();
@@ -1245,14 +1274,14 @@ int LuaUnsyncedRead::TraceScreenRay(lua_State* L)
 		}
 	}
 
-	if ((mx < 0) || (mx >= gu->viewSizeX) ||
-	    (my < 0) || (my >= gu->viewSizeY)) {
+	if ((mx < 0) || (mx >= globalRendering->viewSizeX) ||
+	    (my < 0) || (my >= globalRendering->viewSizeY)) {
 		return 0;
 	}
 
 	const CUnit* unit = NULL;
 	const CFeature* feature = NULL;
-	const float range = gu->viewRange * 1.4f;
+	const float range = globalRendering->viewRange * 1.4f;
 	const float3& pos = camera->pos;
 	const float3 dir = camera->CalcPixelDir(wx, wy);
 
@@ -1651,8 +1680,8 @@ int LuaUnsyncedRead::GetActivePage(lua_State* L)
 int LuaUnsyncedRead::GetMouseState(lua_State* L)
 {
 	CheckNoArgs(L, __FUNCTION__);
-	lua_pushnumber(L, mouse->lastx - gu->viewPosX);
-	lua_pushnumber(L, gu->viewSizeY - mouse->lasty - 1);
+	lua_pushnumber(L, mouse->lastx - globalRendering->viewPosX);
+	lua_pushnumber(L, globalRendering->viewSizeY - mouse->lasty - 1);
 	lua_pushboolean(L, mouse->buttons[SDL_BUTTON_LEFT].pressed);
 	lua_pushboolean(L, mouse->buttons[SDL_BUTTON_MIDDLE].pressed);
 	lua_pushboolean(L, mouse->buttons[SDL_BUTTON_RIGHT].pressed);
@@ -2179,7 +2208,3 @@ int LuaUnsyncedRead::GetDrawSelectionInfo(lua_State* L)
 
 /******************************************************************************/
 /******************************************************************************/
-
-
-
-
