@@ -1,6 +1,6 @@
-/* Author: Tobi Vollebregt */
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "StdAfx.h"     // ugh
+#include "StdAfx.h" // TODO: ugh
 #include <assert.h>
 #include "SmfMapFile.h"
 #include "mapfile.h"
@@ -11,7 +11,7 @@ using std::string;
 
 
 CSmfMapFile::CSmfMapFile(const string& mapname)
-	: ifs(string("maps/") + mapname), featureFileOffset(0)
+	: ifs(mapname), featureFileOffset(0)
 {
 	memset(&header, 0, sizeof(header));
 	memset(&featureHeader, 0, sizeof(featureHeader));
