@@ -1,25 +1,24 @@
-// DrawWater.cpp: implementation of the CAdvWater class.
-//
-//////////////////////////////////////////////////////////////////////
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "StdAfx.h"
 #include "mmgr.h"
 
 #include "AdvWater.h"
+#include "BaseSky.h"
 #include "Game/Game.h"
 #include "Game/Camera.h"
-#include "Rendering/GL/VertexArray.h"
-#include "Map/ReadMap.h"
-#include "LogOutput.h"
 #include "Map/BaseGroundDrawer.h"
-#include "BaseSky.h"
-#include "Rendering/UnitModels/FeatureDrawer.h"
-#include "Rendering/UnitModels/UnitDrawer.h"
-#include "Sim/Projectiles/ProjectileHandler.h"
-#include "GlobalUnsynced.h"
-#include "EventHandler.h"
 #include "Map/MapInfo.h"
-#include "Exceptions.h"
+#include "Map/ReadMap.h"
+#include "Rendering/GlobalRendering.h"
+#include "Rendering/GL/VertexArray.h"
+#include "Rendering/FeatureDrawer.h"
+#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/UnitDrawer.h"
+#include "System/EventHandler.h"
+#include "System/Exceptions.h"
+#include "System/GlobalUnsynced.h"
+#include "System/LogOutput.h"
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -90,7 +89,7 @@ CAdvWater::CAdvWater(bool loadShader)
 	delete[] scrap;
 
 	if (loadShader)
-		waterFP=LoadFragmentProgram("water.fp");
+		waterFP=LoadFragmentProgram("ARB/water.fp");
 
 	waterSurfaceColor = mapInfo->water.surfaceColor;
 
@@ -124,9 +123,9 @@ void CAdvWater::Draw(bool useBlending)
 		return;
 
 	float3 dir,zpos;
-	float3 base=camera->CalcPixelDir(gu->viewPosX,gu->viewSizeY);
-	float3 dv=camera->CalcPixelDir(gu->viewPosX,0)-camera->CalcPixelDir(gu->viewPosX,gu->viewSizeY);
-	float3 dh=camera->CalcPixelDir(gu->viewPosX+gu->viewSizeX,0)-camera->CalcPixelDir(gu->viewPosX,0);
+	float3 base=camera->CalcPixelDir(globalRendering->viewPosX,globalRendering->viewSizeY);
+	float3 dv=camera->CalcPixelDir(globalRendering->viewPosX,0)-camera->CalcPixelDir(globalRendering->viewPosX,globalRendering->viewSizeY);
+	float3 dh=camera->CalcPixelDir(globalRendering->viewPosX+globalRendering->viewSizeX,0)-camera->CalcPixelDir(globalRendering->viewPosX,0);
 
 	float3 xbase;
 	const int numDivs=20;
@@ -315,7 +314,7 @@ void CAdvWater::UpdateWater(CGame* game)
 	glViewport(0, 0, 512, 512);
 	glClear(GL_DEPTH_BUFFER_BIT);
 
-	game->SetDrawMode(CGame::reflectionDraw);
+	game->SetDrawMode(CGame::gameReflectionDraw);
 
 	sky->Draw();
 
@@ -327,19 +326,19 @@ void CAdvWater::UpdateWater(CGame* game)
 	readmap->GetGroundDrawer()->Draw(true);
 	unitDrawer->Draw(true);
 	featureDrawer->Draw();
-	unitDrawer->DrawCloakedUnits(false,true);
-	featureDrawer->DrawFadeFeatures(false,true);
-	ph->Draw(true);
+	unitDrawer->DrawCloakedUnits(true);
+	featureDrawer->DrawFadeFeatures(true);
+	projectileDrawer->Draw(true);
 	eventHandler.DrawWorldReflection();
 
-	game->SetDrawMode(CGame::normalDraw);
+	game->SetDrawMode(CGame::gameNormalDraw);
 
 	drawReflection=false;
 	glDisable(GL_CLIP_PLANE2);
 
 	FBO::Unbind();
 
-	glViewport(gu->viewPosX,0,gu->viewSizeX,gu->viewSizeY);
+	glViewport(globalRendering->viewPosX,0,globalRendering->viewSizeX,globalRendering->viewSizeY);
 	glClearColor(mapInfo->atmosphere.fogColor[0],mapInfo->atmosphere.fogColor[1],mapInfo->atmosphere.fogColor[2],1);
 
 //	delete camera;
