@@ -1,29 +1,32 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
 #include "DynWater.h"
 #include "Game/Game.h"
 #include "Game/Camera.h"
-#include "Rendering/GL/VertexArray.h"
 #include "Map/MapInfo.h"
 #include "Map/ReadMap.h"
-#include "LogOutput.h"
-#include "bitops.h"
-#include "Map/BaseGroundDrawer.h"
-#include "BaseSky.h"
-#include "Rendering/UnitModels/FeatureDrawer.h"
-#include "Rendering/UnitModels/UnitDrawer.h"
-#include "Sim/Projectiles/ProjectileHandler.h"
 #include "Game/UI/MouseHandler.h"
 #include "Game/GameHelper.h"
+#include "Map/BaseGroundDrawer.h"
+#include "Rendering/GlobalRendering.h"
+#include "Rendering/FeatureDrawer.h"
+#include "Rendering/ProjectileDrawer.hpp"
+#include "Rendering/UnitDrawer.h"
 #include "Rendering/ShadowHandler.h"
+#include "Rendering/Env/BaseSky.h"
+#include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/Bitmap.h"
 #include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
-#include "GlobalUnsynced.h"
-#include "EventHandler.h"
-#include "Exceptions.h"
+#include "System/LogOutput.h"
+#include "System/bitops.h"
+#include "System/GlobalUnsynced.h"
+#include "System/EventHandler.h"
+#include "System/Exceptions.h"
 
 #define W_SIZE 5
 #define WF_SIZE 5120
@@ -43,7 +46,7 @@ CDynWater::CDynWater(void)
 	firstDraw=true;
 	drawSolid=true;
 	camPosBig=float3(2048,0,2048);
-	refractSize=gu->viewSizeY>=1024 ? 1024:512;
+	refractSize=globalRendering->viewSizeY>=1024 ? 1024:512;
 
 	glGenTextures(1, &reflectTexture);
 	glBindTexture(GL_TEXTURE_2D, reflectTexture);
@@ -101,26 +104,24 @@ CDynWater::CDynWater(void)
 
 	delete[] scrap;
 
-	if(ProgramStringIsNative(GL_VERTEX_PROGRAM_ARB,"waterDyn.vp"))
-		waterVP=LoadVertexProgram("waterDyn.vp");
+	if (ProgramStringIsNative(GL_VERTEX_PROGRAM_ARB, "ARB/waterDyn.vp"))
+		waterVP = LoadVertexProgram("ARB/waterDyn.vp");
 	else
-		waterVP=LoadVertexProgram("waterDynNT.vp");
+		waterVP = LoadVertexProgram("ARB/waterDynNT.vp");
 
-	waterFP=LoadFragmentProgram("waterDyn.fp");
-	waveFP=LoadFragmentProgram("waterDynWave.fp");
-	waveVP=LoadVertexProgram("waterDynWave.vp");
-	waveFP2=LoadFragmentProgram("waterDynWave2.fp");
-	waveVP2=LoadVertexProgram("waterDynWave2.vp");
-	waveNormalFP=LoadFragmentProgram("waterDynNormal.fp");
-	waveNormalVP=LoadVertexProgram("waterDynNormal.vp");
-	waveCopyHeightFP=LoadFragmentProgram("waterDynWave3.fp");
-	waveCopyHeightVP=LoadVertexProgram("waterDynWave3.vp");
-	dwGroundRefractVP=LoadVertexProgram("dwgroundrefract.vp");
-	dwGroundReflectIVP=LoadVertexProgram("dwgroundreflectinverted.vp");
-	dwDetailNormalFP=LoadFragmentProgram("dwDetailNormal.fp");
-	dwDetailNormalVP=LoadVertexProgram("dwDetailNormal.vp");
-	dwAddSplashFP=LoadFragmentProgram("dwAddSplash.fp");
-	dwAddSplashVP=LoadVertexProgram("dwAddSplash.vp");
+	waterFP          = LoadFragmentProgram("ARB/waterDyn.fp");
+	waveFP           = LoadFragmentProgram("ARB/waterDynWave.fp");
+	waveVP           = LoadVertexProgram("ARB/waterDynWave.vp");
+	waveFP2          = LoadFragmentProgram("ARB/waterDynWave2.fp");
+	waveVP2          = LoadVertexProgram("ARB/waterDynWave2.vp");
+	waveNormalFP     = LoadFragmentProgram("ARB/waterDynNormal.fp");
+	waveNormalVP     = LoadVertexProgram("ARB/waterDynNormal.vp");
+	waveCopyHeightFP = LoadFragmentProgram("ARB/waterDynWave3.fp");
+	waveCopyHeightVP = LoadVertexProgram("ARB/waterDynWave3.vp");
+	dwDetailNormalFP = LoadFragmentProgram("ARB/dwDetailNormal.fp");
+	dwDetailNormalVP = LoadVertexProgram("ARB/dwDetailNormal.vp");
+	dwAddSplashFP    = LoadFragmentProgram("ARB/dwAddSplash.fp");
+	dwAddSplashVP    = LoadVertexProgram("ARB/dwAddSplash.vp");
 
 	waterSurfaceColor = mapInfo->water.surfaceColor;
 
@@ -269,8 +270,6 @@ CDynWater::~CDynWater(void)
 	glSafeDeleteProgram( waveNormalVP );
 	glSafeDeleteProgram( waveCopyHeightFP );
 	glSafeDeleteProgram( waveCopyHeightVP );
-	glSafeDeleteProgram( dwGroundReflectIVP );
-	glSafeDeleteProgram( dwGroundRefractVP );
 	glSafeDeleteProgram( dwDetailNormalVP );
 	glSafeDeleteProgram( dwDetailNormalFP );
 	glSafeDeleteProgram( dwAddSplashVP );
@@ -316,8 +315,8 @@ void CDynWater::Draw()
 	glBindProgramARB( GL_VERTEX_PROGRAM_ARB, waterVP );
 	glEnable( GL_VERTEX_PROGRAM_ARB );
 
-	float dx=float(gu->viewSizeX)/gu->viewSizeY * camera->GetTanHalfFov();
-	float dy=float(gu->viewSizeY)/gu->viewSizeY * camera->GetTanHalfFov();
+	float dx=float(globalRendering->viewSizeX)/globalRendering->viewSizeY * camera->GetTanHalfFov();
+	float dy=float(globalRendering->viewSizeY)/globalRendering->viewSizeY * camera->GetTanHalfFov();
 
 	glProgramEnvParameter4fARB(GL_VERTEX_PROGRAM_ARB, 10, 1.0f/(W_SIZE*256),1.0f/(W_SIZE*256), 0, 0);
 	glProgramEnvParameter4fARB(GL_VERTEX_PROGRAM_ARB, 11, -camPosX/256.0f+0.5f,-camPosZ/256.0f+0.5f, 0, 0);
@@ -446,7 +445,7 @@ void CDynWater::DrawReflection(CGame* game)
 	glClearColor(0.5f,0.6f,0.8f,0);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
-	game->SetDrawMode(CGame::reflectionDraw);
+	game->SetDrawMode(CGame::gameReflectionDraw);
 
 	sky->Draw();
 
@@ -457,8 +456,10 @@ void CDynWater::DrawReflection(CGame* game)
 	bool drawShadows=shadowHandler->drawShadows;
 	shadowHandler->drawShadows=false;
 
-	CBaseGroundDrawer *gd = readmap->GetGroundDrawer ();
-	gd->Draw(true,false,dwGroundReflectIVP);
+	CBaseGroundDrawer* gd = readmap->GetGroundDrawer();
+		gd->SetupReflDrawPass();
+		gd->Draw(true, false);
+		gd->SetupBaseDrawPass();
 
 	double plane[4]={0,1,0,1.0f};
 	glClipPlane(GL_CLIP_PLANE2 ,plane);
@@ -469,20 +470,20 @@ void CDynWater::DrawReflection(CGame* game)
 
 	unitDrawer->Draw(true);
 	featureDrawer->Draw();
-	unitDrawer->DrawCloakedUnits(false,true);
-	featureDrawer->DrawFadeFeatures(false,true);
+	unitDrawer->DrawCloakedUnits(true);
+	featureDrawer->DrawFadeFeatures(true);
 
-	ph->Draw(true);
+	projectileDrawer->Draw(true);
 	eventHandler.DrawWorldReflection();
 
 	sky->DrawSun();
 
-	game->SetDrawMode(CGame::normalDraw);
+	game->SetDrawMode(CGame::gameNormalDraw);
 
 	drawReflection=false;
 	glDisable(GL_CLIP_PLANE2);
 
-	glViewport(gu->viewPosX,0,gu->viewSizeX,gu->viewSizeY);
+	glViewport(globalRendering->viewPosX,0,globalRendering->viewSizeX,globalRendering->viewSizeY);
 	glClearColor(mapInfo->atmosphere.fogColor[0],mapInfo->atmosphere.fogColor[1],mapInfo->atmosphere.fogColor[2],1);
 
 //	delete camera;
@@ -514,10 +515,12 @@ void CDynWater::DrawRefraction(CGame* game)
 	unitDrawer->unitSunColor*=float3(0.5f,0.7f,0.9f);
 	unitDrawer->unitAmbientColor*=float3(0.6f,0.8f,1.0f);
 
-	game->SetDrawMode(CGame::refractionDraw);
+	game->SetDrawMode(CGame::gameRefractionDraw);
 
-	CBaseGroundDrawer *gd = readmap->GetGroundDrawer();
-	gd->Draw(false,false,dwGroundRefractVP);
+	CBaseGroundDrawer* gd = readmap->GetGroundDrawer();
+		gd->SetupRefrDrawPass();
+		gd->Draw(false, false);
+		gd->SetupBaseDrawPass();
 
 	glEnable(GL_CLIP_PLANE2);
 	double plane[4]={0,-1,0,2};
@@ -525,19 +528,19 @@ void CDynWater::DrawRefraction(CGame* game)
 	drawReflection=true;
 	unitDrawer->Draw(false,true);
 	featureDrawer->Draw();
-	unitDrawer->DrawCloakedUnits(true,true);
-	featureDrawer->DrawFadeFeatures(true,true); // FIXME: Make it fade out correctly without "noAdvShading"
+	unitDrawer->DrawCloakedUnits(true);
+	featureDrawer->DrawFadeFeatures(true); // FIXME: Make it fade out correctly without "noAdvShading"
 	drawReflection=false;
-	ph->Draw(false,true);
+	projectileDrawer->Draw(false, true);
 	eventHandler.DrawWorldRefraction();
 	glDisable(GL_CLIP_PLANE2);
 
-	game->SetDrawMode(CGame::normalDraw);
+	game->SetDrawMode(CGame::gameNormalDraw);
 
 	drawRefraction=false;
 
 
-	glViewport(gu->viewPosX,0,gu->viewSizeX,gu->viewSizeY);
+	glViewport(globalRendering->viewPosX,0,globalRendering->viewSizeX,globalRendering->viewSizeY);
 	glClearColor(mapInfo->atmosphere.fogColor[0],mapInfo->atmosphere.fogColor[1],mapInfo->atmosphere.fogColor[2],1);
 
 	unitDrawer->unitSunColor=oldsun;
@@ -840,9 +843,9 @@ void CDynWater::UpdateCamRestraints(void)
 		float3 colpoint;				//a point on the collision line
 
 		if(side.y>0)
-			colpoint=cam2->pos+camHorizontal*gu->viewRange*1.05f-c*(cam2->pos.y/c.y);
+			colpoint=cam2->pos+camHorizontal*globalRendering->viewRange*1.05f-c*(cam2->pos.y/c.y);
 		else
-			colpoint=cam2->pos+camHorizontal*gu->viewRange*1.05f-c*((cam2->pos.y-255/3.5f)/c.y);
+			colpoint=cam2->pos+camHorizontal*globalRendering->viewRange*1.05f-c*((cam2->pos.y-255/3.5f)/c.y);
 
 
 		temp.base=colpoint.x-colpoint.z*temp.dir;	//get intersection between colpoint and z axis
@@ -1141,48 +1144,57 @@ void CDynWater::AddShipWakes()
 	{
 		GML_RECMUTEX_LOCK(unit); // AddShipWakes
 
-		int nadd=uh->renderUnits.size()*4;
-		va->EnlargeArrays(nadd,0,VA_SIZE_TN);
-		va2->EnlargeArrays(nadd,0,VA_SIZE_TN);
+		const std::set<CUnit*>& units = unitDrawer->GetUnsortedUnits();
+		const int nadd = units.size() * 4;
 
-		for(std::list<CUnit*>::iterator ui=uh->renderUnits.begin(); ui!=uh->renderUnits.end();++ui){
-			CUnit* unit=*ui;
-			if(unit->moveType && unit->mobility) {
-				if(unit->unitDef->canhover){	//hover
-					float3 pos=unit->pos;
-					if(fabs(pos.x-camPosBig.x)>WH_SIZE-50 || fabs(pos.z-camPosBig.z)>WH_SIZE-50)
-						continue;
-					if(!(unit->losStatus[gu->myAllyTeam] & LOS_INLOS) && !gu->spectatingFullView)
-						continue;
-					if(pos.y>-4 && pos.y<4){
-						float3 frontAdd=unit->frontdir*unit->radius*0.75f;
-						float3 sideAdd=unit->rightdir*unit->radius*0.75f;
-						float depth=sqrt(sqrt(unit->mass))*0.4f;
-						float3 n(depth, 0.05f*depth, depth);
+		va->EnlargeArrays(nadd, 0, VA_SIZE_TN);
+		va2->EnlargeArrays(nadd, 0, VA_SIZE_TN);
 
-						va2->AddVertexQTN(pos+frontAdd+sideAdd,0,0,n);
-						va2->AddVertexQTN(pos+frontAdd-sideAdd,1,0,n);
-						va2->AddVertexQTN(pos-frontAdd-sideAdd,1,1,n);
-						va2->AddVertexQTN(pos-frontAdd+sideAdd,0,1,n);
+		for (std::set<CUnit*>::const_iterator ui = units.begin(); ui != units.end(); ++ui) {
+			CUnit* unit = *ui;
+
+			if (unit->moveType && unit->mobility) {
+				if (unit->unitDef->canhover) {
+					// hovercraft
+					const float3& pos = unit->pos;
+
+					if ((fabs(pos.x - camPosBig.x) > WH_SIZE - 50) || (fabs(pos.z - camPosBig.z) > WH_SIZE - 50))
+						continue;
+					if (!(unit->losStatus[gu->myAllyTeam] & LOS_INLOS) && !gu->spectatingFullView)
+						continue;
+
+					if (pos.y > -4.0f && pos.y < 4.0f) {
+						const float3 frontAdd = unit->frontdir * unit->radius * 0.75f;
+						const float3 sideAdd = unit->rightdir * unit->radius * 0.75f;
+						const float depth = sqrt(sqrt(unit->mass)) * 0.4f;
+						const float3 n(depth, 0.05f * depth, depth);
+
+						va2->AddVertexQTN(pos + frontAdd + sideAdd, 0, 0, n);
+						va2->AddVertexQTN(pos + frontAdd - sideAdd, 1, 0, n);
+						va2->AddVertexQTN(pos - frontAdd - sideAdd, 1, 1, n);
+						va2->AddVertexQTN(pos - frontAdd + sideAdd, 0, 1, n);
 					}
 				}
-				else if(unit->floatOnWater){	//boat
-					float speedf=unit->speed.Length2D();
-					float3 pos=unit->pos;
-					if(fabs(pos.x-camPosBig.x)>WH_SIZE-50 || fabs(pos.z-camPosBig.z)>WH_SIZE-50)
-						continue;
-					if(!(unit->losStatus[gu->myAllyTeam] & LOS_INLOS) && !gu->spectatingFullView)
-						continue;
-					if(pos.y>-4 && pos.y<1){
-						float3 frontAdd=unit->frontdir*unit->radius*0.75f;
-						float3 sideAdd=unit->rightdir*unit->radius*0.18f;
-						float depth=sqrt(sqrt(unit->mass));
-						float3 n(depth, 0.04f*speedf*depth, depth);
+				else if (unit->floatOnWater) {
+					// surface ship
+					const float speedf = unit->speed.Length2D();
+					const float3& pos = unit->pos;
 
-						va->AddVertexQTN(pos+frontAdd+sideAdd,0,0,n);
-						va->AddVertexQTN(pos+frontAdd-sideAdd,1,0,n);
-						va->AddVertexQTN(pos-frontAdd-sideAdd,1,1,n);
-						va->AddVertexQTN(pos-frontAdd+sideAdd,0,1,n);
+					if ((fabs(pos.x - camPosBig.x) > WH_SIZE - 50) || (fabs(pos.z - camPosBig.z) > WH_SIZE - 50))
+						continue;
+					if (!(unit->losStatus[gu->myAllyTeam] & LOS_INLOS) && !gu->spectatingFullView)
+						continue;
+
+					if (pos.y > -4.0f && pos.y < 1.0f) {
+						const float3 frontAdd = unit->frontdir * unit->radius * 0.75f;
+						const float3 sideAdd = unit->rightdir * unit->radius * 0.18f;
+						const float depth = sqrt(sqrt(unit->mass));
+						const float3 n(depth, 0.04f * speedf * depth, depth);
+
+						va->AddVertexQTN(pos + frontAdd + sideAdd, 0, 0, n);
+						va->AddVertexQTN(pos + frontAdd - sideAdd, 1, 0, n);
+						va->AddVertexQTN(pos - frontAdd - sideAdd, 1, 1, n);
+						va->AddVertexQTN(pos - frontAdd + sideAdd, 0, 1, n);
 					}
 				}
 			}
@@ -1190,12 +1202,12 @@ void CDynWater::AddShipWakes()
 	}
 
 	glActiveTextureARB(GL_TEXTURE0_ARB);
-	glBindTexture(GL_TEXTURE_2D,boatShape);
+	glBindTexture(GL_TEXTURE_2D, boatShape);
 
 	va->DrawArrayTN(GL_QUADS);
 
 	glActiveTextureARB(GL_TEXTURE0_ARB);
-	glBindTexture(GL_TEXTURE_2D,hoverShape);
+	glBindTexture(GL_TEXTURE_2D, hoverShape);
 
 	va2->DrawArrayTN(GL_QUADS);
 
