@@ -1,16 +1,15 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef S3OTEXTUREHANDLER_H
 #define S3OTEXTUREHANDLER_H
-// S3OTextureHandler.h: interface for the CS3OTextureHandler class.
-//
-//////////////////////////////////////////////////////////////////////
 
 #include <map>
 #include <string>
 #include <vector>
 #include "Rendering/GL/myGL.h"
-#include "Rendering/UnitModels/3DModel.h"
 
 struct TexFile;
+struct S3DModel;
 class CFileHandler;
 
 class CS3OTextureHandler
@@ -29,7 +28,6 @@ public:
 	CS3OTextureHandler();
 	virtual ~CS3OTextureHandler();
 
-	std::vector<S3DModel* > loadTextures;
 	void Update();
 	void LoadS3OTexture(S3DModel* model);
 	int LoadS3OTextureNow(const std::string& tex1, const std::string& tex2);
