@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
@@ -12,9 +14,7 @@
 #include "Sim/Misc/DamageArray.h"
 #include "Sim/Misc/QuadField.h"
 #include "Rendering/Env/BaseTreeDrawer.h"
-#include "Rendering/UnitModels/3DOParser.h"
-#include "Rendering/UnitModels/FeatureDrawer.h"
-#include "Rendering/UnitModels/UnitDrawer.h"
+#include "Rendering/Models/3DModel.h"
 #include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/ModInfo.h"
 #include "Sim/Misc/TeamHandler.h"
@@ -24,6 +24,7 @@
 #include "Sim/Projectiles/Unsynced/SmokeProjectile.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/Unit.h"
+#include "System/EventHandler.h"
 #include "GlobalUnsynced.h"
 #include <assert.h>
 
@@ -36,8 +37,7 @@ CR_REG_METADATA(CFeature, (
 				CR_MEMBER(resurrectProgress),
 				CR_MEMBER(health),
 				CR_MEMBER(reclaimLeft),
-				CR_MEMBER(allyteam),
-				CR_MEMBER(team),
+				CR_MEMBER(luaDraw),
 				CR_MEMBER(noSelect),
 				CR_MEMBER(tempNum),
 				CR_MEMBER(lastReclaim),
@@ -56,13 +56,11 @@ CR_REG_METADATA(CFeature, (
 
 
 CFeature::CFeature():
-	model(NULL),
 	isRepairingBeforeResurrect(false),
 	resurrectProgress(0),
 	health(0),
 	reclaimLeft(1),
-	allyteam(0),
-	team(0),
+	luaDraw(false),
 	noSelect(false),
 	tempNum(0),
 	lastReclaim(0),
@@ -75,9 +73,10 @@ CFeature::CFeature():
 	myFire(0),
 	fireTime(0),
 	emitSmokeTime(0),
-	solidOnTop(0)
+	solidOnTop(0),
+	tempalpha(1)
 {
-	immobile=true;
+	immobile = true;
 	physicalState = OnGround;
 }
 
@@ -119,10 +118,6 @@ void CFeature::PostLoad()
 	} else {
 		midPos = pos;
 	}
-
-	if (def->drawType >= DRAWTYPE_TREE) {
-		treeDrawer->AddTree(def->drawType - 1, pos, 1);
-	}
 }
 
 
@@ -204,11 +199,6 @@ void CFeature::Initialize(const float3& _pos, const FeatureDef* _def, short int
 		finalHeight = ground->GetHeight2(pos.x, pos.z);
 	}
 
-	if (def->drawType >= DRAWTYPE_TREE) {
-		treeDrawer->AddTree(def->drawType - 1, pos, 1);
-	}
-
-
 	if (speed != ZeroVector) {
 		deathSpeed = speed;
 	}
@@ -427,7 +417,7 @@ void CFeature::ForcedMove(const float3& newPos, bool snapToGround)
 
 	pos = newPos;
 
-	featureDrawer->UpdateDrawPos(this);
+	eventHandler.FeatureMoved(this);
 
 	// setup finalHeight
 	if (snapToGround) {
@@ -466,15 +456,6 @@ void CFeature::ForcedMove(const float3& newPos, bool snapToGround)
 
 void CFeature::ForcedSpin(const float3& newDir)
 {
-/*
-	heading = GetHeadingFromVector(newDir.x, newDir.z);
-	CalculateTransform();
-	if (def->drawType >= DRAWTYPE_TREE) {
-		treeDrawer->DeleteTree(pos);
-		treeDrawer->AddTree(def->drawType - 1, pos, 1.0f);
-	}
-*/
-
 	float3 updir = UpVector;
 	if (updir == newDir) {
 		//FIXME perhaps save the old right,up,front directions, so we can
@@ -556,14 +537,14 @@ bool CFeature::UpdatePosition()
 				deathSpeed = ZeroVector;
 			}
 
-			featureDrawer->UpdateDrawPos(this);
+			eventHandler.FeatureMoved(this);
 
 			CalculateTransform();
 		}
 
 		if (!reachedFinalPos)
 			finishedUpdate = false;
-	}else{
+	} else {
 		if (pos.y > finalHeight) {
 			//! feature is falling
 			if (def->drawType >= DRAWTYPE_TREE)
@@ -714,10 +695,3 @@ float CFeature::RemainingEnergy() const
 {
 	return RemainingResource(def->energy);
 }
-
-
-
-void CFeature::DrawS3O()
-{
-	unitDrawer->DrawFeatureStatic(this);
-}
