@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
@@ -8,7 +10,7 @@
 #include "Map/Ground.h"
 #include "Map/MapInfo.h"
 #include "myMath.h"
-#include "Rendering/UnitModels/3DModel.h"
+#include "Rendering/Models/3DModel.h"
 #include "Sim/Misc/GeometricObjects.h"
 #include "Sim/Misc/GroundBlockingObjectMap.h"
 #include "Sim/Misc/LosHandler.h"
@@ -149,11 +151,13 @@ CAirMoveType::~CAirMoveType(void)
 
 void CAirMoveType::Update(void)
 {
-	if (owner->beingBuilt)
-		return;
-
 	float3& pos = owner->pos;
 
+	if (owner->stunned || owner->beingBuilt) {
+		UpdateAirPhysics(0, lastAileronPos, lastElevatorPos, 0, ZeroVector);
+		goto EndNormalControl;
+	}
+
 	// note: this is only set to false after
 	// the plane has finished constructing
 	if (useHeading) {
@@ -161,12 +165,6 @@ void CAirMoveType::Update(void)
 		SetState(AIRCRAFT_TAKEOFF);
 	}
 
-	if (owner->stunned) {
-		UpdateAirPhysics(0, lastAileronPos, lastElevatorPos, 0, ZeroVector);
-		goto EndNormalControl;
-	}
-
-
 	if (owner->directControl && !(aircraftState == AIRCRAFT_CRASHING)) {
 		SetState(AIRCRAFT_FLYING);
 		DirectControlStruct* dc = owner->directControl;
@@ -338,8 +336,8 @@ EndNormalControl:
 						owner->UpdateMidPos();
 						owner->speed *= 0.99f;
 						float damage = (((*ui)->speed - owner->speed) * 0.1f).SqLength();
-						owner->DoDamage(DamageArray() * damage, 0, ZeroVector);
-						(*ui)->DoDamage(DamageArray() * damage, 0, ZeroVector);
+						owner->DoDamage(DamageArray(damage), 0, ZeroVector);
+						(*ui)->DoDamage(DamageArray(damage), 0, ZeroVector);
 						hitBuilding = true;
 					} else {
 						float part = owner->mass / (owner->mass + (*ui)->mass);
@@ -349,8 +347,8 @@ EndNormalControl:
 						u->pos += dif * (dist - totRad) * (part);
 						u->UpdateMidPos();
 						float damage = (((*ui)->speed - owner->speed) * 0.1f).SqLength();
-						owner->DoDamage(DamageArray() * damage, 0, ZeroVector);
-						(*ui)->DoDamage(DamageArray() * damage, 0, ZeroVector);
+						owner->DoDamage(DamageArray(damage), 0, ZeroVector);
+						(*ui)->DoDamage(DamageArray(damage), 0, ZeroVector);
 						owner->speed *= 0.99f;
 					}
 				}
@@ -423,8 +421,7 @@ void CAirMoveType::SlowUpdate(void)
 			float h = owner->pos.y - ground->GetApproximateHeight(owner->pos.x, owner->pos.z);
 			owner->losHeight = h + 5;
 			loshandler->MoveUnit(owner, false);
-			if (owner->hasRadarCapacity)
-				radarhandler->MoveUnit(owner);
+			radarhandler->MoveUnit(owner);
 
 			owner->losHeight = oldlh;
 		}
@@ -1064,7 +1061,7 @@ void CAirMoveType::UpdateAirPhysics(float rudder, float aileron, float elevator,
 					damage += (1 - (updir.dot(gNormal))) * 1000;
 				if (damage > 0) {
 					// only do damage while stunned for now
-					owner->DoDamage(DamageArray() * (damage * 0.4f), 0, ZeroVector);
+					owner->DoDamage(DamageArray(damage * 0.4f), 0, ZeroVector);
 				}
 			}
 			pos.y = gHeight + owner->model->radius * 0.2f + 0.01f;
