@@ -1,6 +1,4 @@
-// WorldObject.cpp: implementation of the CWorldObject class.
-//
-//////////////////////////////////////////////////////////////////////
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "StdAfx.h"
 #include "mmgr.h"
@@ -24,11 +22,6 @@ CWorldObject::~CWorldObject()
 {
 }
 
-void CWorldObject::DrawS3O()
-{
-	// implemented by CUnit, CFeature, CWeaponProjectile, CPieceProjectile
-}
-
 void CWorldObject::SetRadius(float r)
 {
 	radius = r;
