@@ -1,28 +1,27 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "PathEstimator.h"
+
 #include <fstream>
 #include <boost/bind.hpp>
 #include <boost/version.hpp>
-#include "lib/minizip/zip.h"
-#include "mmgr.h"
-
 #include <boost/version.hpp>
 
-#include "LogOutput.h"
-#include "Rendering/GL/myGL.h"
-#include "FileSystem/FileHandler.h"
-#include "ConfigHandler.h"
-
-#include "Map/Ground.h"
-#include "Game/SelectedUnits.h"
-#include "Game/GameSetup.h"
+#include "lib/minizip/zip.h"
+#include "mmgr.h"
+#include "PathCache.h"
+#include "PathFinder.h"
+#include "Map/ReadMap.h"
+#include "Sim/MoveTypes/MoveInfo.h"
+#include "Sim/MoveTypes/MoveMath/MoveMath.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
-
-#include "FileSystem/ArchiveZip.h"
-#include "FileSystem/FileSystem.h"
-
-#include "NetProtocol.h"
+#include "System/FileSystem/ArchiveZip.h"
+#include "System/FileSystem/FileSystem.h"
+#include "System/LogOutput.h"
+#include "System/ConfigHandler.h"
+#include "System/NetProtocol.h"
 
 #define PATHDEBUG false
 
@@ -43,15 +42,21 @@ const unsigned int PATHOPT_BLOCKED = 64;
 const unsigned int PATHOPT_SEARCHRELATED = (PATHOPT_OPEN | PATHOPT_CLOSED | PATHOPT_FORBIDDEN | PATHOPT_BLOCKED);
 const unsigned int PATHOPT_OBSOLETE = 128;
 
-const unsigned int PATHESTIMATOR_VERSION = 43;
+const unsigned int PATHESTIMATOR_VERSION = 44;
 const float PATHCOST_INFINITY = 10000000;
 const int SQUARES_TO_UPDATE = 600;
 
+const std::string pathDir = "cache/paths/";
 
-/*
- * constructor, loads precalculated data if it exists
+#if !defined(USE_MMGR)
+void* CPathEstimator::operator new(size_t size) { return ::PF_ALLOC(size); }
+void CPathEstimator::operator delete(void* p, size_t size) { ::PF_FREE(p, size); }
+#endif
+
+/**
+ * Constructor, loads precalculated data if it exists
  */
-CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int mmOpt, std::string name):
+CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int mmOpt, std::string cacheFileName, const std::string& map):
 	BLOCK_SIZE(BSIZE),
 	BLOCK_PIXEL_SIZE(BSIZE * SQUARE_SIZE),
 	BLOCKS_TO_UPDATE(SQUARES_TO_UPDATE / (BLOCK_SIZE * BLOCK_SIZE) + 1),
@@ -59,6 +64,7 @@ CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int
 	nbrOfBlocksX(gs->mapx / BLOCK_SIZE),
 	nbrOfBlocksZ(gs->mapy / BLOCK_SIZE),
 	nbrOfBlocks(nbrOfBlocksX * nbrOfBlocksZ),
+	openBlockBufferIndex(0),
 	moveMathOptions(mmOpt),
 	pathChecksum(0),
 	offsetBlockNum(nbrOfBlocks),costBlockNum(nbrOfBlocks),
@@ -89,9 +95,8 @@ CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int
 	blockState = new BlockInfo[nbrOfBlocks];
 	nbrOfVertices = moveinfo->moveData.size() * nbrOfBlocks * PATH_DIRECTION_VERTICES;
 	vertex = new float[nbrOfVertices];
-	openBlockBufferPointer = openBlockBuffer;
 
-	InitEstimator(name);
+	InitEstimator(cacheFileName, map);
 
 	// As all vertexes are bidirectional and have equal values
 	// in both directions, only one value needs to be stored.
@@ -110,10 +115,11 @@ CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int
 }
 
 
-/*
- * free all used memory
+/**
+ * Free all used memory
  */
-CPathEstimator::~CPathEstimator() {
+CPathEstimator::~CPathEstimator()
+{
 	for (int i = 0; i < nbrOfBlocks; i++)
 		delete[] blockState[i].sqrCenter;
 
@@ -123,7 +129,8 @@ CPathEstimator::~CPathEstimator() {
 }
 
 
-void CPathEstimator::InitEstimator(const std::string& name) {
+void CPathEstimator::InitEstimator(const std::string& cacheFileName, const std::string& map)
+{
 	int numThreads_tmp = configHandler->Get("HardwareThreadCount", 0);
 	size_t numThreads = ((numThreads_tmp < 0) ? 0 : numThreads_tmp);
 
@@ -149,7 +156,7 @@ void CPathEstimator::InitEstimator(const std::string& name) {
 
 	PrintLoadMsg("Reading estimate path costs");
 
-	if (!ReadFile(name)) {
+	if (!ReadFile(cacheFileName, map)) {
 		char calcMsg[512];
 		sprintf(calcMsg, "Analyzing map accessibility [%d]", BLOCK_SIZE);
 		PrintLoadMsg(calcMsg);
@@ -174,7 +181,7 @@ void CPathEstimator::InitEstimator(const std::string& name) {
 		delete pathBarrier;
 
 		PrintLoadMsg("Writing path data file...");
-		WriteFile(name);
+		WriteFile(cacheFileName, map);
 	}
 }
 
@@ -217,7 +224,8 @@ void CPathEstimator::CalcOffsetsAndPathCosts(int thread) {
 }
 
 
-void CPathEstimator::CalculateBlockOffsets(int idx, int thread) {
+void CPathEstimator::CalculateBlockOffsets(int idx, int thread)
+{
 	int x = idx % nbrOfBlocksX;
 	int z = idx / nbrOfBlocksX;
 
@@ -254,8 +262,8 @@ void CPathEstimator::EstimatePathCosts(int idx, int thread) {
 
 
 
-/*
- * finds a square accessable by the given movedata within the given block
+/**
+ * Finds a square accessable by the given movedata within the given block
  */
 void CPathEstimator::FindOffset(const MoveData& moveData, int blockX, int blockZ) {
 	// lower corner position of block
@@ -275,7 +283,7 @@ void CPathEstimator::FindOffset(const MoveData& moveData, int blockX, int blockZ
 			float cost = (dx * dx + dz * dz) + num / (0.001f + moveData.moveMath->SpeedMod(moveData, (int)(lowerX + x), (int)(lowerZ + z)));
 			int mask = CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN;
 
-			if (moveData.moveMath->IsBlocked2(moveData, lowerX + x, lowerZ + z, true) & mask) {
+			if (moveData.moveMath->IsBlocked2(moveData, lowerX + x, lowerZ + z) & mask) {
 				cost += 1000000.0f;
 			}
 			if (cost < best) {
@@ -292,8 +300,8 @@ void CPathEstimator::FindOffset(const MoveData& moveData, int blockX, int blockZ
 }
 
 
-/*
- * calculate all vertices connected from given block
+/**
+ * Calculate all vertices connected from the given block
  * (always 4 out of 8 vertices connected to the block)
  */
 void CPathEstimator::CalculateVertices(const MoveData& moveData, int blockX, int blockZ, int thread) {
@@ -302,8 +310,8 @@ void CPathEstimator::CalculateVertices(const MoveData& moveData, int blockX, int
 }
 
 
-/*
- * calculate requested vertex
+/**
+ * Calculate requested vertex
  */
 void CPathEstimator::CalculateVertex(const MoveData& moveData, int parentBlockX, int parentBlockZ, unsigned int direction, int thread) {
 	// initial calculations
@@ -341,7 +349,7 @@ void CPathEstimator::CalculateVertex(const MoveData& moveData, int parentBlockX,
 	// use this thread's "private" CPathFinder instance
 	// (rather than locking pathFinder->GetPath()) if we
 	// are in one
-	result = pathFinders[thread]->GetPath(moveData, startPos, pfDef, path, false, true, 10000, false);
+	result = pathFinders[thread]->GetPath(moveData, startPos, pfDef, path, false, true, CPathFinder::MAX_SEARCHED_SQUARES, false);
 
 	// store the result
 	if (result == Ok)
@@ -351,8 +359,8 @@ void CPathEstimator::CalculateVertex(const MoveData& moveData, int parentBlockX,
 }
 
 
-/*
- * mark affected blocks as obsolete
+/**
+ * Mark affected blocks as obsolete
  */
 void CPathEstimator::MapChanged(unsigned int x1, unsigned int z1, unsigned int x2, unsigned z2) {
 	// find the upper and lower corner of the rectangular area
@@ -401,8 +409,8 @@ void CPathEstimator::MapChanged(unsigned int x1, unsigned int z1, unsigned int x
 }
 
 
-/*
- * update some obsolete blocks using the FIFO-principle
+/**
+ * Update some obsolete blocks using the FIFO-principle
  */
 void CPathEstimator::Update() {
 	pathCache->Update();
@@ -433,17 +441,26 @@ void CPathEstimator::Update() {
 }
 
 
-/*
- * stores data and does some top-administration
+/**
+ * Stores data and does some top-administration
  */
-IPath::SearchResult CPathEstimator::GetPath(const MoveData& moveData, float3 start, const CPathFinderDef& peDef, Path& path, unsigned int maxSearchedBlocks) {
+IPath::SearchResult CPathEstimator::GetPath(
+	const MoveData& moveData,
+	float3 start,
+	const CPathFinderDef& peDef,
+	Path& path,
+	unsigned int maxSearchedBlocks,
+	bool synced
+) {
 	start.CheckInBounds();
+
 	// clear the path
 	path.path.clear();
 	path.pathCost = PATHCOST_INFINITY;
 
 	// initial calculations
-	maxBlocksToBeSearched = std::min(maxSearchedBlocks, (unsigned int) MAX_SEARCHED_BLOCKS);
+	maxBlocksToBeSearched = std::min(maxSearchedBlocks, MAX_SEARCHED_BLOCKS - 8U);
+
 	startBlock.x = (int)(start.x / BLOCK_PIXEL_SIZE);
 	startBlock.y = (int)(start.z / BLOCK_PIXEL_SIZE);
 	startBlocknr = startBlock.y * nbrOfBlocksX + startBlock.x;
@@ -451,11 +468,13 @@ IPath::SearchResult CPathEstimator::GetPath(const MoveData& moveData, float3 sta
 	goalBlock.x = peDef.goalSquareX / BLOCK_SIZE;
 	goalBlock.y = peDef.goalSquareZ / BLOCK_SIZE;
 
-	CPathCache::CacheItem* ci = pathCache->GetCachedPath(startBlock, goalBlock, peDef.sqGoalRadius, moveData.pathType);
-	if (ci) {
-		// use a cached path if we have one
-		path = ci->path;
-		return ci->result;
+	if (synced) {
+		CPathCache::CacheItem* ci = pathCache->GetCachedPath(startBlock, goalBlock, peDef.sqGoalRadius, moveData.pathType);
+		if (ci) {
+			// use a cached path if we have one (NOTE: only when in synced context)
+			path = ci->path;
+			return ci->result;
+		}
 	}
 
 	// oterhwise search
@@ -464,13 +483,16 @@ IPath::SearchResult CPathEstimator::GetPath(const MoveData& moveData, float3 sta
 	// if search successful, generate new path
 	if (result == Ok || result == GoalOutOfRange) {
 		FinishSearch(moveData, path);
-		// only add succesful paths to the cache
-		pathCache->AddPath(&path, result, startBlock, goalBlock, peDef.sqGoalRadius, moveData.pathType);
+
+		if (synced) {
+			// add succesful paths to the cache (NOTE: only when in synced context)
+			pathCache->AddPath(&path, result, startBlock, goalBlock, peDef.sqGoalRadius, moveData.pathType);
+		}
 
 		if (PATHDEBUG) {
 			LogObject() << "PE: Search completed.\n";
 			LogObject() << "Tested blocks: " << testedBlocks << "\n";
-			LogObject() << "Open blocks: " << (float)(openBlockBufferPointer - openBlockBuffer) << "\n";
+			LogObject() << "Open blocks: " << openBlockBufferIndex << "\n";
 			LogObject() << "Path length: " << (int)(path.path.size()) << "\n";
 			LogObject() << "Path cost: " << path.pathCost << "\n";
 		}
@@ -478,7 +500,7 @@ IPath::SearchResult CPathEstimator::GetPath(const MoveData& moveData, float3 sta
 		if (PATHDEBUG) {
 			LogObject() << "PE: Search failed!\n";
 			LogObject() << "Tested blocks: " << testedBlocks << "\n";
-			LogObject() << "Open blocks: " << (float)(openBlockBufferPointer - openBlockBuffer) << "\n";
+			LogObject() << "Open blocks: " << openBlockBufferIndex << "\n";
 		}
 	}
 
@@ -486,8 +508,8 @@ IPath::SearchResult CPathEstimator::GetPath(const MoveData& moveData, float3 sta
 }
 
 
-/*
- * make some initial calculations and preparations
+/**
+ * Make some initial calculations and preparations
  */
 IPath::SearchResult CPathEstimator::InitSearch(const MoveData& moveData, const CPathFinderDef& peDef) {
 	// is starting square inside goal area?
@@ -504,12 +526,13 @@ IPath::SearchResult CPathEstimator::InitSearch(const MoveData& moveData, const C
 	blockState[startBlocknr].cost = 0;
 	dirtyBlocks.push_back(startBlocknr);
 
+	openBlockBufferIndex = 0;
 	// add the starting block to the open-blocks-queue
-	OpenBlock* ob = openBlockBufferPointer = openBlockBuffer;
-	ob->cost = 0;
-	ob->currentCost = 0;
-	ob->block = startBlock;
-	ob->blocknr = startBlocknr;
+	OpenBlock* ob = &openBlockBuffer[openBlockBufferIndex];
+		ob->cost = 0;
+		ob->currentCost = 0;
+		ob->block = startBlock;
+		ob->blocknr = startBlocknr;
 	openBlocks.push(ob);
 
 	// mark starting point as best found position
@@ -530,12 +553,13 @@ IPath::SearchResult CPathEstimator::InitSearch(const MoveData& moveData, const C
 }
 
 
-/*
- * performs the actual search.
+/**
+ * Performs the actual search.
  */
 IPath::SearchResult CPathEstimator::DoSearch(const MoveData& moveData, const CPathFinderDef& peDef) {
 	bool foundGoal = false;
-	while (!openBlocks.empty() && (openBlockBufferPointer - openBlockBuffer) < (maxBlocksToBeSearched - 8)) {
+
+	while (!openBlocks.empty() && (openBlockBufferIndex < maxBlocksToBeSearched)) {
 		// get the open block with lowest cost
 		OpenBlock* ob = openBlocks.top();
 		openBlocks.pop();
@@ -558,6 +582,8 @@ IPath::SearchResult CPathEstimator::DoSearch(const MoveData& moveData, const CPa
 		}
 
 		// no, test the 8 surrounding blocks
+		// NOTE: each of these calls increments openBlockBufferIndex by 1, so
+		// maxBlocksToBeSearched is always less than <MAX_SEARCHED_BLOCKS - 8>
 		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT);
 		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT_UP);
 		TestBlock(moveData, peDef, *ob, PATHDIR_UP);
@@ -576,7 +602,7 @@ IPath::SearchResult CPathEstimator::DoSearch(const MoveData& moveData, const CPa
 		return Ok;
 
 	// we could not reach the goal
-	if (openBlockBufferPointer - openBlockBuffer >= (maxBlocksToBeSearched - 8))
+	if (openBlockBufferIndex >= maxBlocksToBeSearched)
 		return GoalOutOfRange;
 
 	// search could not reach the goal due to the unit being locked in
@@ -589,9 +615,9 @@ IPath::SearchResult CPathEstimator::DoSearch(const MoveData& moveData, const CPa
 }
 
 
-/*
- * test the accessability of a block and its value,
- * possibly also add it to the open-blocks pqueue
+/**
+ * Test the accessability of a block and its value,
+ * possibly also add it to the open-blocks pqueue.
  */
 void CPathEstimator::TestBlock(const MoveData& moveData, const CPathFinderDef &peDef, OpenBlock& parentOpenBlock, unsigned int direction) {
 	testedBlocks++;
@@ -650,11 +676,14 @@ void CPathEstimator::TestBlock(const MoveData& moveData, const CPathFinderDef &p
 	}
 
 	// store this block as open.
-	OpenBlock* ob = ++openBlockBufferPointer;
-	ob->block = block;
-	ob->blocknr = blocknr;
-	ob->cost = cost;
-	ob->currentCost = currentCost;
+	++openBlockBufferIndex;
+	assert(openBlockBufferIndex < MAX_SEARCHED_BLOCKS);
+
+	OpenBlock* ob = &openBlockBuffer[openBlockBufferIndex];
+		ob->block = block;
+		ob->blocknr = blocknr;
+		ob->cost = cost;
+		ob->currentCost = currentCost;
 	openBlocks.push(ob);
 
 	// Mark the block as open, and its parent.
@@ -665,8 +694,8 @@ void CPathEstimator::TestBlock(const MoveData& moveData, const CPathFinderDef &p
 }
 
 
-/*
- * recreate the path taken to the goal
+/**
+ * Recreate the path taken to the goal
  */
 void CPathEstimator::FinishSearch(const MoveData& moveData, Path& path) {
 	int2 block = goalBlock;
@@ -700,8 +729,8 @@ void CPathEstimator::FinishSearch(const MoveData& moveData, Path& path) {
 }
 
 
-/*
- * clean lists from last search
+/**
+ * Clean lists from last search
  */
 void CPathEstimator::ResetSearch() {
 	while (!openBlocks.empty())
@@ -717,17 +746,17 @@ void CPathEstimator::ResetSearch() {
 }
 
 
-/*
- * try to read offset and vertices data from file, return false on failure
+/**
+ * Try to read offset and vertices data from file, return false on failure
  * TODO: Read-error-check.
  */
-bool CPathEstimator::ReadFile(std::string name)
+bool CPathEstimator::ReadFile(std::string cacheFileName, const std::string& map)
 {
 	unsigned int hash = Hash();
 	char hashString[50];
 	sprintf(hashString, "%u", hash);
 
-	std::string filename = std::string("maps/paths/") + gameSetup->mapName.substr(0, gameSetup->mapName.find_last_of('.') + 1) + hashString + "." + name + ".zip";
+	std::string filename = std::string(pathDir) + map + hashString + "." + cacheFileName + ".zip";
 
 	// open file for reading from a suitable location (where the file exists)
 	CArchiveZip* pfile = new CArchiveZip(filesystem.LocateFile(filename));
@@ -740,23 +769,37 @@ bool CPathEstimator::ReadFile(std::string name)
 	std::auto_ptr<CArchiveZip> auto_pfile(pfile);
 	CArchiveZip& file(*pfile);
 
-	int fh = file.OpenFile("pathinfo");
+	const unsigned fid = file.FindFile("pathinfo");
+	if (fid < file.NumFiles())
+	{
+		pathChecksum = file.GetCrc32(fid);
 
-	if (fh) {
-		pathChecksum = file.GetCrc32("pathinfo");
+		std::vector<boost::uint8_t> buffer;
+		file.GetFile(fid, buffer);
 
-		unsigned int filehash = 0;
- 		// Check hash.
-		file.ReadFile(fh, &filehash, 4);
+		if (buffer.size() < 4)
+			return false;
+
+		unsigned filehash = *((unsigned*)&buffer[0]);
 		if (filehash != hash)
 			return false;
 
+		unsigned pos = sizeof(unsigned);
+
 		// Read block-center-offset data.
+		const unsigned blockSize = moveinfo->moveData.size() * sizeof(int2);
+		if (buffer.size() < pos + blockSize*nbrOfBlocks)
+			return false;
 		for (int blocknr = 0; blocknr < nbrOfBlocks; blocknr++)
-			file.ReadFile(fh, blockState[blocknr].sqrCenter, moveinfo->moveData.size() * sizeof(int2));
+		{
+			std::memcpy(blockState[blocknr].sqrCenter, &buffer[pos], blockSize);
+			pos += blockSize;
+		}
 
 		// Read vertices data.
-		file.ReadFile(fh, vertex, nbrOfVertices * sizeof(float));
+		if (buffer.size() < pos + nbrOfVertices * sizeof(float))
+			return false;
+		std::memcpy(vertex, &buffer[pos], nbrOfVertices * sizeof(float));
 
 		// File read successful.
 		return true;
@@ -766,19 +809,20 @@ bool CPathEstimator::ReadFile(std::string name)
 }
 
 
-/*
- * try to write offset and vertex data to file
+/**
+ * Try to write offset and vertex data to file.
  */
-void CPathEstimator::WriteFile(std::string name) {
+void CPathEstimator::WriteFile(std::string cacheFileName, const std::string& map)
+{
 	// We need this directory to exist
-	if (!filesystem.CreateDirectory("maps/paths"))
+	if (!filesystem.CreateDirectory(pathDir))
 		return;
 
 	unsigned int hash = Hash();
 	char hashString[50];
 	sprintf(hashString,"%u",hash);
 
-	std::string filename = std::string("maps/paths/") + gameSetup->mapName.substr(0, gameSetup->mapName.find_last_of('.') + 1) + hashString + "." + name + ".zip";
+	std::string filename = std::string(pathDir) + map + hashString + "." + cacheFileName + ".zip";
 	zipFile file;
 
 	// open file for writing in a suitable location
@@ -788,7 +832,6 @@ void CPathEstimator::WriteFile(std::string name) {
 		zipOpenNewFileInZip(file, "pathinfo", NULL, NULL, 0, NULL, 0, NULL, Z_DEFLATED, Z_BEST_COMPRESSION);
 
 		// Write hash.
-		unsigned int hash = Hash();
 		zipWriteInFileInZip(file, (void*) &hash, 4);
 
 		// Write block-center-offsets.
@@ -812,175 +855,23 @@ void CPathEstimator::WriteFile(std::string name) {
 
 		std::auto_ptr<CArchiveZip> auto_pfile(pfile);
 		CArchiveZip& file(*pfile);
-		pathChecksum = file.GetCrc32("pathinfo");
+		
+		const unsigned fid = file.FindFile("pathinfo");
+		assert(fid < file.NumFiles());
+		pathChecksum = file.GetCrc32(fid);
 	}
 }
 
 
-/*
-Gives a hash-code identifying the dataset of this estimator.
-*/
-unsigned int CPathEstimator::Hash()
+/**
+ * Returns a hash-code identifying the dataset of this estimator.
+ */
+unsigned int CPathEstimator::Hash() const
 {
 	return (readmap->mapChecksum + moveinfo->moveInfoChecksum + BLOCK_SIZE + moveMathOptions + PATHESTIMATOR_VERSION);
 }
 
-boost::uint32_t CPathEstimator::GetPathChecksum()
-{
-	return pathChecksum;
-}
 
-void CPathEstimator::Draw(void)
-{
-	GML_RECMUTEX_LOCK(sel); // Draw
-
-	MoveData* md = NULL;
-
-	if (!moveinfo->moveData.empty()) {
-		md = moveinfo->moveData[0];
-	} else {
-		return;
-	}
-
-	if (!selectedUnits.selectedUnits.empty() && (*selectedUnits.selectedUnits.begin())->unitDef->movedata) {
-		md = (*selectedUnits.selectedUnits.begin())->unitDef->movedata;
-	}
-
-	glDisable(GL_TEXTURE_2D);
-	glColor3f(1, 1, 0);
-
-/*
-	float blue = BLOCK_SIZE == 32? 1: 0;
-	glBegin(GL_LINES);
-	for (int z = 0; z < nbrOfBlocksZ; z++) {
-		for (int x = 0; x < nbrOfBlocksX; x++) {
-			int blocknr = z * nbrOfBlocksX + x;
-			float3 p1;
-			p1.x = (blockState[blocknr].sqrCenter[md->pathType].x) * 8;
-			p1.z = (blockState[blocknr].sqrCenter[md->pathType].y) * 8;
-			p1.y = ground->GetHeight(p1.x, p1.z) + 10;
-
-			glColor3f(1, 1, blue);
-			glVertexf3(p1);
-			glVertexf3(p1 - UpVector * 10);
-			for (int dir = 0; dir < PATH_DIRECTION_VERTICES; dir++) {
-				int obx = x + directionVector[dir].x;
-				int obz = z + directionVector[dir].y;
-
-				if (obx >= 0 && obz >= 0 && obx < nbrOfBlocksX && obz < nbrOfBlocksZ) {
-					float3 p2;
-					int obblocknr = obz * nbrOfBlocksX + obx;
-
-					p2.x = (blockState[obblocknr].sqrCenter[md->pathType].x) * 8;
-					p2.z = (blockState[obblocknr].sqrCenter[md->pathType].y) * 8;
-					p2.y = ground->GetHeight(p2.x, p2.z) + 10;
-
-					int vertexNbr = md->pathType * nbrOfBlocks * PATH_DIRECTION_VERTICES + blocknr * PATH_DIRECTION_VERTICES + directionVertex[dir];
-					float cost = vertex[vertexNbr];
-
-					glColor3f(1 / (sqrt(cost/BLOCK_SIZE)), 1 / (cost/BLOCK_SIZE), blue);
-					glVertexf3(p1);
-					glVertexf3(p2);
-				}
-			}
-		}
-
-	}
-	glEnd();
-
-
-	glEnable(GL_TEXTURE_2D);
-	for (int z = 0; z < nbrOfBlocksZ; z++) {
-		for (int x = 0; x < nbrOfBlocksX; x++) {
-			int blocknr = z * nbrOfBlocksX + x;
-			float3 p1;
-			p1.x = (blockState[blocknr].sqrCenter[md->pathType].x) * SQUARE_SIZE;
-			p1.z = (blockState[blocknr].sqrCenter[md->pathType].y) * SQUARE_SIZE;
-			p1.y = ground->GetHeight(p1.x, p1.z) + 10;
-
-			glColor3f(1, 1, blue);
-			for (int dir = 0; dir < PATH_DIRECTION_VERTICES; dir++) {
-				int obx = x + directionVector[dir].x;
-				int obz = z + directionVector[dir].y;
-
-				if (obx >= 0 && obz >= 0 && obx < nbrOfBlocksX && obz < nbrOfBlocksZ) {
-					float3 p2;
-					int obblocknr = obz * nbrOfBlocksX + obx;
-
-					p2.x = (blockState[obblocknr].sqrCenter[md->pathType].x) * SQUARE_SIZE;
-					p2.z = (blockState[obblocknr].sqrCenter[md->pathType].y) * SQUARE_SIZE;
-					p2.y = ground->GetHeight(p2.x, p2.z) + 10;
-
-					int vertexNbr = md->pathType * nbrOfBlocks * PATH_DIRECTION_VERTICES + blocknr * PATH_DIRECTION_VERTICES + directionVertex[dir];
-					float cost = vertex[vertexNbr];
-
-					glColor3f(1, 1 / (cost/BLOCK_SIZE), blue);
-
-					p2 = (p1 + p2) / 2;
-					if (camera->pos.SqDistance(p2) < 250000) {
-						font->glWorldPrint(p2,5,"%.0f", cost);
-					}
-				}
-			}
-		}
-	}
-*/
-
-
-	if (BLOCK_SIZE == 8)
-		glColor3f(0.2f, 0.7f, 0.2f);
-	else
-		glColor3f(0.2f, 0.2f, 0.7f);
-
-	glDisable(GL_TEXTURE_2D);
-	glBegin(GL_LINES);
-
-	for (OpenBlock* ob = openBlockBuffer; ob != openBlockBufferPointer; ++ob) {
-		int blocknr = ob->blocknr;
-		float3 p1;
-		p1.x = (blockState[blocknr].sqrCenter[md->pathType].x) * SQUARE_SIZE;
-		p1.z = (blockState[blocknr].sqrCenter[md->pathType].y) * SQUARE_SIZE;
-		p1.y = ground->GetHeight(p1.x, p1.z) + 15;
-
-		float3 p2;
-		int obx = blockState[ob->blocknr].parentBlock.x;
-		int obz = blockState[ob->blocknr].parentBlock.y;
-		int obblocknr = obz * nbrOfBlocksX + obx;
-
-		if (obblocknr >= 0) {
-			p2.x = (blockState[obblocknr].sqrCenter[md->pathType].x) * SQUARE_SIZE;
-			p2.z = (blockState[obblocknr].sqrCenter[md->pathType].y) * SQUARE_SIZE;
-			p2.y = ground->GetHeight(p2.x, p2.z) + 15;
-
-			glVertexf3(p1);
-			glVertexf3(p2);
-		}
-	}
-
-	glEnd();
-
-
-/*
-	glEnable(GL_TEXTURE_2D);
-	glEnable(GL_BLEND);
-	glColor4f(1,0,blue,0.7f);
-	glAlphaFunc(GL_GREATER,0.05f);
-	int a=0;
-	for(OpenBlock*  ob=openBlockBuffer;ob!=openBlockBufferPointer;++ob){
-		int blocknr = ob->blocknr;
-		float3 p1;
-		p1.x=(ob->block.x * BLOCK_SIZE + blockState[blocknr].sqrCenter[md->pathType].x)*SQUARE_SIZE;
-		p1.z=(ob->block.y * BLOCK_SIZE + blockState[blocknr].sqrCenter[md->pathType].y)*SQUARE_SIZE;
-		p1.y=ground->GetHeight(p1.x,p1.z)+15;
-
-		if(camera->pos.SqDistance(p1)<250000){
-			font->glWorldPrint(p1,5,"%.0f %.0f",ob->cost,ob->currentCost);
-		}
-		++a;
-	}
-	glDisable(GL_BLEND);
-*/
-}
 
 float3 CPathEstimator::FindBestBlockCenter(const MoveData* moveData, float3 pos)
 {
@@ -988,18 +879,18 @@ float3 CPathEstimator::FindBestBlockCenter(const MoveData* moveData, float3 pos)
 	CRangedGoalWithCircularConstraint rangedGoal(pos, pos, 0, 0, SQUARE_SIZE * BLOCK_SIZE * SQUARE_SIZE * BLOCK_SIZE * 4);
 	IPath::Path path;
 
-	std::vector<float3> startPos;
+	std::vector<float3> startPositions;
 
 	int xm = (int) (pos.x / (SQUARE_SIZE * BLOCK_SIZE));
 	int ym = (int) (pos.z / (SQUARE_SIZE * BLOCK_SIZE));
 
 	for (int y = std::max(0, ym - 1); y <= std::min(nbrOfBlocksZ - 1, ym + 1); ++y) {
 		for (int x = std::max(0, xm - 1); x <= std::min(nbrOfBlocksX - 1, xm + 1); ++x) {
-			startPos.push_back(float3(blockState[y * nbrOfBlocksX + x].sqrCenter[pathType].x * SQUARE_SIZE, 0, blockState[y * nbrOfBlocksX+x].sqrCenter[pathType].y * SQUARE_SIZE));
+			startPositions.push_back(float3(blockState[y * nbrOfBlocksX + x].sqrCenter[pathType].x * SQUARE_SIZE, 0, blockState[y * nbrOfBlocksX+x].sqrCenter[pathType].y * SQUARE_SIZE));
 		}
 	}
 
-	IPath::SearchResult result = pathFinder->GetPath(*moveData, startPos, rangedGoal, path);
+	IPath::SearchResult result = pathFinder->GetPath(*moveData, startPositions, rangedGoal, path);
 
 	if (result == IPath::Ok && !path.path.empty()) {
 		return path.path.back();
