@@ -1,18 +1,22 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
-#include "SDL_timer.h"
-#include <vector>
-#include <boost/cstdint.hpp>
 #include "mmgr.h"
 
 #include "PathManager.h"
-#include "LogOutput.h"
-#include "myMath.h"
-#include "Sim/MoveTypes/MoveInfo.h"
-#include "Rendering/GL/myGL.h"
-#include "TimeProfiler.h"
+
+#include <vector>
+#include <boost/cstdint.hpp>
+
 #include "PathFinder.h"
 #include "PathEstimator.h"
 #include "Map/MapInfo.h"
+#include "Sim/Misc/GlobalSynced.h"
+#include "Sim/MoveTypes/MoveInfo.h"
+#include "Sim/MoveTypes/MoveMath/MoveMath.h"
+#include "System/LogOutput.h"
+#include "System/myMath.h"
+#include "System/TimeProfiler.h"
 
 const float ESTIMATE_DISTANCE = 55;
 const float MIN_ESTIMATE_DISTANCE = 40;
@@ -21,19 +25,34 @@ const float MIN_DETAILED_DISTANCE = 12;
 const unsigned int MAX_SEARCHED_NODES_ON_REFINE = 2000;
 const unsigned int CPathManager::PATH_RESOLUTION = CPathFinder::PATH_RESOLUTION;
 
-CPathManager* pathManager=0;
 
-CPathManager::CPathManager() {
+
+CPathManager* pathManager = 0;
+
+CPathManager::CPathManager()
+{
 	// Create pathfinder and estimators.
 	pf  = new CPathFinder();
-	pe  = new CPathEstimator(pf,  8, CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN, "pe");
-	pe2 = new CPathEstimator(pf, 32, CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN, "pe2");
+	pe  = new CPathEstimator(pf,  8, CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN, "pe", mapInfo->map.name);
+	pe2 = new CPathEstimator(pf, 32, CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN, "pe2", mapInfo->map.name);
 
 	// Reset id-counter.
 	nextPathId = 0;
+
+	logOutput.Print("[CPathManager] pathing data checksum: %08x\n", GetPathCheckSum());
+
+	#ifdef SYNCDEBUG
+	// clients may have a non-writable cache directory (which causes
+	// the estimator path-file checksum to remain zero), so we can't
+	// update the sync-checker with this in normal builds
+	// NOTE: better to just checksum the in-memory data and broadcast
+	// that instead of relying on the zip-file CRC?
+	{ SyncedUint tmp(GetPathCheckSum()); }
+	#endif
 }
 
-CPathManager::~CPathManager() {
+CPathManager::~CPathManager()
+{
 	delete pe2;
 	delete pe;
 	delete pf;
@@ -41,23 +60,12 @@ CPathManager::~CPathManager() {
 
 
 
-void CPathManager::SetHeatMappingEnabled(bool enabled)
-{
-	pf->SetHeatMapState(enabled);
-}
-
-bool CPathManager::GetHeatMappingEnabled()
-{
-	return pf->GetHeatMapState();
-}
-
-
 /*
 Help-function.
 Turns a start->goal-request into a well-defined request.
 */
 unsigned int CPathManager::RequestPath(const MoveData* moveData, float3 startPos,
-		float3 goalPos, float goalRadius, CSolidObject* caller) {
+		float3 goalPos, float goalRadius, CSolidObject* caller, bool synced) {
 	startPos.CheckInBounds();
 	goalPos.CheckInBounds();
 
@@ -68,7 +76,7 @@ unsigned int CPathManager::RequestPath(const MoveData* moveData, float3 startPos
 	CRangedGoalWithCircularConstraint* rangedGoalPED = new CRangedGoalWithCircularConstraint(startPos,goalPos, goalRadius, 3, 2000);
 
 	// Make request.
-	return RequestPath(moveData, startPos, rangedGoalPED, goalPos, caller);
+	return RequestPath(moveData, startPos, rangedGoalPED, goalPos, caller, synced);
 }
 
 
@@ -76,7 +84,7 @@ unsigned int CPathManager::RequestPath(const MoveData* moveData, float3 startPos
 Request a new multipath, store the result and return a handle-id to it.
 */
 unsigned int CPathManager::RequestPath(const MoveData* md, float3 startPos,
-		CPathFinderDef* peDef, float3 goalPos, CSolidObject* caller) {
+		CPathFinderDef* peDef, float3 goalPos, CSolidObject* caller, bool synced) {
 	SCOPED_TIMER("PFS");
 
 	MoveData* moveData = moveinfo->moveData[md->pathType];
@@ -98,8 +106,9 @@ unsigned int CPathManager::RequestPath(const MoveData* md, float3 startPos,
 
 	if (distanceToGoal < DETAILED_DISTANCE) {
 		// Get a detailed path.
-		IPath::SearchResult result = pf->GetPath(*moveData, startPos, *peDef,
-				newPath->detailedPath, true, false, 10000, true, ownerId);
+		IPath::SearchResult result =
+			pf->GetPath(*moveData, startPos, *peDef, newPath->detailedPath, true, false, CPathFinder::MAX_SEARCHED_SQUARES, true, ownerId);
+		newPath->searchResult = result;
 
 		if (result == IPath::Ok || result == IPath::GoalOutOfRange) {
 			retValue = Store(newPath);
@@ -108,7 +117,8 @@ unsigned int CPathManager::RequestPath(const MoveData* md, float3 startPos,
 		}
 	} else if (distanceToGoal < ESTIMATE_DISTANCE) {
 		// Get an estimate path.
-		IPath::SearchResult result = pe->GetPath(*moveData, startPos, *peDef, newPath->estimatedPath, ownerId);
+		IPath::SearchResult result = pe->GetPath(*moveData, startPos, *peDef, newPath->estimatedPath, CPathEstimator::MAX_SEARCHED_BLOCKS, synced);
+		newPath->searchResult = result;
 
 		if (result == IPath::Ok || result == IPath::GoalOutOfRange) {
 			// Turn a part of it into detailed path.
@@ -122,7 +132,8 @@ unsigned int CPathManager::RequestPath(const MoveData* md, float3 startPos,
 			if (sp.x != 0 &&
 				(((int) sp.x) / (SQUARE_SIZE * 8) != ((int) startPos.x) / (SQUARE_SIZE * 8) ||
 				((int) sp.z) / (SQUARE_SIZE * 8) != ((int) startPos.z) / (SQUARE_SIZE * 8))) {
-				IPath::SearchResult result = pe->GetPath(*moveData, sp, *peDef, newPath->estimatedPath);
+				IPath::SearchResult result = pe->GetPath(*moveData, sp, *peDef, newPath->estimatedPath, CPathEstimator::MAX_SEARCHED_BLOCKS, synced);
+				newPath->searchResult = result;
 
 				if (result == IPath::Ok || result == IPath::GoalOutOfRange) {
 					EstimateToDetailed(*newPath, startPos, ownerId);
@@ -136,18 +147,20 @@ unsigned int CPathManager::RequestPath(const MoveData* md, float3 startPos,
 		}
 	} else {
 		// Get a low-res. estimate path.
-		IPath::SearchResult result = pe2->GetPath(*moveData, startPos, *peDef, newPath->estimatedPath2);
+		IPath::SearchResult result = pe2->GetPath(*moveData, startPos, *peDef, newPath->estimatedPath2, CPathEstimator::MAX_SEARCHED_BLOCKS, synced);
+		newPath->searchResult = result;
 
 		if (result == IPath::Ok || result == IPath::GoalOutOfRange) {
 			// Turn a part of it into hi-res. estimate path.
-			Estimate2ToEstimate(*newPath, startPos, ownerId);
+			Estimate2ToEstimate(*newPath, startPos, ownerId, synced);
 			// And estimate into detailed.
 			EstimateToDetailed(*newPath, startPos, ownerId);
 			// Store the path.
 			retValue = Store(newPath);
 		} else {
 			// sometimes the 32*32 squares can be wrong so if it fails to get a path also try with 8*8 squares
-			IPath::SearchResult result = pe->GetPath(*moveData, startPos, *peDef, newPath->estimatedPath);
+			IPath::SearchResult result = pe->GetPath(*moveData, startPos, *peDef, newPath->estimatedPath, CPathEstimator::MAX_SEARCHED_BLOCKS, synced);
+			newPath->searchResult = result;
 
 			if (result == IPath::Ok || result == IPath::GoalOutOfRange) {
 				EstimateToDetailed(*newPath, startPos, ownerId);
@@ -159,7 +172,7 @@ unsigned int CPathManager::RequestPath(const MoveData* md, float3 startPos,
 				if (sp.x != 0 &&
 					(((int) sp.x) / (SQUARE_SIZE * 8) != ((int) startPos.x) / (SQUARE_SIZE * 8) ||
 					((int) sp.z) / (SQUARE_SIZE * 8) != ((int) startPos.z) / (SQUARE_SIZE * 8))) {
-					IPath::SearchResult result = pe->GetPath(*moveData, sp, *peDef, newPath->estimatedPath);
+					IPath::SearchResult result = pe->GetPath(*moveData, sp, *peDef, newPath->estimatedPath, CPathEstimator::MAX_SEARCHED_BLOCKS, synced);
 
 					if (result == IPath::Ok || result == IPath::GoalOutOfRange) {
 						EstimateToDetailed(*newPath, startPos, ownerId);
@@ -223,13 +236,13 @@ void CPathManager::EstimateToDetailed(MultiPath& path, float3 startPos, int owne
 	//Perform the search.
 	//If this is the final improvement of the path, then use the original goal.
 	IPath::SearchResult result;
-	if(path.estimatedPath.path.empty() && path.estimatedPath2.path.empty())
-		result = pf->GetPath(*path.moveData, startPos, *path.peDef, path.detailedPath, true, false, 10000, true, ownerId);
+	if (path.estimatedPath.path.empty() && path.estimatedPath2.path.empty())
+		result = pf->GetPath(*path.moveData, startPos, *path.peDef, path.detailedPath, true, false, CPathFinder::MAX_SEARCHED_SQUARES, true, ownerId);
 	else
-		result = pf->GetPath(*path.moveData, startPos, rangedGoalPFD, path.detailedPath, true, false, 10000, true, ownerId);
+		result = pf->GetPath(*path.moveData, startPos, rangedGoalPFD, path.detailedPath, true, false, CPathFinder::MAX_SEARCHED_SQUARES, true, ownerId);
 
 	//If no refined path could be found, set goal as desired goal.
-	if(result == IPath::CantGetCloser || result == IPath::Error) {
+	if (result == IPath::CantGetCloser || result == IPath::Error) {
 		path.detailedPath.pathGoal = goalPos;
 	}
 }
@@ -238,22 +251,23 @@ void CPathManager::EstimateToDetailed(MultiPath& path, float3 startPos, int owne
 /*
 Turns a part of the estimate2 path into estimate path.
 */
-void CPathManager::Estimate2ToEstimate(MultiPath& path, float3 startPos, int ownerId) const
+void CPathManager::Estimate2ToEstimate(MultiPath& path, float3 startPos, int ownerId, bool synced) const
 {
 	//If there is no estimate2 path, nothing could be done.
-	if(path.estimatedPath2.path.empty())
+	if (path.estimatedPath2.path.empty())
 		return;
 
 	path.estimatedPath2.path.pop_back();
 	//Remove estimate2 waypoints until
 	//the next one is far enought.
-	while(!path.estimatedPath2.path.empty()
-	&& path.estimatedPath2.path.back().SqDistance2D(startPos) < Square(ESTIMATE_DISTANCE * SQUARE_SIZE))
+	while (!path.estimatedPath2.path.empty() &&
+		path.estimatedPath2.path.back().SqDistance2D(startPos) < Square(ESTIMATE_DISTANCE * SQUARE_SIZE)) {
 		path.estimatedPath2.path.pop_back();
+	}
 
 	//Get the goal of the detailed search.
 	float3 goalPos;
-	if(path.estimatedPath2.path.empty())
+	if (path.estimatedPath2.path.empty())
 		goalPos = path.estimatedPath2.pathGoal;
 	else
 		goalPos = path.estimatedPath2.path.back();
@@ -261,17 +275,17 @@ void CPathManager::Estimate2ToEstimate(MultiPath& path, float3 startPos, int own
 	//Define the search.
 	CRangedGoalWithCircularConstraint rangedGoal(startPos,goalPos, 0,2,20);
 
-	//Perform the search.
-	//If there is no estimate2 path left, use original goal.
+	// Perform the search.
+	// If there is no estimate2 path left, use original goal.
 	IPath::SearchResult result;
-	if(path.estimatedPath2.path.empty())
-		result = pe->GetPath(*path.moveData, startPos, *path.peDef, path.estimatedPath, MAX_SEARCHED_NODES_ON_REFINE);
+	if (path.estimatedPath2.path.empty())
+		result = pe->GetPath(*path.moveData, startPos, *path.peDef, path.estimatedPath, MAX_SEARCHED_NODES_ON_REFINE, synced);
 	else {
-		result = pe->GetPath(*path.moveData, startPos, rangedGoal, path.estimatedPath, MAX_SEARCHED_NODES_ON_REFINE);
+		result = pe->GetPath(*path.moveData, startPos, rangedGoal, path.estimatedPath, MAX_SEARCHED_NODES_ON_REFINE, synced);
 	}
 
-	//If no refined path could be found, set goal as desired goal.
-	if(result == IPath::CantGetCloser || result == IPath::Error) {
+	// If no refined path could be found, set goal as desired goal.
+	if (result == IPath::CantGetCloser || result == IPath::Error) {
 		path.estimatedPath.pathGoal = goalPos;
 	}
 }
@@ -281,60 +295,73 @@ void CPathManager::Estimate2ToEstimate(MultiPath& path, float3 startPos, int own
 Removes and return the next waypoint in the multipath corresponding to given id.
 */
 float3 CPathManager::NextWaypoint(unsigned int pathId, float3 callerPos, float minDistance,
-		int numRetries, int ownerId) const
+		int numRetries, int ownerId, bool synced) const
 {
 	SCOPED_TIMER("PFS");
 
-	//0 indicate a no-path id.
-	if(pathId == 0)
-		return float3(-1,-1,-1);
+	// 0 indicates a no-path id
+	if (pathId == 0)
+		return float3(-1.0f, -1.0f, -1.0f);
 
-	if(numRetries>4)
-		return float3(-1,-1,-1);
+	if (numRetries > 4)
+		return float3(-1.0f, -1.0f, -1.0f);
 
 	//Find corresponding multipath.
 	std::map<unsigned int, MultiPath*>::const_iterator pi = pathMap.find(pathId);
-	if(pi == pathMap.end())
-		return float3(-1,-1,-1);
+	if (pi == pathMap.end())
+		return float3(-1.0f, -1.0f, -1.0f);
 	MultiPath* multiPath = pi->second;
 
-	if(callerPos==ZeroVector){
-		if(!multiPath->detailedPath.path.empty())
-			callerPos=multiPath->detailedPath.path.back();
+	if (callerPos == ZeroVector) {
+		if (!multiPath->detailedPath.path.empty())
+			callerPos = multiPath->detailedPath.path.back();
 	}
 
-	//check if detailed path need bettering
-	if(!multiPath->estimatedPath.path.empty()
-	&& (multiPath->estimatedPath.path.back().SqDistance2D(callerPos) < Square(MIN_DETAILED_DISTANCE * SQUARE_SIZE)
-	|| multiPath->detailedPath.path.size() <= 2)){
+	// check if detailed path needs bettering
+	if (!multiPath->estimatedPath.path.empty() &&
+		(multiPath->estimatedPath.path.back().SqDistance2D(callerPos) < Square(MIN_DETAILED_DISTANCE * SQUARE_SIZE) ||
+		multiPath->detailedPath.path.size() <= 2)) {
 
-		if(!multiPath->estimatedPath2.path.empty()		//if so check if estimated path also need bettering
-			&& (multiPath->estimatedPath2.path.back().SqDistance2D(callerPos) < Square(MIN_ESTIMATE_DISTANCE * SQUARE_SIZE)
-			|| multiPath->estimatedPath.path.size() <= 2)){
-				Estimate2ToEstimate(*multiPath, callerPos, ownerId);
+		if (!multiPath->estimatedPath2.path.empty() &&  // if so, check if estimated path also needs bettering
+			(multiPath->estimatedPath2.path.back().SqDistance2D(callerPos) < Square(MIN_ESTIMATE_DISTANCE * SQUARE_SIZE) ||
+			multiPath->estimatedPath.path.size() <= 2)) {
+
+			Estimate2ToEstimate(*multiPath, callerPos, ownerId, synced);
 		}
 
-		if(multiPath->caller)
+		if (multiPath->caller) {
 			multiPath->caller->UnBlock();
+		}
+
 		EstimateToDetailed(*multiPath, callerPos, ownerId);
-		if(multiPath->caller)
+
+		if (multiPath->caller) {
 			multiPath->caller->Block();
+		}
 	}
 
-	//Repeat until a waypoint distant enought are found.
 	float3 waypoint;
 	do {
-		//Get next waypoint.
-		if(multiPath->detailedPath.path.empty()) {
-			if(multiPath->estimatedPath2.path.empty() && multiPath->estimatedPath.path.empty())
-				return multiPath->finalGoal;
-			else
-				return NextWaypoint(pathId,callerPos,minDistance,numRetries+1,ownerId);
+		// get the next waypoint from the high-res path
+		//
+		// if this is not possible, then either we are
+		// at the goal OR the path could not reach all
+		// the way to it (ie. a GoalOutOfRange result)
+		if (multiPath->detailedPath.path.empty()) {
+			if (multiPath->estimatedPath2.path.empty() && multiPath->estimatedPath.path.empty()) {
+				if (multiPath->searchResult == IPath::Ok) {
+					return multiPath->finalGoal;
+				} else {
+					return float3(-1.0f, -1.0f, -1.0f);
+				}
+			} else {
+				return NextWaypoint(pathId, callerPos, minDistance, numRetries + 1, ownerId, synced);
+			}
 		} else {
 			waypoint = multiPath->detailedPath.path.back();
 			multiPath->detailedPath.path.pop_back();
 		}
-	} while(callerPos.SqDistance2D(waypoint) < Square(minDistance) && waypoint != multiPath->detailedPath.pathGoal);
+	} while (callerPos.SqDistance2D(waypoint) < Square(minDistance) && waypoint != multiPath->detailedPath.pathGoal);
 
 	return waypoint;
 }
@@ -350,7 +377,7 @@ void CPathManager::DeletePath(unsigned int pathId) {
 
 	//Find the multipath.
 	std::map<unsigned int, MultiPath*>::iterator pi = pathMap.find(pathId);
-	if(pi == pathMap.end())
+	if (pi == pathMap.end())
 		return;
 	MultiPath* multiPath = pi->second;
 
@@ -372,15 +399,12 @@ void CPathManager::TerrainChange(float3 upperCorner, float3 lowerCorner) {
 Tells estimators about changes in or on the map.
 */
 void CPathManager::TerrainChange(unsigned int x1, unsigned int z1, unsigned int x2, unsigned int z2) {
-//	LogObject() << "Terrain changed: (" << int(x1) << int(z1) << int(x2) << int(z2) << "\n";	//Debug
 	pe->MapChanged(x1, z1, x2, z2);
 	pe2->MapChanged(x1, z1, x2, z2);
 }
 
 
-/*
-Runned every 1/30sec during runtime.
-*/
+
 void CPathManager::Update()
 {
 	SCOPED_TIMER("PFS Update");
@@ -389,61 +413,39 @@ void CPathManager::Update()
 	pe2->Update();
 }
 
-
-/*
-Draw path-lines for each path in pathmap.
-*/
-void CPathManager::Draw() {
-	glDisable(GL_TEXTURE_2D);
-	glDisable(GL_LIGHTING);
-	glLineWidth(3);
-	std::map<unsigned int, MultiPath*>::iterator pi;
-	for(pi = pathMap.begin(); pi != pathMap.end(); pi++) {
-		MultiPath* path = pi->second;
-		IPath::path_list_type::iterator pvi;
-
-		//Start drawing a line.
-		glBegin(GL_LINE_STRIP);
-
-		//Drawing estimatePath2.
-		glColor4f(0,0,1,1);
-		for(pvi = path->estimatedPath2.path.begin(); pvi != path->estimatedPath2.path.end(); pvi++) {
-			float3 pos = *pvi;
-			pos.y += 5;
-			glVertexf3(pos);
-		}
-
-		//Drawing estimatePath.
-		glColor4f(0,1,0,1);
-		for(pvi = path->estimatedPath.path.begin(); pvi != path->estimatedPath.path.end(); pvi++) {
-			float3 pos = *pvi;
-			pos.y += 5;
-			glVertexf3(pos);
-		}
-
-		//Drawing detailPath
-		glColor4f(1,0,0,1);
-		for(pvi = path->detailedPath.path.begin(); pvi != path->detailedPath.path.end(); pvi++) {
-			float3 pos = *pvi;
-			pos.y += 5;
-			glVertexf3(pos);
-		}
-
-		//Stop drawing a line.
-		glEnd();
-
-		//Tell the CPathFinderDef to visualise it self.
-		path->peDef->Draw();
+// used to deposit heat on the heat-map as a unit moves along its path
+void CPathManager::UpdatePath(const CSolidObject* owner, unsigned int pathId)
+{
+	if (!pathId) {
+		return;
 	}
-	glLineWidth(1);
-#if !defined(USE_GML) || !GML_ENABLE_SIM // making this threadsafe might be too costly performance wise
-	pf->Draw();
-	pe->Draw();
-	pe2->Draw();
+
+#ifndef USE_GML
+	static std::vector<int2> points;
+#else
+	std::vector<int2> points;
 #endif
+
+	GetDetailedPathSquares(pathId, points);
+
+	float scale = 1.0f / points.size();
+	unsigned int i = points.size();
+
+	for (std::vector<int2>::const_iterator it = points.begin(); it != points.end(); ++it) {
+		SetHeatOnSquare(it->x, it->y, i * scale * owner->mobility->heatProduced, owner->id); i--;
+	}
 }
 
 
+
+void CPathManager::SetHeatMappingEnabled(bool enabled) { pf->SetHeatMapState(enabled); }
+bool CPathManager::GetHeatMappingEnabled() { return pf->GetHeatMapState(); }
+
+void CPathManager::SetHeatOnSquare(int x, int y, int value, int ownerId) { pf->UpdateHeatValue(x, y, value, ownerId); }
+const int CPathManager::GetHeatOnSquare(int x, int y) { return pf->GetHeatValue(x, y); }
+
+
+
 void CPathManager::GetDetailedPath(unsigned pathId, std::vector<float3>& points) const
 {
 	points.clear();
@@ -525,22 +527,10 @@ void CPathManager::GetEstimatedPath(unsigned int pathId,
 }
 
 
-void CPathManager::SetHeatOnPos(float3 pos, int value, int ownerId)
-{
-	int x = (int)pos.x/SQUARE_SIZE;
-	int y = (int)pos.z/SQUARE_SIZE;
-	SetHeatOnSquare(x, y, value, ownerId);
-}
 
-void CPathManager::SetHeatOnSquare(int x, int y, int value, int ownerId)
+boost::uint32_t CPathManager::GetPathCheckSum()
 {
-	pf->UpdateHeatValue(x, y, value, ownerId);
-}
-
-
-boost::uint32_t CPathManager::GetPathChecksum()
-{
-	return pe->GetPathChecksum() + pe2->GetPathChecksum();
+	return (pe->GetPathChecksum() + pe2->GetPathChecksum());
 }
 
 
