@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef __BUBBLE_PROJECTILE_H__
 #define __BUBBLE_PROJECTILE_H__
 
@@ -8,7 +10,7 @@ class CBubbleProjectile :
 {
 	CR_DECLARE(CBubbleProjectile);
 public:
-	CBubbleProjectile(float3 pos,float3 speed,float ttl,float startSize,float sizeExpansion, CUnit* owner,float alpha GML_PARG_H);
+	CBubbleProjectile(float3 pos,float3 speed,float ttl,float startSize,float sizeExpansion, CUnit* owner,float alpha);
 	virtual ~CBubbleProjectile();
 	void Update();
 	void Draw();
