@@ -1,24 +1,28 @@
-#ifndef __GENERIC_PARTICLE_PROJECTILE_H__
-#define __GENERIC_PARTICLE_PROJECTILE_H__
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
+#ifndef GENERIC_PARTICLE_PROJECTILE_H
+#define GENERIC_PARTICLE_PROJECTILE_H
 
 #include "Sim/Projectiles/Projectile.h"
 #include "Rendering/Textures/TextureAtlas.h"
 
 class CColorMap;
+
 class CGenericParticleProjectile : public CProjectile
 {
 	CR_DECLARE(CGenericParticleProjectile);
-public:
-	CGenericParticleProjectile(const float3& pos, const float3& speed, CUnit* owner GML_PARG_H);
-	~CGenericParticleProjectile(void);
 
-	virtual void Update();
+public:
+	CGenericParticleProjectile(const float3& pos, const float3& speed, CUnit* owner);
+	~CGenericParticleProjectile();
+
 	virtual void Draw();
+	virtual void Update();
 
 	float3 gravity;
 
-	AtlasedTexture *texture;
-	CColorMap *colorMap;
+	AtlasedTexture* texture;
+	CColorMap* colorMap;
 	bool directional;
 
 	float life;
@@ -30,4 +34,4 @@ public:
 	float sizeMod;
 };
 
-#endif
+#endif // GENERIC_PARTICLE_PROJECTILE_H
