@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef LARGEBEAMLASERPROJECTILE_H
 #define LARGEBEAMLASERPROJECTILE_H
 
@@ -9,7 +11,7 @@ class CLargeBeamLaserProjectile : public CWeaponProjectile
 	CR_DECLARE(CLargeBeamLaserProjectile);
 public:
 	CLargeBeamLaserProjectile(const float3& startPos, const float3& endPos, const float3& color, const float3& color2,
-		CUnit* owner, const WeaponDef* weaponDef GML_PARG_H);
+		CUnit* owner, const WeaponDef* weaponDef);
 	~CLargeBeamLaserProjectile(void);
 
 	float3 startPos;
