@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
@@ -17,11 +19,11 @@
 #include "LogOutput.h"
 #include "Map/Ground.h"
 #include "Map/MapDamage.h"
+#include "Rendering/UnitDrawer.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/glExtra.h"
 #include "Rendering/GL/VertexArray.h"
-#include "Rendering/UnitModels/3DModel.h"
-#include "Rendering/UnitModels/UnitDrawer.h"
+#include "Rendering/Models/3DModel.h"
 #include "Lua/LuaRules.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Features/FeatureHandler.h"
@@ -154,10 +156,11 @@ CBuilderCAI::CBuilderCAI(CUnit* owner):
 		c.tooltip = "Capture: Captures a unit from the enemy";
 		possibleCommands.push_back(c);
 	}
-	CBuilder* fac = (CBuilder*)owner;
+
+	CBuilder* builder = (CBuilder*) owner;
 
 	map<int, string>::const_iterator bi;
-	for (bi = fac->unitDef->buildOptions.begin(); bi != fac->unitDef->buildOptions.end(); ++bi) {
+	for (bi = builder->unitDef->buildOptions.begin(); bi != builder->unitDef->buildOptions.end(); ++bi) {
 		const string name = bi->second;
 		const UnitDef* ud = unitDefHandler->GetUnitDefByName(name);
 		if (ud == NULL) {
@@ -229,29 +232,22 @@ inline bool CBuilderCAI::ObjInBuildRange(const CWorldObject* obj) const
 inline bool CBuilderCAI::OutOfImmobileRange(const Command& cmd) const
 {
 	if (owner->unitDef->canmove) {
-		return false; // unit can move
+		return false; // builder can move
 	}
 	if (((cmd.options & INTERNAL_ORDER) == 0) || (cmd.params.size() != 1)) {
 		return false; // not an internal object targetted command
 	}
 
-	const int id = (int)cmd.params[0];
-	CWorldObject* obj = NULL;
-	if (id < 0) {
-		return false;
-	}
-	else if (static_cast<unsigned int>(id) < uh->MaxUnits()) {
-		obj = uh->units[id];
-	}
-	else {
-		// features don't move, but maybe the unit was transported?
-		CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
-		if (feature) {
-			obj = feature;
-		}
-	}
+	const int objID = cmd.params[0];
+	const CWorldObject* obj = uh->GetUnit(objID);
+
 	if (obj == NULL) {
-		return false;
+		// features don't move, but maybe the unit was transported?
+		obj = featureHandler->GetFeature(objID - uh->MaxUnits());
+
+		if (obj == NULL) {
+			return false;
+		}
 	}
 
 	switch (cmd.id) {
@@ -303,11 +299,10 @@ void CBuilderCAI::GiveCommandReal(const Command& c, bool fromSynced)
 		return;
 	}
 
-	if(!(c.options & SHIFT_KEY) && nonQueingCommands.find(c.id)==nonQueingCommands.end()
+	if (!(c.options & SHIFT_KEY) && nonQueingCommands.find(c.id) == nonQueingCommands.end()
 			&& c.id != CMD_WAIT) {
-		building=false;
-		CBuilder* fac=(CBuilder*)owner;
-		fac->StopBuild();
+		building = false;
+		((CBuilder*) owner)->StopBuild();
 	}
 
 	map<int,string>::iterator boi = buildOptions.find(c.id);
@@ -361,6 +356,8 @@ void CBuilderCAI::GiveCommandReal(const Command& c, bool fromSynced)
 
 void CBuilderCAI::SlowUpdate()
 {
+	if(gs->paused) // Commands issued may invoke SlowUpdate when paused
+		return;
 	if (commandQue.empty()) {
 		CMobileCAI::SlowUpdate();
 		return;
@@ -441,7 +438,7 @@ void CBuilderCAI::SlowUpdate()
 								ReclaimFeature(f);
 							}
 							else {
-								if ((owner->team == gu->myTeam) && !(buildRetries & 7)) {
+								if ((owner->team == gu->myTeam) && !(buildRetries & 7) && gu->buildWarnings) {
 									logOutput.Print(
 										"%s: build-position <%.2f, %.2f, %.2f> blocked after %d attempts",
 										owner->unitDef->humanName.c_str(),
@@ -553,23 +550,21 @@ void CBuilderCAI::ExecuteStop(Command& c)
 
 void CBuilderCAI::ExecuteRepair(Command& c)
 {
-	CBuilder* fac = (CBuilder*) owner;
+	CBuilder* builder = (CBuilder*) owner;
 	assert(owner->unitDef->canRepair || owner->unitDef->canAssist);
 
 	if (c.params.size() == 1 || c.params.size() == 5) {
 		// repair unit
-		unsigned int uid = (unsigned int) c.params[0];
-		CUnit* unit = NULL;
+		CUnit* unit = uh->GetUnit(c.params[0]);
 
-		if (uid >= uh->MaxUnits()) {
+		if (unit == NULL) {
+			FinishCommand();
 			return;
 		}
 
-		unit = uh->units[uid];
-
 		if (tempOrder && owner->moveState == 1) {
 			// limit how far away we go
-			if (unit && LinePointDist(commandPos1, commandPos2, unit->pos) > 500) {
+			if (LinePointDist(commandPos1, commandPos2, unit->pos) > 500) {
 				StopMove();
 				FinishCommand();
 				return;
@@ -580,8 +575,8 @@ void CBuilderCAI::ExecuteRepair(Command& c)
 			const float3 pos(c.params[1], c.params[2], c.params[3]);
 			const float radius = c.params[4] + 100.0f; // do not walk too far outside repair area
 
-			if (unit && ((pos - unit->pos).SqLength2D() > radius * radius ||
-				(fac->curBuild == unit && unit->isMoving && !ObjInBuildRange(unit)))) {
+			if (((pos - unit->pos).SqLength2D() > radius * radius ||
+				(builder->curBuild == unit && unit->isMoving && !ObjInBuildRange(unit)))) {
 				StopMove();
 				FinishCommand();
 				return;
@@ -590,20 +585,20 @@ void CBuilderCAI::ExecuteRepair(Command& c)
 
 		// don't consider units under construction irreparable
 		// even if they can be repaired
-		if (unit && (unit->beingBuilt || unit->unitDef->repairable)
+		if ((unit->beingBuilt || unit->unitDef->repairable)
 		    && (unit->health < unit->maxHealth) &&
 		    ((unit != owner) || owner->unitDef->canSelfRepair) &&
 		    (!unit->soloBuilder || (unit->soloBuilder == owner)) &&
 			(!(c.options & INTERNAL_ORDER) || (c.options & CONTROL_KEY) || !IsUnitBeingReclaimed(unit, owner)) &&
-		    UpdateTargetLostTimer(uid)) {
+		    UpdateTargetLostTimer(unit->id)) {
 
-			if (f3SqDist(unit->pos, fac->pos) < Square(fac->buildDistance + unit->radius - 8.0f)) {
+			if (f3SqDist(unit->pos, builder->pos) < Square(builder->buildDistance + unit->radius - 8.0f)) {
 				StopMove();
-				fac->SetRepairTarget(unit);
-				owner->moveType->KeepPointingTo(unit->pos, fac->buildDistance * 0.9f + unit->radius, false);
+				builder->SetRepairTarget(unit);
+				owner->moveType->KeepPointingTo(unit->pos, builder->buildDistance * 0.9f + unit->radius, false);
 			} else {
 				if (f3SqDist(goalPos, unit->pos) > 1.0f) {
-					SetGoal(unit->pos, owner->pos, fac->buildDistance * 0.9f + unit->radius);
+					SetGoal(unit->pos, owner->pos, builder->buildDistance * 0.9f + unit->radius);
 				}
 			}
 		} else {
@@ -615,7 +610,7 @@ void CBuilderCAI::ExecuteRepair(Command& c)
 		const float3 pos(c.params[0], c.params[1], c.params[2]);
 		const float radius = c.params[3];
 
-		fac->StopBuild();
+		builder->StopBuild();
 		if (FindRepairTargetAndRepair(pos, radius, c.options, false, (c.options & META_KEY))) {
 			inCommand = false;
 			SlowUpdate();
@@ -634,31 +629,37 @@ void CBuilderCAI::ExecuteRepair(Command& c)
 void CBuilderCAI::ExecuteCapture(Command& c)
 {
 	assert(owner->unitDef->canCapture);
-	CBuilder* fac = (CBuilder*) owner;
+	CBuilder* builder = (CBuilder*) owner;
 
 	if (c.params.size() == 1 || c.params.size() == 5) {
 		// capture unit
-		CUnit* unit = uh->units[(int)c.params[0]];
+		CUnit* unit = uh->GetUnit(c.params[0]);
+
+		if (unit == NULL) {
+			FinishCommand();
+			return;
+		}
 
 		if (c.params.size() == 5) {
 			const float3 pos(c.params[1], c.params[2], c.params[3]);
 			const float radius = c.params[4] + 100; // do not walk too far outside capture area
 
-			if (unit && ((pos - unit->pos).SqLength2D() > (radius * radius) ||
-				(fac->curCapture == unit && unit->isMoving && !ObjInBuildRange(unit)))) {
+			if (((pos - unit->pos).SqLength2D() > (radius * radius) ||
+				(builder->curCapture == unit && unit->isMoving && !ObjInBuildRange(unit)))) {
 				StopMove();
 				FinishCommand();
 				return;
 			}
 		}
-		if (unit && unit->unitDef->capturable && unit->team != owner->team && UpdateTargetLostTimer(unit->id)) {
-			if (f3SqDist(unit->pos, fac->pos) < Square(fac->buildDistance + unit->radius - 8)) {
+
+		if (unit->unitDef->capturable && unit->team != owner->team && UpdateTargetLostTimer(unit->id)) {
+			if (f3SqDist(unit->pos, builder->pos) < Square(builder->buildDistance + unit->radius - 8)) {
 				StopMove();
-				fac->SetCaptureTarget(unit);
-				owner->moveType->KeepPointingTo(unit->pos, fac->buildDistance * 0.9f + unit->radius, false);
+				builder->SetCaptureTarget(unit);
+				owner->moveType->KeepPointingTo(unit->pos, builder->buildDistance * 0.9f + unit->radius, false);
 			} else {
 				if (f3SqDist(goalPos, unit->pos) > 1) {
-					SetGoal(unit->pos, owner->pos, fac->buildDistance * 0.9f + unit->radius);
+					SetGoal(unit->pos, owner->pos, builder->buildDistance * 0.9f + unit->radius);
 				}
 			}
 		} else {
@@ -670,7 +671,7 @@ void CBuilderCAI::ExecuteCapture(Command& c)
 		const float3 pos(c.params[0], c.params[1], c.params[2]);
 		const float radius = c.params[3];
 
-		fac->StopBuild();
+		builder->StopBuild();
 
 		if (FindCaptureTargetAndCapture(pos, radius, c.options, (c.options & META_KEY))) {
 			inCommand = false;
@@ -690,66 +691,71 @@ void CBuilderCAI::ExecuteCapture(Command& c)
 void CBuilderCAI::ExecuteGuard(Command& c)
 {
 	assert(owner->unitDef->canGuard);
-	CBuilder* fac = (CBuilder*) owner;
-	CUnit* guarded = uh->units[(int)c.params[0]];
+	CBuilder* builder = (CBuilder*) owner;
+	CUnit* guarded = uh->GetUnit(c.params[0]);
 
-	if (guarded && guarded != owner && UpdateTargetLostTimer((int)c.params[0])) {
+	if (guarded != NULL && guarded != owner && UpdateTargetLostTimer(guarded->id)) {
 		if (CBuilder* b = dynamic_cast<CBuilder*>(guarded)) {
 			if (b->terraforming) {
-				if (f3SqDist(fac->pos, b->terraformCenter) <
-						Square((fac->buildDistance * 0.8f) + (b->terraformRadius * 0.7f))) {
+				if (f3SqDist(builder->pos, b->terraformCenter) <
+						Square((builder->buildDistance * 0.8f) + (b->terraformRadius * 0.7f))) {
 					StopMove();
-					owner->moveType->KeepPointingTo(b->terraformCenter, fac->buildDistance*0.9f, false);
-					fac->HelpTerraform(b);
+					owner->moveType->KeepPointingTo(b->terraformCenter, builder->buildDistance * 0.9f, false);
+					builder->HelpTerraform(b);
 				} else {
 					StopSlowGuard();
-					SetGoal(b->terraformCenter,fac->pos,fac->buildDistance*0.7f+b->terraformRadius*0.6f);
+					SetGoal(b->terraformCenter, builder->pos, builder->buildDistance * 0.7f + b->terraformRadius * 0.6f);
 				}
 				return;
-			} else if (b->curReclaim && owner->unitDef->canReclaim){
+			} else if (b->curReclaim && owner->unitDef->canReclaim) {
 				StopSlowGuard();
-				if(!ReclaimObject(b->curReclaim)){
+				if (!ReclaimObject(b->curReclaim)) {
 					StopMove();
 				}
 				return;
-			} else if (b->curResurrect && owner->unitDef->canResurrect){
+			} else if (b->curResurrect && owner->unitDef->canResurrect) {
 				StopSlowGuard();
-				if(!ResurrectObject(b->curResurrect)){
+				if (!ResurrectObject(b->curResurrect)) {
 					StopMove();
 				}
 				return;
 			} else {
-				fac->StopBuild();
+				builder->StopBuild();
 			}
+
 			if (b->curBuild &&
 			    (!b->curBuild->soloBuilder || (b->curBuild->soloBuilder == owner)) &&
 			    (( b->curBuild->beingBuilt && owner->unitDef->canAssist) ||
 			     (!b->curBuild->beingBuilt && owner->unitDef->canRepair))) {
 				StopSlowGuard();
+
 				Command nc;
-				nc.id=CMD_REPAIR;
-				nc.options=c.options;
-				nc.params.push_back(b->curBuild->id);
+					nc.id = CMD_REPAIR;
+					nc.options = c.options;
+					nc.params.push_back(b->curBuild->id);
+
 				commandQue.push_front(nc);
-				inCommand=false;
+				inCommand = false;
 				SlowUpdate();
 				return;
 			}
 		}
 
-		if (CFactory* f = dynamic_cast<CFactory*>(guarded)) {
-			if (f->curBuild &&
-			    (!f->curBuild->soloBuilder || (f->curBuild->soloBuilder == owner)) &&
-			    (( f->curBuild->beingBuilt && owner->unitDef->canAssist) ||
-			     (!f->curBuild->beingBuilt && owner->unitDef->canRepair))) {
+		if (CFactory* fac = dynamic_cast<CFactory*>(guarded)) {
+			if (fac->curBuild &&
+			    (!fac->curBuild->soloBuilder || (fac->curBuild->soloBuilder == owner)) &&
+			    (( fac->curBuild->beingBuilt && owner->unitDef->canAssist) ||
+			     (!fac->curBuild->beingBuilt && owner->unitDef->canRepair))) {
 				StopSlowGuard();
+
 				Command nc;
-				nc.id=CMD_REPAIR;
-				nc.options=c.options;
-				nc.params.push_back(f->curBuild->id);
+					nc.id = CMD_REPAIR;
+					nc.options = c.options;
+					nc.params.push_back(fac->curBuild->id);
+
 				commandQue.push_front(nc);
-				inCommand=false;
-				//					SlowUpdate();
+				inCommand = false;
+				// SlowUpdate();
 				return;
 			}
 		}
@@ -760,20 +766,22 @@ void CBuilderCAI::ExecuteGuard(Command& c)
 		float3 curPos = owner->pos;
 		float3 dif = guarded->pos - curPos;
 		dif.Normalize();
-		float3 goal = guarded->pos - dif * (fac->buildDistance * 0.5f);
+		float3 goal = guarded->pos - dif * (builder->buildDistance * 0.5f);
+
 		if (f3SqLen(guarded->pos - curPos) >
-				(fac->buildDistance*1.1f + guarded->radius) *
-				(fac->buildDistance*1.1f + guarded->radius)) {
+				(builder->buildDistance * 1.1f + guarded->radius) *
+				(builder->buildDistance * 1.1f + guarded->radius)) {
 			StopSlowGuard();
 		}
 		if (f3SqLen(guarded->pos - curPos) <
-				(fac->buildDistance*0.9f + guarded->radius) *
-				(fac->buildDistance*0.9f + guarded->radius)) {
+				(builder->buildDistance * 0.9f + guarded->radius) *
+				(builder->buildDistance * 0.9f + guarded->radius)) {
 			StartSlowGuard(guarded->maxSpeed);
 			StopMove();
-			//				logOutput.Print("should point with type 3?");
+
 			owner->moveType->KeepPointingTo(guarded->pos,
-				fac->buildDistance*0.9f+guarded->radius, false);
+				builder->buildDistance * 0.9f + guarded->radius, false);
+
 			if ((guarded->health < guarded->maxHealth) &&
 			    (guarded->unitDef->repairable) &&
 			    (!guarded->soloBuilder || (guarded->soloBuilder == owner)) &&
@@ -782,11 +790,12 @@ void CBuilderCAI::ExecuteGuard(Command& c)
 				StopSlowGuard();
 
 				Command nc;
-				nc.id=CMD_REPAIR;
-				nc.options=c.options;
-				nc.params.push_back(guarded->id);
+					nc.id = CMD_REPAIR;
+					nc.options = c.options;
+					nc.params.push_back(guarded->id);
+
 				commandQue.push_front(nc);
-				inCommand=false;
+				inCommand = false;
 				return;
 			} else {
 				NonMoving();
@@ -808,19 +817,24 @@ void CBuilderCAI::ExecuteGuard(Command& c)
 
 void CBuilderCAI::ExecuteReclaim(Command& c)
 {
-	CBuilder* fac=(CBuilder*)owner;
+	CBuilder* builder = (CBuilder*) owner;
 	assert(owner->unitDef->canReclaim);
+
 	if (c.params.size() == 1 || c.params.size() == 5) {
 		const int signedId = (int) c.params[0];
+
 		if (signedId < 0) {
 			logOutput.Print("Trying to reclaim unit or feature with id < 0 (%i), aborting.", signedId);
 			return;
 		}
-		const unsigned int id = signedId;
-		if (id >= uh->MaxUnits()) { // reclaim feature
-		CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
-			if (feature) {
-				if(((c.options & INTERNAL_ORDER) && !(c.options & CONTROL_KEY) && IsFeatureBeingResurrected(feature->id, owner)) ||
+
+		const unsigned int uid = signedId;
+
+		if (uid >= uh->MaxUnits()) { // reclaim feature
+			CFeature* feature = featureHandler->GetFeature(uid - uh->MaxUnits());
+
+			if (feature != NULL) {
+				if (((c.options & INTERNAL_ORDER) && !(c.options & CONTROL_KEY) && IsFeatureBeingResurrected(feature->id, owner)) ||
 					!ReclaimObject(feature)) {
 					StopMove();
 					FinishCommand();
@@ -834,15 +848,22 @@ void CBuilderCAI::ExecuteReclaim(Command& c)
 				RemoveUnitFromFeatureReclaimers(owner);
 			}
 			RemoveUnitFromReclaimers(owner);
+		} else { // reclaim unit
+			CUnit* unit = uh->GetUnitUnsafe(uid);
 
-		} else {                  // reclaim unit
-			CUnit* unit = uh->units[id];
-			if(c.params.size() == 5) {
-				float3 pos(c.params[1], c.params[2], c.params[3]);
-				float radius=c.params[4]+100; // do not walk too far outside reclaim area
-				if(unit && ((pos-unit->pos).SqLength2D()>radius*radius ||
-					(fac->curReclaim == unit && unit->isMoving && !ObjInBuildRange(unit)) ||
-					(unit->unitDef->builder && !unit->commandAI->commandQue.empty() && teamHandler->Ally(owner->allyteam, unit->allyteam)))) {
+			if (unit != NULL && c.params.size() == 5) {
+				const float3 pos(c.params[1], c.params[2], c.params[3]);
+				const float radius = c.params[4] + 100.0f; // do not walk too far outside reclaim area
+
+				const bool outOfReclaimRange =
+					((pos - unit->pos).SqLength2D() > radius * radius) ||
+					(builder->curReclaim == unit && unit->isMoving && !ObjInBuildRange(unit));
+				const bool busyAlliedBuilder =
+					unit->unitDef->builder &&
+					!unit->commandAI->commandQue.empty() &&
+					teamHandler->Ally(owner->allyteam, unit->allyteam);
+
+				if (outOfReclaimRange || busyAlliedBuilder) {
 					StopMove();
 					RemoveUnitFromReclaimers(owner);
 					FinishCommand();
@@ -850,8 +871,9 @@ void CBuilderCAI::ExecuteReclaim(Command& c)
 					return;
 				}
 			}
-			if(unit && unit!=owner && unit->unitDef->reclaimable && UpdateTargetLostTimer(id) && unit->AllowedReclaim(owner) ){
-				if(!ReclaimObject(unit)){
+
+			if (unit != NULL && unit != owner && unit->unitDef->reclaimable && UpdateTargetLostTimer(unit->id) && unit->AllowedReclaim(owner)) {
+				if (!ReclaimObject(unit)) {
 					StopMove();
 					FinishCommand();
 				} else {
@@ -861,17 +883,22 @@ void CBuilderCAI::ExecuteReclaim(Command& c)
 				RemoveUnitFromReclaimers(owner);
 				FinishCommand();
 			}
+
 			RemoveUnitFromFeatureReclaimers(owner);
 		}
 	} else if (c.params.size() == 4) {
 		// area reclaim
 		const float3 pos(c.params[0], c.params[1], c.params[2]);
 		const float radius = c.params[3];
-		const bool recUnits = ((c.options & META_KEY) != 0);
+		const bool recUnits = !!(c.options & META_KEY);
+		const bool recEnemyOnly = (c.options & META_KEY) && (c.options & CONTROL_KEY);
+		const bool recSpecial = !!(c.options & CONTROL_KEY);
+
 		RemoveUnitFromReclaimers(owner);
 		RemoveUnitFromFeatureReclaimers(owner);
-		fac->StopBuild();
-		if (FindReclaimTargetAndReclaim(pos, radius, c.options, true, true, recUnits, false, false)) {
+		builder->StopBuild();
+
+		if (FindReclaimTargetAndReclaim(pos, radius, c.options, true, recUnits, false, false, recEnemyOnly, recSpecial)) {
 			inCommand=false;
 			SlowUpdate();
 			return;
@@ -890,16 +917,17 @@ void CBuilderCAI::ExecuteReclaim(Command& c)
 
 
 bool CBuilderCAI::ResurrectObject(CFeature *feature) {
-	CBuilder* fac=(CBuilder*)owner;
-	if (f3SqDist(feature->pos, fac->pos) < Square(fac->buildDistance*0.9f+feature->radius)) {
+	CBuilder* builder = (CBuilder*) owner;
+
+	if (f3SqDist(feature->pos, builder->pos) < Square(builder->buildDistance * 0.9f + feature->radius)) {
 		StopMove();
-		owner->moveType->KeepPointingTo(feature->pos, fac->buildDistance*0.9f+feature->radius, false);
-		fac->SetResurrectTarget(feature);
+		owner->moveType->KeepPointingTo(feature->pos, builder->buildDistance * 0.9f + feature->radius, false);
+		builder->SetResurrectTarget(feature);
 	} else {
 		if (f3SqDist(goalPos, feature->pos) > 1) {
-			SetGoal(feature->pos,owner->pos, fac->buildDistance*0.8f+feature->radius);
+			SetGoal(feature->pos,owner->pos, builder->buildDistance * 0.8f + feature->radius);
 		} else {
-			if(owner->moveType->progressState==AMoveType::Failed){
+			if (owner->moveType->progressState == AMoveType::Failed) {
 				return false;
 			}
 		}
@@ -911,13 +939,16 @@ bool CBuilderCAI::ResurrectObject(CFeature *feature) {
 void CBuilderCAI::ExecuteResurrect(Command& c)
 {
 	assert(owner->unitDef->canResurrect);
-	CBuilder* fac=(CBuilder*)owner;
-	if (c.params.size()==1) {
+	CBuilder* builder = (CBuilder*) owner;
+
+	if (c.params.size() == 1) {
 		unsigned int id = (unsigned int) c.params[0];
+
 		if (id >= uh->MaxUnits()) { // resurrect feature
-		CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
+			CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
+
 			if (feature && feature->createdFromUnit != "") {
-				if(((c.options & INTERNAL_ORDER) && !(c.options & CONTROL_KEY) && IsFeatureBeingReclaimed(feature->id, owner)) ||
+				if (((c.options & INTERNAL_ORDER) && !(c.options & CONTROL_KEY) && IsFeatureBeingReclaimed(feature->id, owner)) ||
 					!ResurrectObject(feature)) {
 					StopMove();
 					RemoveUnitFromResurrecters(owner);
@@ -929,13 +960,13 @@ void CBuilderCAI::ExecuteResurrect(Command& c)
 			} else {
 				RemoveUnitFromResurrecters(owner);
 
-				if (fac->lastResurrected && uh->units[fac->lastResurrected] && owner->unitDef->canRepair) {
+				if (builder->lastResurrected && uh->GetUnitUnsafe(builder->lastResurrected) != NULL && owner->unitDef->canRepair) {
 					// resurrection finished, start repair
 					c.id = CMD_REPAIR; // kind of hackery to overwrite the current order i suppose
 					c.params.clear();
-					c.params.push_back(fac->lastResurrected);
+					c.params.push_back(builder->lastResurrected);
 					c.options |= INTERNAL_ORDER;
-					fac->lastResurrected = 0;
+					builder->lastResurrected = 0;
 					inCommand = false;
 					SlowUpdate();
 					return;
@@ -972,22 +1003,25 @@ void CBuilderCAI::ExecuteResurrect(Command& c)
 void CBuilderCAI::ExecutePatrol(Command& c)
 {
 	assert(owner->unitDef->canPatrol);
-	if(c.params.size()<3){		//this shouldnt happen but anyway ...
+	if (c.params.size() < 3) {
+		// this shouldnt happen but anyway ...
 		logOutput.Print("Error: got patrol cmd with less than 3 params on %s in buildercai",
 			owner->unitDef->humanName.c_str());
 		return;
 	}
+
 	Command temp;
-	temp.id = CMD_FIGHT;
-	temp.params.push_back(c.params[0]);
-	temp.params.push_back(c.params[1]);
-	temp.params.push_back(c.params[2]);
-	temp.options = c.options|INTERNAL_ORDER;
+		temp.id = CMD_FIGHT;
+		temp.params.push_back(c.params[0]);
+		temp.params.push_back(c.params[1]);
+		temp.params.push_back(c.params[2]);
+		temp.options = c.options|INTERNAL_ORDER;
+
 	commandQue.push_back(c);
 	commandQue.pop_front();
 	commandQue.push_front(temp);
 	Command tmpC;
-	tmpC.id = CMD_PATROL;
+		tmpC.id = CMD_PATROL;
 	eoh->CommandFinished(*owner, tmpC);
 	SlowUpdate();
 }
@@ -996,18 +1030,20 @@ void CBuilderCAI::ExecutePatrol(Command& c)
 void CBuilderCAI::ExecuteFight(Command& c)
 {
 	assert((c.options & INTERNAL_ORDER) || owner->unitDef->canFight);
-	CBuilder* fac=(CBuilder*)owner;
-	if(tempOrder){
-		tempOrder=false;
-		inCommand=true;
+	CBuilder* builder = (CBuilder*) owner;
+
+	if (tempOrder) {
+		tempOrder = false;
+		inCommand = true;
 	}
-	if(c.params.size()<3){		//this shouldnt happen but anyway ...
+	if (c.params.size() < 3) {		//this shouldnt happen but anyway ...
 		logOutput.Print("Error: got fight cmd with less than 3 params on %s in BuilderCAI",owner->unitDef->humanName.c_str());
 		return;
 	}
-	if(c.params.size() >= 6){
-		if(!inCommand){
-			commandPos1 = float3(c.params[3],c.params[4],c.params[5]);
+
+	if (c.params.size() >= 6) {
+		if (!inCommand) {
+			commandPos1 = float3(c.params[3], c.params[4], c.params[5]);
 		}
 	} else {
 		// Some hackery to make sure the line (commandPos1,commandPos2) is NOT
@@ -1020,7 +1056,9 @@ void CBuilderCAI::ExecuteFight(Command& c)
 			commandPos1 = owner->pos;
 		}
 	}
-	float3 pos(c.params[0],c.params[1],c.params[2]);
+
+	float3 pos(c.params[0], c.params[1], c.params[2]);
+
 	if (!inCommand) {
 		inCommand = true;
 		commandPos2 = pos;
@@ -1028,12 +1066,15 @@ void CBuilderCAI::ExecuteFight(Command& c)
 	if (c.params.size() >= 6) {
 		pos = ClosestPointOnLine(commandPos1, commandPos2, owner->pos);
 	}
-	if (pos!=goalPos) {
+	if (pos != goalPos) {
 		SetGoal(pos, owner->pos);
 	}
+	const bool resurrectMode = !!(c.options & ALT_KEY);
+	const bool reclaimEnemyMode = !!(c.options & META_KEY);
+	const bool reclaimEnemyOnlyMode = (c.options & CONTROL_KEY) && (c.options & META_KEY);
 	float3 curPosOnLine = ClosestPointOnLine(commandPos1, commandPos2, owner->pos);
-	if ((owner->unitDef->canRepair || owner->unitDef->canAssist) && // Priority 1: Repair
-	    FindRepairTargetAndRepair(curPosOnLine, 300*owner->moveState+fac->buildDistance-8, c.options, true, (c.options & META_KEY))){
+	if (!reclaimEnemyOnlyMode && (owner->unitDef->canRepair || owner->unitDef->canAssist) && // Priority 1: Repair
+	    FindRepairTargetAndRepair(curPosOnLine, 300*owner->moveState+builder->buildDistance-8, c.options, true, resurrectMode)){
 		tempOrder = true;
 		inCommand = false;
 		if (lastPC1 != gs->frameNum) {  //avoid infinite loops
@@ -1042,7 +1083,7 @@ void CBuilderCAI::ExecuteFight(Command& c)
 		}
 		return;
 	}
-	if ((c.options & META_KEY) && owner->unitDef->canResurrect && // Priority 2: Resurrect (optional)
+	if (!reclaimEnemyOnlyMode && resurrectMode && owner->unitDef->canResurrect && // Priority 2: Resurrect (optional)
 	    FindResurrectableFeatureAndResurrect(curPosOnLine, 300, c.options, false)) {
 		tempOrder = true;
 		inCommand = false;
@@ -1053,7 +1094,7 @@ void CBuilderCAI::ExecuteFight(Command& c)
 		return;
 	}
 	if (owner->unitDef->canReclaim && // Priority 3: Reclaim / reclaim non resurrectable (optional) / reclaim enemy units (optional)
-	    FindReclaimTargetAndReclaim(curPosOnLine, 300, c.options, false, true, (c.options & ALT_KEY), (c.options & META_KEY), (c.options & ALT_KEY))) {
+	    FindReclaimTargetAndReclaim(curPosOnLine, 300, c.options, false, false, resurrectMode, reclaimEnemyMode, reclaimEnemyOnlyMode, false)) {
 		tempOrder = true;
 		inCommand = false;
 		if (lastPC3 != gs->frameNum) {  //avoid infinite loops
@@ -1078,23 +1119,24 @@ void CBuilderCAI::ExecuteFight(Command& c)
 void CBuilderCAI::ExecuteRestore(Command& c)
 {
 	assert(owner->unitDef->canRestore);
-	CBuilder* fac=(CBuilder*)owner;
-	if(inCommand){
-		if(!fac->terraforming){
+	CBuilder* builder = (CBuilder*) owner;
+
+	if (inCommand) {
+		if (!builder->terraforming) {
 			FinishCommand();
 		}
-	} else if(owner->unitDef->canRestore){
-		float3 pos(c.params[0],c.params[1],c.params[2]);
-		pos.y = ground->GetHeight2(pos.x,pos.y);
-		float radius(c.params[3]);
-		if (radius>200) radius = 200;
-		if (f3SqDist(fac->pos, pos) < Square(fac->buildDistance-1)) {
+	} else if (owner->unitDef->canRestore) {
+		float3 pos(c.params[0], c.params[1], c.params[2]);
+			pos.y = ground->GetHeight2(pos.x, pos.y);
+		const float radius = std::min(c.params[3], 200.0f);
+
+		if (f3SqDist(builder->pos, pos) < Square(builder->buildDistance - 1)) {
 			StopMove();
-			fac->StartRestore(pos,radius);
-			owner->moveType->KeepPointingTo(pos, fac->buildDistance*0.9f, false);
-			inCommand=true;
+			builder->StartRestore(pos, radius);
+			owner->moveType->KeepPointingTo(pos, builder->buildDistance * 0.9f, false);
+			inCommand = true;
 		} else {
-			SetGoal(pos,owner->pos, fac->buildDistance*0.9f);
+			SetGoal(pos, owner->pos, builder->buildDistance * 0.9f);
 		}
 	}
 	return;
@@ -1259,12 +1301,13 @@ bool CBuilderCAI::IsFeatureBeingResurrected(int featureId, CUnit *friendUnit)
 }
 
 
-bool CBuilderCAI::ReclaimObject(CSolidObject* object){
-	CBuilder* fac=(CBuilder*)owner;
-	if (f3SqDist(object->pos, fac->pos) < Square(fac->buildDistance-1+object->radius)) {
+bool CBuilderCAI::ReclaimObject(CSolidObject* object) {
+	CBuilder* builder = (CBuilder*) owner;
+
+	if (f3SqDist(object->pos, builder->pos) < Square(builder->buildDistance - 1 + object->radius)) {
 		StopMove();
-		owner->moveType->KeepPointingTo(object->pos, fac->buildDistance*0.9f+object->radius, false);
-		fac->SetReclaimTarget(object);
+		owner->moveType->KeepPointingTo(object->pos, builder->buildDistance * 0.9f + object->radius, false);
+		builder->SetReclaimTarget(object);
 	} else {
 		if (f3SqDist(goalPos, object->pos) > 1) {
 			SetGoal(object->pos, owner->pos);
@@ -1287,29 +1330,26 @@ bool CBuilderCAI::FindReclaimTargetAndReclaim(const float3& pos,
                                                    float radius,
                                                    unsigned char options,
                                                    bool noResCheck,
-                                                   bool recAnyTeam,
                                                    bool recUnits,
-												   bool recNonRez,
-												   bool recEnemy)
+                                                   bool recNonRez,
+                                                   bool recEnemy,
+                                                   bool recEnemyOnly,
+                                                   bool recSpecial)
 {
 	const CSolidObject* best = NULL;
 	float bestDist = 1.0e30f;
 	bool stationary = false;
+	bool metal = false;
 	int rid = -1;
 
-	if(recUnits) {
+	if(recUnits || recEnemy || recEnemyOnly) {
 		const std::vector<CUnit*> units = qf->GetUnitsExact(pos, radius);
 		for (std::vector<CUnit*>::const_iterator ui = units.begin(); ui != units.end(); ++ui) {
 			const CUnit* u = *ui;
 			if (u != owner
 			    && u->unitDef->reclaimable
-			    && (
-			        (!recEnemy && (u->team == owner->team))
-			        || (
-			            (recAnyTeam || (recEnemy && !teamHandler->Ally(owner->allyteam, u->allyteam)))
-			            && (u->losStatus[owner->allyteam] & (LOS_INRADAR|LOS_INLOS))
-			           )
-			       )
+			    && ((!recEnemy && !recEnemyOnly) || !teamHandler->Ally(owner->allyteam, u->allyteam))
+			    && (u->losStatus[owner->allyteam] & (LOS_INRADAR|LOS_INLOS))
 			   ) {
 				// do not reclaim friendly builders that are busy
 				if(!u->unitDef->builder || u->commandAI->commandQue.empty() || !teamHandler->Ally(owner->allyteam, u->allyteam)) {
@@ -1334,16 +1374,19 @@ bool CBuilderCAI::FindReclaimTargetAndReclaim(const float3& pos,
 			rid = best->id;
 	}
 
-	if(!best || !stationary) {
+	if((!best || !stationary) && !recEnemyOnly) {
 		const CTeam* team = teamHandler->Team(owner->team);
 		best = NULL;
 		const std::vector<CFeature*> features = qf->GetFeaturesExact(pos, radius);
 		for (std::vector<CFeature*>::const_iterator fi = features.begin(); fi != features.end(); ++fi) {
 			const CFeature* f = *fi;
-			if (f->def->reclaimable && ((options & CONTROL_KEY) || f->def->autoreclaim) && (!recNonRez || !(f->def->destructable && f->createdFromUnit != "")) &&
-				(recAnyTeam || (f->allyteam != owner->allyteam))) {
+			if (f->def->reclaimable && (recSpecial || f->def->autoreclaim) && 
+				(!recNonRez || !(f->def->destructable && f->createdFromUnit != ""))) {
+				if (recSpecial && metal && f->def->metal <= 0.0) {
+					continue;
+				}
 				float dist = f3SqLen(f->pos - owner->pos);
-				if ((dist < bestDist) &&
+				if ((dist < bestDist || (recSpecial && !metal && f->def->metal > 0.0)) &&
 					(noResCheck ||
 					((f->def->metal  > 0.0f) && (team->metal  < team->metalStorage)) ||
 					((f->def->energy > 0.0f) && (team->energy < team->energyStorage)))) {
@@ -1356,6 +1399,9 @@ bool CBuilderCAI::FindReclaimTargetAndReclaim(const float3& pos,
 					if(!(options & CONTROL_KEY) && IsFeatureBeingResurrected(f->id, owner)) {
 						continue;
 					}
+					if(recSpecial && !metal && f->def->metal > 0.0f) {
+						metal = true;
+					}
 					bestDist = dist;
 					best = f;
 				}
@@ -1366,18 +1412,19 @@ bool CBuilderCAI::FindReclaimTargetAndReclaim(const float3& pos,
 	}
 
 	if (rid >= 0) {
-		Command cmd;
 		if (!noResCheck) {
 			// FIGHT commands always resource check
 			PushOrUpdateReturnFight();
 		}
-		cmd.options = options | INTERNAL_ORDER;
-		cmd.id = CMD_RECLAIM;
-		cmd.params.push_back(rid);
-		cmd.params.push_back(pos.x);
-		cmd.params.push_back(pos.y);
-		cmd.params.push_back(pos.z);
-		cmd.params.push_back(radius);
+
+		Command cmd;
+			cmd.options = options | INTERNAL_ORDER;
+			cmd.id = CMD_RECLAIM;
+			cmd.params.push_back(rid);
+			cmd.params.push_back(pos.x);
+			cmd.params.push_back(pos.y);
+			cmd.params.push_back(pos.z);
+			cmd.params.push_back(radius);
 		commandQue.push_front(cmd);
 		return true;
 	}
@@ -1423,9 +1470,9 @@ bool CBuilderCAI::FindResurrectableFeatureAndResurrect(const float3& pos,
 
 	if (best) {
 		Command c2;
-		c2.options = options | INTERNAL_ORDER;
-		c2.id = CMD_RESURRECT;
-		c2.params.push_back(uh->MaxUnits() + best->id);
+			c2.options = options | INTERNAL_ORDER;
+			c2.id = CMD_RESURRECT;
+			c2.params.push_back(uh->MaxUnits() + best->id);
 		commandQue.push_front(c2);
 		return true;
 	}
@@ -1474,9 +1521,9 @@ bool CBuilderCAI::FindCaptureTargetAndCapture(const float3& pos, float radius,
 
 	if (best) {
 		Command nc;
-		nc.id = CMD_CAPTURE;
-		nc.options = options | INTERNAL_ORDER;
-		nc.params.push_back(best->id);
+			nc.id = CMD_CAPTURE;
+			nc.options = options | INTERNAL_ORDER;
+			nc.params.push_back(best->id);
 		commandQue.push_front(nc);
 		return true;
 	}
@@ -1579,25 +1626,25 @@ bool CBuilderCAI::FindRepairTargetAndRepair(const float3& pos, float radius,
 	}
 
 	if (!haveEnemy) {
-		Command cmd;
 		if (attackEnemy) {
 			PushOrUpdateReturnFight();
 		}
-		cmd.id = CMD_REPAIR;
-		cmd.options = options | INTERNAL_ORDER;
-		cmd.params.push_back(best->id);
-		cmd.params.push_back(pos.x);
-		cmd.params.push_back(pos.y);
-		cmd.params.push_back(pos.z);
-		cmd.params.push_back(radius);
+		Command cmd;
+			cmd.id = CMD_REPAIR;
+			cmd.options = options | INTERNAL_ORDER;
+			cmd.params.push_back(best->id);
+			cmd.params.push_back(pos.x);
+			cmd.params.push_back(pos.y);
+			cmd.params.push_back(pos.z);
+			cmd.params.push_back(radius);
 		commandQue.push_front(cmd);
 	}
 	else {
-		Command cmd;
 		PushOrUpdateReturnFight(); // attackEnemy must be true
-		cmd.id = CMD_ATTACK;
-		cmd.options = options | INTERNAL_ORDER;
-		cmd.params.push_back(best->id);
+		Command cmd;
+			cmd.id = CMD_ATTACK;
+			cmd.options = options | INTERNAL_ORDER;
+			cmd.params.push_back(best->id);
 		commandQue.push_front(cmd);
 	}
 
@@ -1676,10 +1723,10 @@ void CBuilderCAI::DrawCommands(void)
 				break;
 			}
 			case CMD_GUARD: {
-				const CUnit* unit = uh->units[int(ci->params[0])];
-				if((unit != NULL) && isTrackable(unit)) {
-					const float3 endPos =
-						helper->GetUnitErrorPos(unit, owner->allyteam);
+				const CUnit* unit = uh->GetUnit(ci->params[0]);
+
+				if ((unit != NULL) && isTrackable(unit)) {
+					const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
 					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.guard);
 				}
 				break;
@@ -1696,10 +1743,10 @@ void CBuilderCAI::DrawCommands(void)
 			case CMD_ATTACK:
 			case CMD_DGUN: {
 				if (ci->params.size() == 1) {
-					const CUnit* unit = uh->units[int(ci->params[0])];
-					if((unit != NULL) && isTrackable(unit)) {
-						const float3 endPos =
-						  helper->GetUnitErrorPos(unit, owner->allyteam);
+					const CUnit* unit = uh->GetUnit(ci->params[0]);
+
+					if ((unit != NULL) && isTrackable(unit)) {
+						const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
 						lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
 					}
 				} else {
@@ -1727,9 +1774,10 @@ void CBuilderCAI::DrawCommands(void)
 								signedId);
 						break;
 					}
-					const unsigned int id = signedId;
-					if (id >= uh->MaxUnits()) {
 
+					const unsigned int id = signedId;
+
+					if (id >= uh->MaxUnits()) {
 						GML_RECMUTEX_LOCK(feat); // DrawCommands
 
 						CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
@@ -1738,10 +1786,10 @@ void CBuilderCAI::DrawCommands(void)
 							lineDrawer.DrawLineAndIcon(ci->id, endPos, color);
 						}
 					} else {
-						const CUnit* unit = uh->units[id];
-						if((unit != NULL) && (unit != owner) && isTrackable(unit)) {
-							const float3 endPos =
-								helper->GetUnitErrorPos(unit, owner->allyteam);
+						const CUnit* unit = uh->GetUnitUnsafe(id);
+
+						if ((unit != NULL) && (unit != owner) && isTrackable(unit)) {
+							const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
 							lineDrawer.DrawLineAndIcon(ci->id, endPos, color);
 						}
 					}
@@ -1761,31 +1809,26 @@ void CBuilderCAI::DrawCommands(void)
 					lineDrawer.RestartWithColor(color);
 				} else {
 					if (ci->params.size() >= 1) {
-						const unsigned int uid = (unsigned int) ci->params[0];
-						const CUnit* unit = NULL;
+						const CUnit* unit = uh->GetUnit(ci->params[0]);
 
-						if (uid < uh->MaxUnits()) {
-							unit = uh->units[uid];
-
-							if ((unit != NULL) && isTrackable(unit)) {
-								const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
-								lineDrawer.DrawLineAndIcon(ci->id, endPos, color);
-							}
+						if ((unit != NULL) && isTrackable(unit)) {
+							const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
+							lineDrawer.DrawLineAndIcon(ci->id, endPos, color);
 						}
 					}
 				}
 				break;
 			}
-			case CMD_LOAD_ONTO:{
-				const CUnit* unit = uh->units[int(ci->params[0])];
+			case CMD_LOAD_ONTO: {
+				const CUnit* unit = uh->GetUnitUnsafe(ci->params[0]);
 				lineDrawer.DrawLineAndIcon(ci->id, unit->pos, cmdColors.load);
 				break;
 			}
-			case CMD_WAIT:{
+			case CMD_WAIT: {
 				DrawWaitIcon(*ci);
 				break;
 			}
-			case CMD_SELFD:{
+			case CMD_SELFD: {
 				lineDrawer.DrawIconAtLastPos(ci->id);
 				break;
 			}
@@ -1828,6 +1871,7 @@ void CBuilderCAI::DrawQuedBuildingSquares(void)
 	int quadcounter = 0;
 	int uwqcounter = 0;
 	int linecounter = 0;
+
 	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
 		if (buildOptions.find(ci->id) != buildOptions.end()) {
 			BuildInfo bi(*ci);
