@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
@@ -29,7 +31,7 @@
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/Weapon.h"
 #include "EventHandler.h"
-#include "LoadSaveInterface.h"
+#include "LoadSave/LoadSaveInterface.h"
 #include "LogOutput.h"
 #include "myMath.h"
 #include "creg/STL_Set.h"
@@ -75,6 +77,7 @@ CR_REG_METADATA(CCommandAI, (
 CCommandAI::CCommandAI():
 	stockpileWeapon(0),
 	lastUserCommand(-1000),
+	selfDCountdown(0),
 	lastFinishCommand(0),
 	owner(owner),
 	orderTarget(0),
@@ -90,6 +93,7 @@ CCommandAI::CCommandAI():
 CCommandAI::CCommandAI(CUnit* owner):
 	stockpileWeapon(0),
 	lastUserCommand(-1000),
+	selfDCountdown(0),
 	lastFinishCommand(0),
 	owner(owner),
 	orderTarget(0),
@@ -335,16 +339,34 @@ bool CCommandAI::isAttackCapable() const
 }
 
 
-static inline bool isCommandInMap(const Command& c)
-{
-	if (c.params.size() >= 3 &&
-			(c.params[0] < 0.f || c.params[2] < 0.f
-			 || c.params[0] > gs->mapx*SQUARE_SIZE
-			 || c.params[2] > gs->mapy*SQUARE_SIZE))
+
+static inline const CUnit* GetCommandUnit(const Command& c, int idx) {
+	if (idx >= c.params.size()) {
+		return NULL;
+	}
+
+	if (c.IsAreaCommand()) {
 		return false;
+	}
+
+	const CUnit* unit = uh->GetUnit(c.params[idx]);
+	return unit;
+}
+
+static inline bool IsCommandInMap(const Command& c)
+{
+	// TODO:
+	//   extend the check to commands for which
+	//   position is not stored in params[0..2]
+	if (c.params.size() >= 3) {
+		return (float3(c.params[0], c.params[1], c.params[2]).IsInBounds());
+	}
+
 	return true;
 }
 
+
+
 bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 {
 	// check if the command is in the map first
@@ -360,73 +382,113 @@ bool CCommandAI::AllowedCommand(const Command& c, bool fromSynced)
 		case CMD_FIGHT:
 		case CMD_DGUN:
 		case CMD_UNLOAD_UNIT:
-		case CMD_UNLOAD_UNITS:
-			if (!isCommandInMap(c)) { return false; }
-			break;
-		default:
+		case CMD_UNLOAD_UNITS: {
+			if (!IsCommandInMap(c)) { return false; }
+		} break;
+
+		default: {
 			// build commands
-			if (c.id < 0 && !isCommandInMap(c)) { return false; }
-			break;
+			if (c.id < 0 && !IsCommandInMap(c)) { return false; }
+		} break;
 	}
 
+
 	const UnitDef* ud = owner->unitDef;
-	int maxHeightDiff = SQUARE_SIZE;
 	// AI's may do as they like
-	CSkirmishAIHandler::ids_t saids = skirmishAIHandler.GetSkirmishAIsInTeam(owner->team);
-	const bool aiOrder = (saids.size() > 0);
+	const CSkirmishAIHandler::ids_t& saids = skirmishAIHandler.GetSkirmishAIsInTeam(owner->team);
+	const bool aiOrder = (!saids.empty());
 
 	switch (c.id) {
 		case CMD_DGUN:
-			if (!owner->unitDef->canDGun)
+			if (!ud->canDGun)
 				return false;
+
 		case CMD_ATTACK: {
 			if (!isAttackCapable())
 				return false;
 
-			if (c.params.size() == 3) {
-				// check if attack ground is really attack ground
-				if (!aiOrder && !fromSynced &&
-					fabs(c.params[1] - ground->GetHeight2(c.params[0], c.params[2])) > maxHeightDiff) {
+			if (c.params.size() == 1) {
+				const CUnit* attackee = GetCommandUnit(c, 0);
+
+				if (attackee && !attackee->pos.IsInBounds()) {
 					return false;
 				}
+			} else {
+				if (c.params.size() == 3) {
+					const float3 cPos(c.params[0], c.params[1], c.params[2]);
+
+					// check if attack ground is really attack ground
+					if (!aiOrder && !fromSynced &&
+						fabs(cPos.y - ground->GetHeight2(cPos.x, cPos.z)) > SQUARE_SIZE) {
+						return false;
+					}
+				}
 			}
 			break;
 		}
 
 		case CMD_MOVE:      if (!ud->canmove)       return false; break;
 		case CMD_FIGHT:     if (!ud->canFight)      return false; break;
-		case CMD_GUARD:     if (!ud->canGuard)      return false; break;
-		case CMD_PATROL:    if (!ud->canPatrol)     return false; break;
-		case CMD_CAPTURE:   if (!ud->canCapture)    return false; break;
-		case CMD_RECLAIM:   if (!ud->canReclaim)    return false; break;
+		case CMD_GUARD: {
+			const CUnit* guardee = GetCommandUnit(c, 0);
+
+			if (!ud->canGuard) { return false; }
+			if (owner && !owner->pos.IsInBounds()) { return false; }
+			if (guardee && !guardee->pos.IsInBounds()) { return false; }
+		} break;
+
+		case CMD_PATROL: {
+			if (!ud->canPatrol) { return false; }
+		} break;
+
+		case CMD_CAPTURE: {
+			const CUnit* capturee = GetCommandUnit(c, 0);
+
+			if (!ud->canCapture) { return false; }
+			if (capturee && !capturee->pos.IsInBounds()) { return false; }
+		} break;
+
+		case CMD_RECLAIM: {
+			const CUnit* reclaimeeUnit = GetCommandUnit(c, 0);
+			const CFeature* reclaimeeFeature = NULL;
+
+			if (!ud->canReclaim) { return false; }
+			if (reclaimeeUnit && !reclaimeeUnit->unitDef->reclaimable) { return false; }
+			if (reclaimeeUnit && !reclaimeeUnit->AllowedReclaim(owner)) { return false; }
+			if (reclaimeeUnit && !reclaimeeUnit->pos.IsInBounds()) { return false; }
+
+			if (reclaimeeUnit == NULL && !c.IsAreaCommand()) {
+				const unsigned int reclaimeeFeatureID(c.params[0]);
+
+				if (!c.params.empty() && reclaimeeFeatureID >= uh->MaxUnits()) {
+					reclaimeeFeature = featureHandler->GetFeature(reclaimeeFeatureID - uh->MaxUnits());
+
+					if (reclaimeeFeature && !reclaimeeFeature->def->reclaimable) {
+						return false;
+					}
+				}
+			}
+		} break;
+
 		case CMD_RESTORE: {
-			if (!ud->canRestore || mapDamage->disabled) return false; break;
-		}
-		case CMD_RESURRECT: if (!ud->canResurrect)  return false; break;
+			if (!ud->canRestore || mapDamage->disabled) {
+				return false;
+			}
+		} break;
+
+		case CMD_RESURRECT: {
+			if (!ud->canResurrect) { return false; }
+		} break;
+
 		case CMD_REPAIR: {
-			if (!ud->canRepair && !ud->canAssist)   return false; break;
-		}
+			const CUnit* repairee = GetCommandUnit(c, 0);
+
+			if (!ud->canRepair && !ud->canAssist) { return false; }
+			if (repairee && !repairee->pos.IsInBounds()) { return false; }
+			if (repairee && ((repairee->beingBuilt && !ud->canAssist) || (!repairee->beingBuilt && !ud->canRepair))) { return false; }
+		} break;
 	}
 
-	if ((c.id == CMD_RECLAIM) && (c.params.size() == 1 || c.params.size() == 5)) {
-		const unsigned int unitID = (unsigned int) c.params[0];
-		if (unitID < uh->MaxUnits()) { // not a feature
-			CUnit* unit = uh->units[unitID];
-			if (unit && !unit->unitDef->reclaimable)
-				return false;
-			if (unit && !unit->AllowedReclaim(owner)) return false;
-		} else {
-			const CFeature* feature = featureHandler->GetFeature(unitID - uh->MaxUnits());
-			if (feature && !feature->def->reclaimable)
-				return false;
-		}
-	}
-
-	if ((c.id == CMD_REPAIR) && (c.params.size() == 1 || c.params.size() == 5)) {
-		CUnit* unit = uh->units[(int) c.params[0]];
-		if (unit && ((unit->beingBuilt && !ud->canAssist) || (!unit->beingBuilt && !ud->canRepair)))
-			return false;
-	}
 
 	if (c.id == CMD_FIRE_STATE
 			&& (c.params.empty() || !CanChangeFireState()))
@@ -761,6 +823,9 @@ void CCommandAI::GiveWaitCommand(const Command& c)
 		}
 		eventHandler.UnitIdle(owner);
 	}
+	else {
+		SlowUpdate();
+	}
 
 	return;
 }
@@ -1108,12 +1173,14 @@ std::vector<Command> CCommandAI::GetOverlapQueued(const Command &c,
 }
 
 
-int CCommandAI::UpdateTargetLostTimer(int unitid)
+int CCommandAI::UpdateTargetLostTimer(int unitID)
 {
 	if (targetLostTimer)
 		--targetLostTimer;
 
-	if (uh->units[unitid] && (uh->units[unitid]->losStatus[owner->allyteam] & LOS_INRADAR))
+	const CUnit* unit = uh->GetUnit(unitID);
+
+	if (unit && (unit->losStatus[owner->allyteam] & LOS_INRADAR))
 		targetLostTimer = TARGET_LOST_TIMER;
 
 	return targetLostTimer;
@@ -1144,11 +1211,9 @@ void CCommandAI::ExecuteAttack(Command &c)
 		owner->commandShotCount = -1;
 
 		if (c.params.size() == 1) {
-			const unsigned int targetID = (unsigned int) c.params[0];
-			const bool legalTarget      = (targetID >= 0) && (targetID < uh->MaxUnits());
-			CUnit* targetUnit           = (legalTarget)? uh->units[targetID]: 0x0;
+			CUnit* targetUnit = uh->GetUnit(c.params[0]);
 
-			if (legalTarget && targetUnit != 0x0 && targetUnit != owner) {
+			if (targetUnit != NULL && targetUnit != owner) {
 				owner->AttackUnit(targetUnit, c.id == CMD_DGUN);
 
 				if (orderTarget)
@@ -1193,6 +1258,8 @@ void CCommandAI::ExecuteDGun(Command &c)
 
 void CCommandAI::SlowUpdate()
 {
+	if(gs->paused) // Commands issued may invoke SlowUpdate when paused
+		return;
 	if (commandQue.empty()) {
 		return;
 	}
@@ -1282,13 +1349,14 @@ void CCommandAI::DrawCommands(void)
 	}
 
 	CCommandQueue::iterator ci;
-	for(ci=commandQue.begin();ci!=commandQue.end();++ci){
-		switch(ci->id){
+	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
+		switch (ci->id) {
 			case CMD_ATTACK:
-			case CMD_DGUN:{
-				if(ci->params.size()==1){
-					const CUnit* unit = uh->units[int(ci->params[0])];
-					if((unit != NULL) && isTrackable(unit)) {
+			case CMD_DGUN: {
+				if (ci->params.size() == 1) {
+					const CUnit* unit = uh->GetUnit(ci->params[0]);
+
+					if ((unit != NULL) && isTrackable(unit)) {
 						const float3 endPos =
 							helper->GetUnitErrorPos(unit, owner->allyteam);
 						lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
@@ -1328,15 +1396,13 @@ void CCommandAI::DrawDefaultCommand(const Command& c) const
 	const int paramsCount = c.params.size();
 
 	if (paramsCount == 1) {
-		const unsigned int unitID = (unsigned int) c.params[0];
-		if (unitID < uh->MaxUnits()) {
-			const CUnit* unit = uh->units[unitID];
-			if((unit != NULL) && isTrackable(unit)) {
-				const float3 endPos =
-					helper->GetUnitErrorPos(unit, owner->allyteam);
-				lineDrawer.DrawLineAndIcon(dd->cmdIconID, endPos, dd->color);
-			}
+		const CUnit* unit = uh->GetUnit(c.params[0]);
+
+		if ((unit != NULL) && isTrackable(unit)) {
+			const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
+			lineDrawer.DrawLineAndIcon(dd->cmdIconID, endPos, dd->color);
 		}
+
 		return;
 	}
 
@@ -1560,11 +1626,12 @@ void CCommandAI::StopAttackingAllyTeam(int ally)
 
 	// erasing in the middle invalidates all iterators
 	for (CCommandQueue::iterator it = commandQue.begin(); it != commandQue.end(); ++it) {
-		const Command &c = *it;
+		const Command& c = *it;
+
 		if ((c.id == CMD_FIGHT || c.id == CMD_ATTACK) && c.params.size() == 1) {
-			int targetId = (int)c.params[0];
-			if (targetId < uh->MaxUnits() && uh->units[targetId]
-					&& uh->units[targetId]->allyteam == ally) {
+			const CUnit* target = uh->GetUnit(c.params[0]);
+
+			if (target && target->allyteam == ally) {
 				todel.push_back(it - commandQue.begin());
 			}
 		}
