@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
@@ -362,6 +364,8 @@ bool CMobileCAI::LandRepairIfNeeded()
 
 void CMobileCAI::SlowUpdate()
 {
+	if(gs->paused) // Commands issued may invoke SlowUpdate when paused
+		return;
 	bool wantToLand = false;
 	if (dynamic_cast<AAirMoveType*>(owner->moveType)) {
 		wantToLand = LandRepairIfNeeded();
@@ -442,19 +446,22 @@ void CMobileCAI::ExecuteSetWantedMaxSpeed(Command &c)
 */
 void CMobileCAI::ExecuteMove(Command &c)
 {
-	float3 pos = float3(c.params[0], c.params[1], c.params[2]);
-	if(pos != goalPos){
-		SetGoal(pos, owner->pos);
+	const float3 cmdPos(c.params[0], c.params[1], c.params[2]);
+
+	if (cmdPos != goalPos) {
+		SetGoal(cmdPos, owner->pos);
 	}
-	if((owner->pos - goalPos).SqLength2D() < cancelDistance ||
-			owner->moveType->progressState == AMoveType::Failed){
+	if ((owner->pos - goalPos).SqLength2D() < cancelDistance ||
+			owner->moveType->progressState == AMoveType::Failed) {
 		FinishCommand();
 	}
 	return;
 }
 
 void CMobileCAI::ExecuteLoadUnits(Command &c) {
-	CTransportUnit* tran = dynamic_cast<CTransportUnit*>(uh->units[(int) c.params[0]]);
+	CUnit* unit = uh->GetUnit(c.params[0]);
+	CTransportUnit* tran = dynamic_cast<CTransportUnit*>(unit);
+
 	if (!tran) {
 		FinishCommand();
 		return;
@@ -474,7 +481,6 @@ void CMobileCAI::ExecuteLoadUnits(Command &c) {
 		return;
 	}
 
-	CUnit* unit = uh->units[(int) c.params[0]];
 	if (!unit) {
 		return;
 	}
@@ -617,15 +623,16 @@ void CMobileCAI::ExecuteGuard(Command &c)
 {
 	assert(owner->unitDef->canGuard);
 	assert(!c.params.empty());
-	if(int(c.params[0]) >= 0 && uh->units[int(c.params[0])] != NULL
-			&& UpdateTargetLostTimer(int(c.params[0]))){
-		CUnit* guarded = uh->units[int(c.params[0])];
-		if(owner->unitDef->canAttack && guarded->lastAttack + 40 < gs->frameNum
+
+	CUnit* guarded = uh->GetUnit(c.params[0]);
+
+	if (guarded != NULL && UpdateTargetLostTimer(guarded->id)) {
+		if (owner->unitDef->canAttack && guarded->lastAttack + 40 < gs->frameNum
 				&& IsValidTarget(guarded->lastAttacker))
 		{
 			StopSlowGuard();
 			Command nc;
-			nc.id=CMD_ATTACK;
+			nc.id = CMD_ATTACK;
 			nc.params.push_back(guarded->lastAttacker->id);
 			nc.options = c.options;
 			commandQue.push_front(nc);
@@ -706,12 +713,10 @@ void CMobileCAI::ExecuteAttack(Command &c)
 		owner->commandShotCount = -1;
 
 		if (c.params.size() == 1) {
-			const unsigned int targetID = (unsigned int) c.params[0];
-			const bool legalTarget      = (targetID < uh->MaxUnits());
-			CUnit* targetUnit           = (legalTarget)? uh->units[targetID]: 0x0;
+			CUnit* targetUnit = uh->GetUnit(c.params[0]);
 
 			// check if we have valid target parameter and that we aren't attacking ourselves
-			if (legalTarget && targetUnit != NULL && targetUnit != owner) {
+			if (targetUnit != NULL && targetUnit != owner) {
 				float3 fix = targetUnit->pos + owner->posErrorVector * 128;
 				float3 diff = float3(fix - owner->pos).Normalize();
 
@@ -727,7 +732,7 @@ void CMobileCAI::ExecuteAttack(Command &c)
 				return;
 			}
 		}
-		else {
+		else if (c.params.size() >= 3) {
 			// user gave force-fire attack command
 			float3 pos(c.params[0], c.params[1], c.params[2]);
 			SetGoal(pos, owner->pos);
@@ -867,7 +872,7 @@ void CMobileCAI::ExecuteAttack(Command &c)
 	}
 
 	// user is attacking ground
-	else {
+	else if (c.params.size() >= 3) {
 		const float3 pos(c.params[0], c.params[1], c.params[2]);
 		const float3 diff = owner->pos - pos;
 
@@ -895,7 +900,7 @@ void CMobileCAI::ExecuteAttack(Command &c)
 				}
 			} else {
 				const bool inAngle = w->TryTargetRotate(pos, c.id == CMD_DGUN);
-				const bool inRange = diff.Length2D() < (w->range - (w->relWeaponPos).Length2D());
+				const bool inRange = diff.SqLength2D() < Square(w->range - (w->relWeaponPos).Length2D());
 
 				if (inAngle || inRange) {
 					StopMove();
@@ -970,59 +975,61 @@ void CMobileCAI::DrawCommands(void)
 	}
 
 	CCommandQueue::iterator ci;
-	for(ci=commandQue.begin();ci!=commandQue.end();++ci){
-		switch(ci->id){
-			case CMD_MOVE:{
-				const float3 endPos(ci->params[0],ci->params[1],ci->params[2]);
+	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
+		switch (ci->id) {
+			case CMD_MOVE: {
+				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
 				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.move);
 				break;
 			}
-			case CMD_PATROL:{
-				const float3 endPos(ci->params[0],ci->params[1],ci->params[2]);
+			case CMD_PATROL: {
+				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
 				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.patrol);
 				break;
 			}
-			case CMD_FIGHT:{
-				if(ci->params.size() != 1){
-					const float3 endPos(ci->params[0],ci->params[1],ci->params[2]);
+			case CMD_FIGHT: {
+				if (ci->params.size() != 1) {
+					const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
 					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.fight);
 					break;
 				}
 			}
 			case CMD_ATTACK:
-			case CMD_DGUN:{
-				if(ci->params.size()==1){
-					const CUnit* unit = uh->units[int(ci->params[0])];
-					if((unit != NULL) && isTrackable(unit)) {
-						const float3 endPos =
-							helper->GetUnitErrorPos(unit, owner->allyteam);
+			case CMD_DGUN: {
+				if (ci->params.size() == 1) {
+					const CUnit* unit = uh->GetUnit(ci->params[0]);
+
+					if ((unit != NULL) && isTrackable(unit)) {
+						const float3 endPos = helper->GetUnitErrorPos(unit, owner->allyteam);
 						lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
 					}
-				} else {
-					const float3 endPos(ci->params[0],ci->params[1],ci->params[2]);
+				}
+				else if (ci->params.size() >= 3) {
+					const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
 					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.attack);
 				}
 				break;
 			}
-			case CMD_GUARD:{
-				const CUnit* unit = uh->units[int(ci->params[0])];
-				if((unit != NULL) && isTrackable(unit)) {
+			case CMD_GUARD: {
+				const CUnit* unit = uh->GetUnit(ci->params[0]);
+
+				if ((unit != NULL) && isTrackable(unit)) {
 					const float3 endPos =
 						helper->GetUnitErrorPos(unit, owner->allyteam);
 					lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.guard);
 				}
 				break;
 			}
-			case CMD_LOAD_ONTO:{
-				const CUnit* unit = uh->units[int(ci->params[0])];
+			case CMD_LOAD_ONTO: {
+				const CUnit* unit = uh->GetUnitUnsafe(ci->params[0]);
 				lineDrawer.DrawLineAndIcon(ci->id, unit->pos, cmdColors.load);
 				break;
 			}
-			case CMD_WAIT:{
+			case CMD_WAIT: {
 				DrawWaitIcon(*ci);
 				break;
 			}
-			case CMD_SELFD:{
+			case CMD_SELFD: {
 				lineDrawer.DrawIconAtLastPos(ci->id);
 				break;
 			}
@@ -1073,8 +1080,8 @@ void CMobileCAI::NonMoving(void)
 
 void CMobileCAI::FinishCommand(void)
 {
-	if(!(commandQue.front().options & INTERNAL_ORDER)){
-		lastUserGoal=owner->pos;
+	if (!(commandQue.front().options & INTERNAL_ORDER)) {
+		lastUserGoal = owner->pos;
 	}
 	StopSlowGuard();
 	CCommandAI::FinishCommand();
