@@ -1,8 +1,7 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef GROUP_H
 #define GROUP_H
-// Group.h: interface for the CGroup class.
-//
-//////////////////////////////////////////////////////////////////////
 
 #include <vector>
 
@@ -31,7 +30,7 @@ public:
 	void RemoveUnit(CUnit* unit); ///< call unit.SetGroup(NULL) instead of calling this directly
 	bool AddUnit(CUnit* unit);    ///< dont call this directly call unit.SetGroup and let that call this
 	const std::vector<CommandDescription>& GetPossibleCommands();
-	int GetDefaultCmd(const CUnit* unit, const CFeature* feature);
+	int GetDefaultCmd(const CUnit* unit, const CFeature* feature) const;
 	void GiveCommand(Command c);
 	void ClearUnits(void);
 
