@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "BaseNetProtocol.h"
 
@@ -5,9 +7,8 @@
 #include "mmgr.h"
 
 #include "Game/PlayerStatistics.h"
-#include "Sim/Misc/Team.h"
+#include "Sim/Misc/TeamStatistics.h"
 #include "Net/RawPacket.h"
-#include "Rendering/InMapDraw.h"
 #include "Net/PackPacket.h"
 #include "Net/ProtocolDef.h"
 #if defined(_MSC_VER)
@@ -81,6 +82,14 @@ PacketType CBaseNetProtocol::SendGameID(const uchar* buf)
 	return PacketType(packet);
 }
 
+PacketType CBaseNetProtocol::SendPathCheckSum(uchar myPlayerNum, boost::uint32_t checksum)
+{
+	PackPacket* packet = new PackPacket(1 + 1 + sizeof(boost::uint32_t), NETMSG_PATH_CHECKSUM);
+	*packet << myPlayerNum;
+	*packet << checksum;
+	return PacketType(packet);
+}
+
 
 PacketType CBaseNetProtocol::SendCommand(uchar myPlayerNum, int id, uchar options, const std::vector<float>& params)
 {
@@ -118,7 +127,7 @@ PacketType CBaseNetProtocol::SendAICommand(uchar myPlayerNum, short unitID, int
 
 PacketType CBaseNetProtocol::SendAIShare(uchar myPlayerNum, uchar sourceTeam, uchar destTeam, float metal, float energy, const std::vector<short>& unitIDs)
 {
-	uint16_t totalNumBytes = (1 + sizeof(uint16_t)) + (3 + (2 * sizeof(float)) + (unitIDs.size() * sizeof(short)));
+	boost::uint16_t totalNumBytes = (1 + sizeof(boost::uint16_t)) + (3 + (2 * sizeof(float)) + (unitIDs.size() * sizeof(short)));
 
 	PackPacket* packet = new PackPacket(totalNumBytes, NETMSG_AISHARE);
 	*packet << totalNumBytes << myPlayerNum << sourceTeam << destTeam << metal << energy << unitIDs;
@@ -146,6 +155,20 @@ PacketType CBaseNetProtocol::SendCPUUsage(float cpuUsage)
 	return PacketType(packet);
 }
 
+PacketType CBaseNetProtocol::SendCustomData(uchar myPlayerNum, uchar dataType, int dataValue)
+{
+	PackPacket* packet = new PackPacket(7, NETMSG_CUSTOM_DATA);
+	*packet << myPlayerNum << dataType << dataValue;
+	return PacketType(packet);
+}
+
+PacketType CBaseNetProtocol::SendSpeedControl(uchar myPlayerNum, int speedCtrl) {
+	return SendCustomData(myPlayerNum, CUSTOM_DATA_SPEEDCONTROL, speedCtrl);
+}
+
+PacketType CBaseNetProtocol::SendLuaDrawTime(uchar myPlayerNum, int mSec) {
+	return SendCustomData(myPlayerNum, CUSTOM_DATA_LUADRAWTIME, mSec);
+}
 
 PacketType CBaseNetProtocol::SendDirectControl(uchar myPlayerNum)
 {
@@ -162,11 +185,11 @@ PacketType CBaseNetProtocol::SendDirectControlUpdate(uchar myPlayerNum, uchar st
 }
 
 
-PacketType CBaseNetProtocol::SendAttemptConnect(const std::string name, const std::string& passwd, const std::string version)
+PacketType CBaseNetProtocol::SendAttemptConnect(const std::string& name, const std::string& passwd, const std::string& version, bool reconnect)
 {
-	boost::uint16_t size = 6 + name.size() + passwd.size() + version.size();
+	boost::uint16_t size = 7 + name.size() + passwd.size() + version.size();
 	PackPacket* packet = new PackPacket(size , NETMSG_ATTEMPTCONNECT);
-	*packet << size << name << passwd << version;
+	*packet << size << name << passwd << version << uchar(reconnect);
 	return PacketType(packet);
 }
 
@@ -204,15 +227,15 @@ PacketType CBaseNetProtocol::SendGameOver()
 }
 
 
-// [NETMSG_MAPDRAW = 31] uchar messageSize = 9, myPlayerNum, command = CInMapDraw::NET_ERASE; short x, z; bool
+// [NETMSG_MAPDRAW = 31] uchar messageSize = 9, myPlayerNum, command = MAPDRAW_ERASE; short x, z; bool
 PacketType CBaseNetProtocol::SendMapErase(uchar myPlayerNum, short x, short z)
 {
 	PackPacket* packet = new PackPacket(8, NETMSG_MAPDRAW);
-	*packet << static_cast<uchar>(8) << myPlayerNum << static_cast<uchar>(CInMapDraw::NET_ERASE) << x << z;
+	*packet << static_cast<uchar>(8) << myPlayerNum << static_cast<uchar>(MAPDRAW_ERASE) << x << z;
 	return PacketType(packet);
 }
 
-// [NETMSG_MAPDRAW = 31] uchar messageSize, uchar myPlayerNum, command = CInMapDraw::NET_POINT; short x, z; bool; std::string label;
+// [NETMSG_MAPDRAW = 31] uchar messageSize, uchar myPlayerNum, command = MAPDRAW_POINT; short x, z; bool; std::string label;
 PacketType CBaseNetProtocol::SendMapDrawPoint(uchar myPlayerNum, short x, short z, const std::string& label, bool fromLua)
 {
 	const unsigned size = 9 + label.size() + 1;
@@ -220,7 +243,7 @@ PacketType CBaseNetProtocol::SendMapDrawPoint(uchar myPlayerNum, short x, short
 	*packet <<
 		static_cast<uchar>(size) <<
 		myPlayerNum <<
-		static_cast<uchar>(CInMapDraw::NET_POINT) <<
+		static_cast<uchar>(MAPDRAW_POINT) <<
 		x <<
 		z <<
 		uchar(fromLua) <<
@@ -228,14 +251,14 @@ PacketType CBaseNetProtocol::SendMapDrawPoint(uchar myPlayerNum, short x, short
 	return PacketType(packet);
 }
 
-// [NETMSG_MAPDRAW = 31] uchar messageSize = 13, myPlayerNum, command = CInMapDraw::NET_LINE; short x1, z1, x2, z2; bool
+// [NETMSG_MAPDRAW = 31] uchar messageSize = 13, myPlayerNum, command = MAPDRAW_LINE; short x1, z1, x2, z2; bool
 PacketType CBaseNetProtocol::SendMapDrawLine(uchar myPlayerNum, short x1, short z1, short x2, short z2, bool fromLua)
 {
 	PackPacket* packet = new PackPacket(13, NETMSG_MAPDRAW);
 	*packet <<
 		static_cast<uchar>(13) <<
 		myPlayerNum <<
-		static_cast<uchar>(CInMapDraw::NET_LINE) <<
+		static_cast<uchar>(MAPDRAW_LINE) <<
 		x1 << z1 <<
 		x2 << z2 <<
 		uchar(fromLua);
@@ -272,8 +295,8 @@ PacketType CBaseNetProtocol::SendStartPos(uchar myPlayerNum, uchar teamNum, ucha
 
 PacketType CBaseNetProtocol::SendPlayerInfo(uchar myPlayerNum, float cpuUsage, int ping)
 {
-	PackPacket* packet = new PackPacket(8, NETMSG_PLAYERINFO);
-	*packet << myPlayerNum << cpuUsage << static_cast<boost::uint16_t>(ping);
+	PackPacket* packet = new PackPacket(10, NETMSG_PLAYERINFO);
+	*packet << myPlayerNum << cpuUsage << static_cast<boost::uint32_t>(ping);
 	return PacketType(packet);
 }
 
@@ -370,6 +393,16 @@ PacketType CBaseNetProtocol::SendUnRegisterNetMsg( uchar myPlayerNum, NETMSG msg
 }
 
 
+PacketType CBaseNetProtocol::SendCreateNewPlayer( uchar playerNum, bool spectator, uchar teamNum, std::string playerName )
+{
+	unsigned size = 1 + sizeof(uchar) + sizeof(uchar) + sizeof(uchar) + sizeof (boost::uint16_t) +playerName.size()+1;
+	PackPacket* packet = new PackPacket( size, NETMSG_CREATE_NEWPLAYER);
+	*packet << static_cast<boost::uint16_t>(size) << playerNum << (uchar)spectator << teamNum << playerName;
+	return PacketType(packet);
+
+}
+
+
 #ifdef SYNCDEBUG
 PacketType CBaseNetProtocol::SendSdCheckrequest(int frameNum)
 {
@@ -399,6 +432,7 @@ PacketType CBaseNetProtocol::SendSdBlockrequest(unsigned short begin, unsigned s
 	
 }
 
+
 PacketType CBaseNetProtocol::SendSdBlockresponse(uchar myPlayerNum, std::vector<unsigned> checksums)
 {
 	unsigned size = 1 + 2 + 1 + checksums.size() * 4;
@@ -431,6 +465,7 @@ CBaseNetProtocol::CBaseNetProtocol()
 	proto->AddType(NETMSG_CHAT, -1);
 	proto->AddType(NETMSG_RANDSEED, 5);
 	proto->AddType(NETMSG_GAMEID, 17);
+	proto->AddType(NETMSG_PATH_CHECKSUM, 1 + 1 + sizeof(boost::uint32_t));
 	proto->AddType(NETMSG_COMMAND, -2);
 	proto->AddType(NETMSG_SELECT, -2);
 	proto->AddType(NETMSG_PAUSE, 3);
@@ -442,6 +477,7 @@ CBaseNetProtocol::CBaseNetProtocol()
 	proto->AddType(NETMSG_USER_SPEED, 6);
 	proto->AddType(NETMSG_INTERNAL_SPEED, 5);
 	proto->AddType(NETMSG_CPU_USAGE, 5);
+	proto->AddType(NETMSG_CUSTOM_DATA, 7);
 	proto->AddType(NETMSG_DIRECT_CONTROL, 2);
 	proto->AddType(NETMSG_DC_UPDATE, 7);
 	proto->AddType(NETMSG_ATTEMPTCONNECT, -2);
@@ -454,18 +490,20 @@ CBaseNetProtocol::CBaseNetProtocol()
 	proto->AddType(NETMSG_SYNCRESPONSE, 9);
 	proto->AddType(NETMSG_SYSTEMMSG, -2);
 	proto->AddType(NETMSG_STARTPOS, 16);
-	proto->AddType(NETMSG_PLAYERINFO, 8);
+	proto->AddType(NETMSG_PLAYERINFO, 10);
 	proto->AddType(NETMSG_PLAYERLEFT, 3);
 	proto->AddType(NETMSG_LUAMSG, -2);
 	proto->AddType(NETMSG_TEAM, 5);
 	proto->AddType(NETMSG_GAMEDATA, -2);
 	proto->AddType(NETMSG_ALLIANCE, 4);
 	proto->AddType(NETMSG_CCOMMAND, -2);
-	proto->AddType(NETMSG_TEAMSTAT, 2 + sizeof(CTeam::Statistics));
+	proto->AddType(NETMSG_TEAMSTAT, 2 + sizeof(TeamStatistics));
 	proto->AddType(NETMSG_REQUEST_TEAMSTAT, 4 );
 	proto->AddType(NETMSG_REGISTER_NETMSG, 3 );
 	proto->AddType(NETMSG_UNREGISTER_NETMSG, 3);
 
+	proto->AddType(NETMSG_CREATE_NEWPLAYER, -2);
+
 	proto->AddType(NETMSG_AI_CREATED, -1);
 	proto->AddType(NETMSG_AI_STATE_CHANGED, 7);
 
