@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef UTIL_H
 #define UTIL_H
 
@@ -47,6 +49,11 @@ std::string StringReplace(const std::string& text,
                           const std::string& from,
                           const std::string& to);
 
+/// Removes leading and trailing whitespace from a string, in place.
+void StringTrimInPlace(std::string& str);
+/// Removes leading and trailing whitespace from a string, in a copy.
+std::string StringTrim(const std::string& str);
+
 
 static inline std::string IntToString(int i, const std::string& format = "%i")
 {
@@ -55,6 +62,13 @@ static inline std::string IntToString(int i, const std::string& format = "%i")
 	return std::string(buf);
 }
 
+static inline std::string FloatToString(float f, const std::string& format = "%f")
+{
+	char buf[64];
+	SNPRINTF(buf, sizeof(buf), format.c_str(), f);
+	return std::string(buf);
+}
+
 /**
  * @brief Safely delete object by first setting pointer to NULL and then deleting.
  * This way it is guaranteed other objects can not access the object through the
