@@ -1,12 +1,8 @@
-/**
- * @file float3.cpp
- * @brief float3 source
- *
- * Implementation of float3 class
- */
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "float3.h"
 #include "Vec2.h"
+#include "creg/creg_cond.h"
 
 
 // TODO: this should go in Vec2.cpp if that is ever created
