@@ -1,12 +1,9 @@
-/**
- * @file float4.cpp
- * @brief float4 source
- *
- * Implementation of float4 class
- */
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "float4.h"
 
+#include "creg/creg_cond.h"
+
 CR_BIND(float4, );
 CR_REG_METADATA(float4, (CR_MEMBER(x), CR_MEMBER(y), CR_MEMBER(z), CR_MEMBER(w)));
 
@@ -16,4 +13,4 @@ float4::float4() : w(0.0f)
 	assert(&y == &x+1);
 	assert(&z == &y+1);
 	assert(&w == &z+1);
-}
\ No newline at end of file
+}
