@@ -32,6 +32,9 @@
 #include "System/Info.h"
 #include "System/Option.h"
 
+#ifdef WIN32
+#include <windows.h>
+#endif
 
 // unitsync only:
 #include "LuaParserAPI.h"
@@ -140,7 +143,7 @@ class ScopedMapLoader {
 			}
 
 			vfsHandler = new CVFSHandler();
-			vfsHandler->AddMapArchiveWithDeps(mapName, false);
+			vfsHandler->AddArchiveWithDeps(mapName, false);
 		}
 
 		~ScopedMapLoader()
@@ -212,8 +215,12 @@ EXPORT(const char*) GetSpringVersion()
 }
 
 
+static void internal_deleteMapInfos();
+
 static void _UnInit()
 {
+	internal_deleteMapInfos();
+
 	lpClose();
 
 	FileSystemHandler::Cleanup();
@@ -447,12 +454,7 @@ EXPORT(void) AddAllArchives(const char* root)
 	try {
 		CheckInit();
 		CheckNullOrEmpty(root);
-
-		vector<string> ars = archiveScanner->GetArchives(root);
-		for (vector<string>::iterator i = ars.begin(); i != ars.end(); ++i) {
-			logOutput.Print(LOG_UNITSYNC, "adding archive: %s\n", i->c_str());
-			vfsHandler->AddArchive(*i, false);
-		}
+		vfsHandler->AddArchiveWithDeps(root, false);
 	}
 	UNITSYNC_CATCH_BLOCKS;
 }
@@ -493,7 +495,7 @@ EXPORT(unsigned int) GetArchiveChecksum(const char* arname)
 		CheckNullOrEmpty(arname);
 
 		logOutput.Print(LOG_UNITSYNC, "archive checksum: %s\n", arname);
-		return archiveScanner->GetArchiveChecksum(arname);
+		return archiveScanner->GetSingleArchiveChecksum(arname);
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return 0;
@@ -518,6 +520,239 @@ EXPORT(const char*) GetArchivePath(const char* arname)
 }
 
 
+static void safe_strzcpy(char* dst, std::string src, size_t max)
+{
+	if (src.length() > max-1) {
+		src = src.substr(0, max-1);
+	}
+	strcpy(dst, src.c_str());
+}
+
+
+/**
+ * @brief map related meta-data
+ */
+struct InternalMapInfo
+{
+	std::string description;  ///< Description (max 255 chars)
+	std::string author;       ///< Creator of the map (max 200 chars)
+	int tidalStrength;        ///< Tidal strength
+	int gravity;              ///< Gravity
+	float maxMetal;           ///< Metal scale factor
+	int extractorRadius;      ///< Extractor radius (of metal extractors)
+	int minWind;              ///< Minimum wind speed
+	int maxWind;              ///< Maximum wind speed
+	int width;                ///< Width of the map
+	int height;               ///< Height of the map
+	std::vector<float> xPos;  ///< Start positions X coordinates defined by the map
+	std::vector<float> zPos;  ///< Start positions Z coordinates defined by the map
+};
+
+static bool internal_GetMapInfo(const char* name, InternalMapInfo* outInfo)
+{
+	CheckInit();
+	CheckNullOrEmpty(name);
+	CheckNull(outInfo);
+
+	logOutput.Print(LOG_UNITSYNC, "get map info: %s", name);
+
+	ScopedMapLoader mapLoader(name);
+	const string mapName = archiveScanner->MapNameToMapFile(name);
+
+	std::string err("");
+
+	MapParser mapParser(mapName);
+	if (!mapParser.IsValid()) {
+		err = mapParser.GetErrorLog();
+	}
+	const LuaTable mapTable = mapParser.GetRoot();
+
+	// Retrieve the map header as well
+	if (err.empty()) {
+		const std::string extension = mapName.substr(mapName.length() - 3);
+		if (extension == "smf") {
+			try {
+				CSmfMapFile file(mapName);
+				const SMFHeader& mh = file.GetHeader();
+
+				outInfo->width  = mh.mapx * SQUARE_SIZE;
+				outInfo->height = mh.mapy * SQUARE_SIZE;
+			}
+			catch (content_error&) {
+				outInfo->width  = -1;
+			}
+		}
+		else {
+			const int w = mapTable.GetInt("gameAreaW", 0);
+			const int h = mapTable.GetInt("gameAreaW", 1);
+
+			outInfo->width  = w * SQUARE_SIZE;
+			outInfo->height = h * SQUARE_SIZE;
+		}
+
+		// Make sure we found stuff in both the smd and the header
+		if (outInfo->width <= 0) {
+			err = "Bad map width";
+		}
+		else if (outInfo->height <= 0) {
+			err = "Bad map height";
+		}
+	}
+
+	// If the map did not parse, say so now
+	if (!err.empty()) {
+		SetLastError(err);
+		outInfo->description = err;
+		return false;
+	}
+
+	outInfo->description = mapTable.GetString("description", "");
+
+	outInfo->tidalStrength   = mapTable.GetInt("tidalstrength", 0);
+	outInfo->gravity         = mapTable.GetInt("gravity", 0);
+	outInfo->extractorRadius = mapTable.GetInt("extractorradius", 0);
+	outInfo->maxMetal        = mapTable.GetFloat("maxmetal", 0.0f);
+
+	outInfo->author = mapTable.GetString("author", "");
+
+	const LuaTable atmoTable = mapTable.SubTable("atmosphere");
+	outInfo->minWind = atmoTable.GetInt("minWind", 0);
+	outInfo->maxWind = atmoTable.GetInt("maxWind", 0);
+
+	// Find as many start positions as there are defined by the map
+	for (size_t curTeam = 0; true; ++curTeam) {
+		float3 pos(-1.0f, -1.0f, -1.0f); // defaults
+		if (!mapParser.GetStartPos(curTeam, pos)) {
+			break; // position could not be parsed
+		}
+		outInfo->xPos.push_back(pos.x);
+		outInfo->zPos.push_back(pos.z);
+		logOutput.Print(LOG_UNITSYNC, "  startpos: %.0f, %.0f", pos.x, pos.z);
+	}
+
+	return true;
+}
+
+/** @deprecated */
+static bool _GetMapInfoEx(const char* name, MapInfo* outInfo, int version)
+{
+	CheckInit();
+	CheckNullOrEmpty(name);
+	CheckNull(outInfo);
+
+	bool fetchOk;
+
+	InternalMapInfo internalMapInfo;
+	fetchOk = internal_GetMapInfo(name, &internalMapInfo);
+
+	if (fetchOk) {
+		safe_strzcpy(outInfo->description, internalMapInfo.description, 255);
+		outInfo->tidalStrength   = internalMapInfo.tidalStrength;
+		outInfo->gravity         = internalMapInfo.gravity;
+		outInfo->maxMetal        = internalMapInfo.maxMetal;
+		outInfo->extractorRadius = internalMapInfo.extractorRadius;
+		outInfo->minWind         = internalMapInfo.minWind;
+		outInfo->maxWind         = internalMapInfo.maxWind;
+
+		outInfo->width           = internalMapInfo.width;
+		outInfo->height          = internalMapInfo.height;
+		outInfo->posCount        = internalMapInfo.xPos.size();
+		if (outInfo->posCount > 16) {
+			// legacy interface does not support more then 16
+			outInfo->posCount = 16;
+		}
+		for (size_t curTeam = 0; curTeam < outInfo->posCount; ++curTeam) {
+			outInfo->positions[curTeam].x = internalMapInfo.xPos[curTeam];
+			outInfo->positions[curTeam].z = internalMapInfo.zPos[curTeam];
+		}
+
+		if (version >= 1) {
+			safe_strzcpy(outInfo->author, internalMapInfo.author, 200);
+		}
+	} else {
+		// contains the error message
+		safe_strzcpy(outInfo->description, internalMapInfo.description, 255);
+ 
+ 		// Fill in stuff so TASClient does not crash
+ 		outInfo->posCount = 0;
+		if (version >= 1) {
+			outInfo->author[0] = '\0';
+		}
+		return false;
+	}
+
+	return fetchOk;
+}
+
+
+/**
+ * @brief Retrieve map info
+ * @param name name of the map, e.g. "SmallDivide.smf"
+ * @param outInfo pointer to structure which is filled with map info
+ * @param version this determines which fields of the MapInfo structure are filled
+ * @return Zero on error; non-zero on success
+ * @deprecated
+ * @see GetMapCount
+ *
+ * If version >= 1, then the author field is filled.
+ *
+ * Important: the description and author fields must point to a valid, and sufficiently long buffer
+ * to store their contents.  Description is max 255 chars, and author is max 200 chars. (including
+ * terminating zero byte).
+ *
+ * If an error occurs (return value 0), the description is set to an error message.
+ * However, using GetNextError() is the recommended way to get the error message.
+ *
+ * Example:
+ *		@code
+ *		char description[255];
+ *		char author[200];
+ *		MapInfo mi;
+ *		mi.description = description;
+ *		mi.author = author;
+ *		if (GetMapInfoEx("somemap.smf", &mi, 1)) {
+ *			// now mi contains map data
+ *		} else {
+ *			// handle the error
+ *		}
+ *		@endcode
+ */
+EXPORT(int) GetMapInfoEx(const char* name, MapInfo* outInfo, int version)
+{
+	int ret = 0;
+
+	try {
+		const bool fetchOk = _GetMapInfoEx(name, outInfo, version);
+		ret = fetchOk ? 1 : 0;
+	}
+	UNITSYNC_CATCH_BLOCKS;
+
+	return ret;
+}
+
+
+/**
+ * @brief Retrieve map info, equivalent to GetMapInfoEx(name, outInfo, 0)
+ * @param name name of the map, e.g. "SmallDivide.smf"
+ * @param outInfo pointer to structure which is filled with map info
+ * @return Zero on error; non-zero on success
+ * @deprecated
+ * @see GetMapCount
+ */
+EXPORT(int) GetMapInfo(const char* name, MapInfo* outInfo)
+{
+	int ret = 0;
+
+	try {
+		const bool fetchOk = _GetMapInfoEx(name, outInfo, 0);
+		ret = fetchOk ? 1 : 0;
+	}
+	UNITSYNC_CATCH_BLOCKS;
+
+	return ret;
+}
+
+
 // Updated on every call to GetMapCount
 static vector<string> mapNames;
 
@@ -583,171 +818,255 @@ EXPORT(const char*) GetMapName(int index)
 }
 
 
-static void safe_strzcpy(char* dst, std::string src, size_t max)
-{
-	if (src.length() > max-1) {
-		src = src.substr(0, max-1);
+static std::map<int, InternalMapInfo> mapInfos;
+
+static InternalMapInfo* internal_getMapInfo(int index) {
+
+	if (index >= mapNames.size()) {
+		SetLastError("invalid map index");
+	} else {
+		if (mapInfos.find(index) == mapInfos.end()) {
+			try {
+				InternalMapInfo imi;
+				if (internal_GetMapInfo(mapNames[index].c_str(), &imi)) {
+					mapInfos[index] = imi;
+					return &(mapInfos[index]);
+				}
+			}
+			UNITSYNC_CATCH_BLOCKS;
+		} else {
+			return &(mapInfos[index]);
+		}
 	}
-	strcpy(dst, src.c_str());
+
+	return NULL;
 }
 
+static void internal_deleteMapInfos() {
 
-static int _GetMapInfoEx(const char* name, MapInfo* outInfo, int version)
-{
-	CheckInit();
-	CheckNullOrEmpty(name);
-	CheckNull(outInfo);
-
-	logOutput.Print(LOG_UNITSYNC, "get map info: %s", name);
-
-	const string mapName = name;
-	ScopedMapLoader mapLoader(mapName);
-
-	string err("");
-
-	MapParser mapParser(mapName);
-	if (!mapParser.IsValid()) {
-		err = mapParser.GetErrorLog();
+	while (!mapInfos.empty()) {
+		std::map<int, InternalMapInfo>::iterator mi = mapInfos.begin();
+		mapInfos.erase(mi);
 	}
-	const LuaTable mapTable = mapParser.GetRoot();
+}
 
-	// Retrieve the map header as well
-	if (err.empty()) {
-		const string extension = mapName.substr(mapName.length() - 3);
-		if (extension == "smf") {
-			try {
-				CSmfMapFile file(name);
-				const SMFHeader& mh = file.GetHeader();
+/**
+ * @brief Get the description of a map
+ * @return NULL on error; the description of the map
+ *         (e.g. "Lot of metal in middle") on success
+ */
+EXPORT(const char*) GetMapDescription(int index) {
 
-				outInfo->width  = mh.mapx * SQUARE_SIZE;
-				outInfo->height = mh.mapy * SQUARE_SIZE;
-			}
-			catch (content_error&) {
-				outInfo->width  = -1;
-			}
-		}
-		else {
-			int w = mapTable.GetInt("gameAreaW", 0);
-			int h = mapTable.GetInt("gameAreaW", 1);
-
-			outInfo->width  = w * SQUARE_SIZE;
-			outInfo->height = h * SQUARE_SIZE;
-		}
-
-		// Make sure we found stuff in both the smd and the header
-		if (outInfo->width <= 0) {
-			err = "Bad map width";
-		}
-		else if (outInfo->height <= 0) {
-			err = "Bad map height";
-		}
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->description.c_str();
 	}
 
-	// If the map didn't parse, say so now
-	if (!err.empty()) {
-		SetLastError(err);
-		safe_strzcpy(outInfo->description, err, 255);
+	return NULL;
+}
 
-		// Fill in stuff so tasclient won't crash
-		outInfo->posCount = 0;
-		if (version >= 1) {
-			outInfo->author[0] = 0;
-		}
-		return 0;
+/**
+ * @brief Get the name of the author of a map
+ * @return NULL on error; the name of the author of a map on success
+ */
+EXPORT(const char*) GetMapAuthor(int index) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->author.c_str();
 	}
 
-	const string desc = mapTable.GetString("description", "");
-	safe_strzcpy(outInfo->description, desc, 255);
+	return NULL;
+}
 
-	outInfo->tidalStrength   = mapTable.GetInt("tidalstrength", 0);
-	outInfo->gravity         = mapTable.GetInt("gravity", 0);
-	outInfo->extractorRadius = mapTable.GetInt("extractorradius", 0);
-	outInfo->maxMetal        = mapTable.GetFloat("maxmetal", 0.0f);
+/**
+ * @brief Get the width of a map
+ * @return -1 on error; the width of a map
+ */
+EXPORT(int) GetMapWidth(int index) {
 
-	if (version >= 1) {
-		const string author = mapTable.GetString("author", "");
-		safe_strzcpy(outInfo->author, author, 200);
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->width;
 	}
 
-	const LuaTable atmoTable = mapTable.SubTable("atmosphere");
-	outInfo->minWind = atmoTable.GetInt("minWind", 0);
-	outInfo->maxWind = atmoTable.GetInt("maxWind", 0);
+	return -1;
+}
 
-	// Find the start positions
-	int curTeam;
-	for (curTeam = 0; curTeam < 16; ++curTeam) {
-		float3 pos(-1.0f, -1.0f, -1.0f); // defaults
-		if (!mapParser.GetStartPos(curTeam, pos)) {
-			break; // position could not be parsed
-		}
-		outInfo->positions[curTeam].x = pos.x;
-		outInfo->positions[curTeam].z = pos.z;
-		logOutput.Print(LOG_UNITSYNC, "  startpos: %.0f, %.0f", pos.x, pos.z);
+/**
+ * @brief Get the height of a map
+ * @return -1 on error; the height of a map
+ */
+EXPORT(int) GetMapHeight(int index) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->height;
 	}
 
-	outInfo->posCount = curTeam;
+	return -1;
+}
 
+/**
+ * @brief Get the tidal speed of a map
+ * @return -1 on error; the tidal speed of the map on success
+ */
+EXPORT(int) GetMapTidalStrength(int index) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->tidalStrength;
+	}
+
+	return -1;
+}
+
+/**
+ * @brief Get the minimum wind speed on a map
+ * @return -1 on error; the minimum wind speed on a map
+ */
+EXPORT(int) GetMapWindMin(int index) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->minWind;
+	}
+
+	return -1;
+}
+
+/**
+ * @brief Get the maximum wind strenght on a map
+ * @return -1 on error; the maximum wind strenght on a map
+ */
+EXPORT(int) GetMapWindMax(int index) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->maxWind;
+	}
+
+	return -1;
+}
+
+/**
+ * @brief Get the gravity of a map
+ * @return -1 on error; the gravity of the map on success
+ */
+EXPORT(int) GetMapGravity(int index) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->gravity;
+	}
+
+	return -1;
+}
+
+/**
+ * @brief Get the number of resources supported available
+ * @return -1 on error; the number of resources supported available on success
+ */
+EXPORT(int) GetMapResourceCount(int index) {
 	return 1;
 }
 
+/**
+ * @brief Get the name of a map resource
+ * @return NULL on error; the name of a map resource (e.g. "Metal") on success
+ */
+EXPORT(const char*) GetMapResourceName(int index, int resourceIndex) {
+
+	if (resourceIndex == 0) {
+		return "Metal";
+	} else {
+		SetLastError("No valid map resource index");
+	}
+
+	return NULL;
+}
 
 /**
- * @brief Retrieve map info
- * @param name name of the map, e.g. "SmallDivide.smf"
- * @param outInfo pointer to structure which is filled with map info
- * @param version this determines which fields of the MapInfo structure are filled
- * @return Zero on error; non-zero on success
- *
- * If version >= 1, then the author field is filled.
- *
- * Important: the description and author fields must point to a valid, and sufficiently long buffer
- * to store their contents.  Description is max 255 chars, and author is max 200 chars. (including
- * terminating zero byte).
- *
- * If an error occurs (return value 0), the description is set to an error message.
- * However, using GetNextError() is the recommended way to get the error message.
- *
- * Example:
- *		@code
- *		char description[255];
- *		char author[200];
- *		MapInfo mi;
- *		mi.description = description;
- *		mi.author = author;
- *		if (GetMapInfoEx("somemap.smf", &mi, 1)) {
- *			//now mi is contains map data
- *		} else {
- *			//handle the error
- *		}
- *		@endcode
+ * @brief Get the scale factor of a resource map
+ * @return 0.0f on error; the scale factor of a resource map on success
  */
-EXPORT(int) GetMapInfoEx(const char* name, MapInfo* outInfo, int version)
-{
-	try {
-		return _GetMapInfoEx(name, outInfo, version);
+EXPORT(float) GetMapResourceMax(int index, int resourceIndex) {
+
+	if (resourceIndex == 0) {
+		const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+		if (mapInfo) {
+			return mapInfo->maxMetal;
+		}
+	} else {
+		SetLastError("No valid map resource index");
 	}
-	UNITSYNC_CATCH_BLOCKS;
-	return 0;
+
+	return 0.0f;
+}
+
+/**
+ * @brief Get the extractor radius for a map resource
+ * @return -1 on error; the extractor radius for a map resource on success
+ */
+EXPORT(int) GetMapResourceExtractorRadius(int index, int resourceIndex) {
+
+	if (resourceIndex == 0) {
+		const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+		if (mapInfo) {
+			return mapInfo->extractorRadius;
+		}
+	} else {
+		SetLastError("No valid map resource index");
+	}
+
+	return -1;
 }
 
 
 /**
- * @brief Retrieve map info, equivalent to GetMapInfoEx(name, outInfo, 0)
- * @param name name of the map, e.g. "SmallDivide.smf"
- * @param outInfo pointer to structure which is filled with map info
- * @return Zero on error; non-zero on success
- * @see GetMapInfoEx
+ * @brief Get the number of defined start positions for a map
+ * @return -1 on error; the number of defined start positions for a map
+ *         on success
  */
-EXPORT(int) GetMapInfo(const char* name, MapInfo* outInfo)
-{
-	try {
-		return _GetMapInfoEx(name, outInfo, 0);
+EXPORT(int) GetMapPosCount(int index) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->xPos.size();
 	}
-	UNITSYNC_CATCH_BLOCKS;
-	return 0;
+
+	return -1;
 }
 
+/**
+ * @brief Get the position on the x-axis for a start position on a map
+ * @return -1.0f on error; the position on the x-axis for a start position
+ *         on a map on success
+ */
+EXPORT(float) GetMapPosX(int index, int posIndex) {
 
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->xPos[posIndex];
+	}
+
+	return -1.0f;
+}
+
+/**
+ * @brief Get the position on the z-axis for a start position on a map
+ * @return -1.0f on error; the position on the z-axis for a start position
+ *         on a map on success
+ */
+EXPORT(float) GetMapPosZ(int index, int posIndex) {
+
+	const InternalMapInfo* mapInfo = internal_getMapInfo(index);
+	if (mapInfo) {
+		return mapInfo->zPos[posIndex];
+	}
+
+	return -1.0f;
+}
 
 /**
  * @brief return the map's minimum height
@@ -829,7 +1148,7 @@ EXPORT(int) GetMapArchiveCount(const char* mapName)
 		CheckInit();
 		CheckNullOrEmpty(mapName);
 
-		mapArchives = archiveScanner->GetArchivesForMap(mapName);
+		mapArchives = archiveScanner->GetArchives(mapName);
 		return mapArchives.size();
 	}
 	UNITSYNC_CATCH_BLOCKS;
@@ -871,7 +1190,7 @@ EXPORT(unsigned int) GetMapChecksum(int index)
 		CheckInit();
 		CheckBounds(index, mapNames.size());
 
-		return archiveScanner->GetMapChecksum(mapNames[index]);
+		return archiveScanner->GetArchiveCompleteChecksum(mapNames[index]);
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return 0;
@@ -889,7 +1208,7 @@ EXPORT(unsigned int) GetMapChecksumFromName(const char* mapName)
 	try {
 		CheckInit();
 
-		return archiveScanner->GetMapChecksum(mapName);
+		return archiveScanner->GetArchiveCompleteChecksum(mapName);
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return 0;
@@ -906,9 +1225,9 @@ EXPORT(unsigned int) GetMapChecksumFromName(const char* mapName)
 #define PACKRGB(r, g, b) (((r<<11)&RM) | ((g << 5)&GM) | (b&BM) )
 
 // Used to return the image
-static char* imgbuf[1024*1024*2];
+static unsigned short imgbuf[1024*1024];
 
-static void* GetMinimapSM3(string mapName, int miplevel)
+static unsigned short* GetMinimapSM3(string mapName, int miplevel)
 {
 	MapParser mapParser(mapName);
 	const string minimapFile = mapParser.GetRoot().GetString("minimap", "");
@@ -927,11 +1246,10 @@ static void* GetMinimapSM3(string mapName, int miplevel)
 	if (1024 >> miplevel != bm.xsize || 1024 >> miplevel != bm.ysize)
 		bm = bm.CreateRescaled (1024 >> miplevel, 1024 >> miplevel);
 
-	unsigned short *dst = (unsigned short*)imgbuf;
-	unsigned char *src = bm.mem;
-	for (int y=0;y<bm.ysize;y++)
-		for (int x=0;x<bm.xsize;x++)
-		{
+	unsigned short* dst = (unsigned short*)imgbuf;
+	unsigned char* src = bm.mem;
+	for (int y=0; y < bm.ysize; y++) {
+		for (int x=0; x < bm.xsize; x++) {
 			*dst = 0;
 
 			*dst |= ((src[0]>>3) << 11) & RM;
@@ -941,19 +1259,19 @@ static void* GetMinimapSM3(string mapName, int miplevel)
 			dst ++;
 			src += 4;
 		}
+	}
 
 	return imgbuf;
 }
 
-static void* GetMinimapSMF(string mapName, int miplevel)
+static unsigned short* GetMinimapSMF(string mapName, int miplevel)
 {
 	CSmfMapFile in(mapName);
 	std::vector<uint8_t> buffer;
 	const int mipsize = in.ReadMinimap(buffer, miplevel);
 
 	// Do stuff
-	void* ret = (void*)imgbuf;
-	unsigned short* colors = (unsigned short*)ret;
+	unsigned short* colors = (unsigned short*)((void*)imgbuf);
 
 	unsigned char* temp = &buffer[0];
 
@@ -1002,7 +1320,8 @@ static void* GetMinimapSMF(string mapName, int miplevel)
 		}
 		temp += 8;
 	}
-	return (void*)ret;
+
+	return colors;
 }
 
 /**
@@ -1019,7 +1338,7 @@ static void* GetMinimapSMF(string mapName, int miplevel)
  * An example usage would be GetMinimap("SmallDivide.smf", 2).
  * This would return a 16 bit packed RGB-565 256x256 (= 1024/2^2) bitmap.
  */
-EXPORT(void*) GetMinimap(const char* filename, int miplevel)
+EXPORT(unsigned short*) GetMinimap(const char* filename, int miplevel)
 {
 	try {
 		CheckInit();
@@ -1028,12 +1347,12 @@ EXPORT(void*) GetMinimap(const char* filename, int miplevel)
 		if (miplevel < 0 || miplevel > 8)
 			throw std::out_of_range("Miplevel must be between 0 and 8 (inclusive) in GetMinimap.");
 
-		const string mapName = filename;
-		ScopedMapLoader mapLoader(mapName);
+		ScopedMapLoader mapLoader(filename);
+		const string mapName = archiveScanner->MapNameToMapFile(filename);
 
 		const string extension = mapName.substr(mapName.length() - 3);
 
-		void* ret = NULL;
+		unsigned short* ret = NULL;
 
 		if (extension == "smf") {
 			ret = GetMinimapSMF(mapName, miplevel);
@@ -1067,7 +1386,7 @@ EXPORT(int) GetInfoMapSize(const char* filename, const char* name, int* width, i
 		CheckNull(height);
 
 		ScopedMapLoader mapLoader(filename);
-		CSmfMapFile file(filename);
+		CSmfMapFile file(archiveScanner->MapNameToMapFile(filename));
 		MapBitmapInfo bmInfo = file.GetInfoMapSize(name);
 
 		*width = bmInfo.width;
@@ -1100,7 +1419,7 @@ EXPORT(int) GetInfoMapSize(const char* filename, const char* name, int* width, i
  * this function to convert from one format to another. Currently only the
  * conversion from 16 bpp to 8 bpp is implemented.
  */
-EXPORT(int) GetInfoMap(const char* filename, const char* name, void* data, int typeHint)
+EXPORT(int) GetInfoMap(const char* filename, const char* name, unsigned char* data, int typeHint)
 {
 	try {
 		CheckInit();
@@ -1110,7 +1429,7 @@ EXPORT(int) GetInfoMap(const char* filename, const char* name, void* data, int t
 
 		string n = name;
 		ScopedMapLoader mapLoader(filename);
-		CSmfMapFile file(filename);
+		CSmfMapFile file(archiveScanner->MapNameToMapFile(filename));
 		int actualType = (n == "height" ? bm_grayscale_16 : bm_grayscale_8);
 
 		if (actualType == typeHint) {
@@ -1130,7 +1449,7 @@ EXPORT(int) GetInfoMap(const char* filename, const char* name, void* data, int t
 
 			const unsigned short* inp = temp;
 			const unsigned short* inp_end = temp + size;
-			unsigned char* outp = (unsigned char*) data;
+			unsigned char* outp = data;
 			for (; inp < inp_end; ++inp, ++outp) {
 				*outp = *inp >> 8;
 			}
@@ -1149,7 +1468,7 @@ EXPORT(int) GetInfoMap(const char* filename, const char* name, void* data, int t
 //////////////////////////
 //////////////////////////
 
-vector<CArchiveScanner::ModData> modData;
+vector<CArchiveScanner::ArchiveData> modData;
 
 
 /**
@@ -1432,7 +1751,7 @@ EXPORT(unsigned int) GetPrimaryModChecksum(int index)
 		CheckInit();
 		CheckBounds(index, modData.size());
 
-		return archiveScanner->GetModChecksum(GetPrimaryModArchive(index));
+		return archiveScanner->GetArchiveCompleteChecksum(GetPrimaryModArchive(index));
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return 0;
@@ -1450,7 +1769,7 @@ EXPORT(unsigned int) GetPrimaryModChecksumFromName(const char* name)
 	try {
 		CheckInit();
 
-		return archiveScanner->GetModChecksum(archiveScanner->ModNameToModArchive(name));
+		return archiveScanner->GetArchiveCompleteChecksum(archiveScanner->ArchiveFromName(name));
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return 0;
@@ -2281,18 +2600,11 @@ static void LuaPushNamedNumber(lua_State* L, const string& key, float value)
 
 static int LuaGetMapInfo(lua_State* L)
 {
-	const string mapName = luaL_checkstring(L, 1);
+	const std::string mapName = luaL_checkstring(L, 1);
 
-	MapInfo mi;
-	char auth[256];
-	char desc[256];
-	mi.author = auth;
- 	mi.author[0] = 0;
-	mi.description = desc;
-	mi.description[0] = 0;
-
-	if (!GetMapInfoEx(mapName.c_str(), &mi, 1)) {
-		logOutput.Print(LOG_UNITSYNC, "LuaGetMapInfo: _GetMapInfoEx(\"%s\") failed", mapName.c_str());
+	InternalMapInfo mi;
+	if (!internal_GetMapInfo(mapName.c_str(), &mi)) {
+		logOutput.Print(LOG_UNITSYNC, "LuaGetMapInfo: internal_GetMapInfo(\"%s\") failed", mapName.c_str());
 		return 0;
 	}
 
@@ -2312,11 +2624,11 @@ static int LuaGetMapInfo(lua_State* L)
 
 	lua_pushstring(L, "startPos");
 	lua_newtable(L);
-	for (int i = 0; i < mi.posCount; i++) {
-		lua_pushnumber(L, i + 1);
+	for (size_t p = 0; p < mi.xPos.size(); p++) {
+		lua_pushnumber(L, p + 1);
 		lua_newtable(L);
-		LuaPushNamedNumber(L, "x", mi.positions[i].x);
-		LuaPushNamedNumber(L, "z", mi.positions[i].z);
+		LuaPushNamedNumber(L, "x", mi.xPos[p]);
+		LuaPushNamedNumber(L, "z", mi.zPos[p]);
 		lua_rawset(L, -3);
 	}
 	lua_rawset(L, -3);
@@ -2462,7 +2774,7 @@ EXPORT(void) CloseFileVFS(int handle)
  * @return -1 on error; the number of bytes read on success
  * (if this is less than length you reached the end of the file.)
  */
-EXPORT(int) ReadFileVFS(int handle, void* buf, int length)
+EXPORT(int) ReadFileVFS(int handle, unsigned char* buf, int length)
 {
 	try {
 		CheckFileHandle(handle);
@@ -2671,7 +2983,6 @@ EXPORT(void) CloseArchive(int archive)
 	UNITSYNC_CATCH_BLOCKS;
 }
 
-
 EXPORT(int) FindFilesArchive(int archive, int cur, char* nameBuf, int* size)
 {
 	try {
@@ -2683,13 +2994,16 @@ EXPORT(int) FindFilesArchive(int archive, int cur, char* nameBuf, int* size)
 
 		logOutput.Print(LOG_UNITSYNC, "findfilesarchive: %d\n", archive);
 
-		string name;
-		int s;
-
-		int ret = a->FindFiles(cur, &name, &s);
-		strcpy(nameBuf, name.c_str()); // FIXME: oops, buffer overflow
-		*size = s;
-		return ret;
+		if (cur < a->NumFiles())
+		{
+			string name;
+			int s;
+			a->FileInfo(cur, name, s);
+			strcpy(nameBuf, name.c_str()); // FIXME: oops, buffer overflow
+			*size = s;
+			return ++cur;
+		}
+		return 0;
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return 0;
@@ -2712,7 +3026,7 @@ EXPORT(int) OpenArchiveFile(int archive, const char* name)
 		CheckNullOrEmpty(name);
 
 		CArchiveBase* a = openArchives[archive];
-		return a->OpenFile(name);
+		return a->FindFile(name);
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return 0;
@@ -2728,7 +3042,7 @@ EXPORT(int) OpenArchiveFile(int archive, const char* name)
  * @return -1 on error; the number of bytes read on success
  * (if this is less than numBytes you reached the end of the file.)
  */
-EXPORT(int) ReadArchiveFile(int archive, int handle, void* buffer, int numBytes)
+EXPORT(int) ReadArchiveFile(int archive, int handle, unsigned char* buffer, int numBytes)
 {
 	try {
 		CheckArchiveHandle(archive);
@@ -2736,7 +3050,11 @@ EXPORT(int) ReadArchiveFile(int archive, int handle, void* buffer, int numBytes)
 		CheckPositive(numBytes);
 
 		CArchiveBase* a = openArchives[archive];
-		return a->ReadFile(handle, buffer, numBytes);
+		std::vector<uint8_t> buf;
+		if (!a->GetFile(handle, buf))
+			return -1;
+		std::memcpy(buffer, &buf[0], std::min(buf.size(), (size_t)numBytes));
+		return std::min(buf.size(), (size_t)numBytes);
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return -1;
@@ -2751,10 +3069,7 @@ EXPORT(int) ReadArchiveFile(int archive, int handle, void* buffer, int numBytes)
 EXPORT(void) CloseArchiveFile(int archive, int handle)
 {
 	try {
-		CheckArchiveHandle(archive);
-
-		CArchiveBase* a = openArchives[archive];
-		a->CloseFile(handle);
+		// nuting
 	}
 	UNITSYNC_CATCH_BLOCKS;
 }
@@ -2772,7 +3087,10 @@ EXPORT(int) SizeArchiveFile(int archive, int handle)
 		CheckArchiveHandle(archive);
 
 		CArchiveBase* a = openArchives[archive];
-		return a->FileSize(handle);
+		string name;
+		int s;
+		a->FileInfo(handle, name, s);
+		return s;
 	}
 	UNITSYNC_CATCH_BLOCKS;
 	return -1;
@@ -2789,7 +3107,7 @@ const char *GetStr(string str)
 	//static char strBuf[STRBUF_SIZE];
 
 	if (str.length() + 1 > STRBUF_SIZE) {
-		sprintf(strBuf, "Increase STRBUF_SIZE (needs %d bytes)", str.length() + 1);
+		sprintf(strBuf, "Increase STRBUF_SIZE (needs "_STPF_" bytes)", str.length() + 1);
 	}
 	else {
 		strcpy(strBuf, str.c_str());
